(* Diego Olivier Fernandez Pons
 * Universit Pierre et Marie Curie, Paris VI
 * diego-olivier.fernandez-pons@cicrp.jussieu.fr
 * 
 * Graphs represented by range trees
 * Graphes reprsents par des arbres de recherche par plage
 * Grafos representados por rboles de busqueda por intervallos
 *)

open BairePervasives

let moduleName = "AbRangeGraph"

(* This data structure is equivalent to
 * type ('a, 'b) graph = (node, (node, 'b) map) map
 * type ('a, 'b) graph = (node, 'a) map
 *)

(* Cette structure de donne est quivalente 
 * type ('a, 'b) graph = (node, (node, 'b) map) map
 * type ('a, 'b) graph = (node, 'a) map
 *)

(* Esta estructura de datos es equivalente a
 * type ('a, 'b) graph = (node, (node, 'b) map) map
 * type ('a, 'b) graph = (node, 'a) map
 *)

type node = int
type 'b edge = node * 'b * node

type 'b two = 
  | ETwo 
  | NTwo of 'b two * node * 'b * 'b two * int

type ('a, 'b) graph = 
  | E 
  | N of ('a, 'b) graph * node * 'a * ('a, 'b) graph * 'b two * int


(** Utils **)
(** Utilitaires **)
(** tiles **)


let invalid = fun function_name error_type -> 
  invalid_argument moduleName function_name error_type

let makeTree = fun l k v r s -> 
  let h = match (l, r) with
    | (E, E) -> 1
    | (E, N (_, _, _, _, _, n)) | (N (_, _, _, _, _, n), E) -> 1 + n
    | (N (_, _, _, _, _, m), N (_, _, _, _, _, n)) -> 1 + max m n
  in
    N (l, k, v, r, s, h)

let rotate_L = fun l k v r s ->
  match r with
    | N (rl, rk, rv, rr, rs, _) -> makeTree (makeTree l k v rl s) rk rv rr rs
    | _ -> makeTree l k v r s
	
let rotate_R = fun l k v r s ->
  match l with
    | N (ll, lk, lv, lr, ls, _) -> makeTree ll lk lv (makeTree lr k v r s) ls
    | _ -> makeTree l k v r s

let rotate_RL = fun l k v r s ->
  match r with
    | N (N (rll, rlk, rlv, rlr, rls, _), rk, rv, rr, rs, _) -> 
	makeTree (makeTree l k v rll s) rlk rlv (makeTree rlr rk rv rr rs) rls
    | _ -> makeTree l k v r s

let rotate_LR = fun l k v r s ->
  match l with
    | N (ll, lk, lv, N (lrl, lrk, lrv, lrr, lrs, _), ls, _) ->
	makeTree (makeTree ll lk lv lrl ls) lrk lrv (makeTree lrr k v r s) lrs
    | _ -> makeTree l k v r s

let height = function
  | E -> 0
  | N (_, _, _, _, _, n) -> n

let makeBTree = fun l k v r s ->
  let hl = height l and hr = height r in
    if hl > hr + 1 then
      (match l with
	 | E -> invalid "makeBTree" UnexpectedPattern
         | N (ll, _, _, lr, _, _) ->
             if height ll >= height lr then rotate_R l k v r s
	     else rotate_LR l k v r s
      )
    else if hr > hl + 1 then
      (match r with
	 | E -> invalid "makeBTree" UnexpectedPattern
         | N (rl, _, _, rr, _, _) ->
             if height rr >= height rl then rotate_L l k v r s
	     else rotate_RL l k v r s
      )
    else makeTree l k v r s
      
let rec makeSTree = fun l k v r s ->
  let hl = height l and hr = height r in
    if hl > hr + 2 then
      (match l with
	 | E -> invalid "makeSTree" UnexpectedPattern
	 | N (ll, lk, lv, lr, ls, _) -> 
	     makeBTree ll lk lv (makeSTree lr k v r s) ls
      )
    else if hr > hl + 2 then
      (match r with
	 | E -> invalid "makeSTree" UnexpectedPattern
	 | N (rl, rk, rv, rr, rs, _) -> 
	     makeBTree (makeSTree l k v rl s) rk rv rr rs
      )
    else makeBTree l k v r s

let rec append = fun a b ->
  match (a, b) with
    | (E, _) -> b
    | (_, E) -> a
    | (N (l1, k1, v1, r1, s1, _), N (l2, k2, v2, r2, s2, _)) ->
        makeBTree l1 k1 v1 (makeBTree (append r1 l2) k2 v2 r2 s2) s1

let rec appendS = fun a b ->
  match (a, b) with
    | (E, _) -> b
    | (_, E) -> a
    | (N (l1, k1, v1, r1, s1, _), N (l2, k2, v2, r2, s2, _)) ->
        makeSTree l1 k1 v1 (makeSTree (appendS r1 l2) k2 v2 r2 s2) s1

let makeTwo = fun l k v r -> 
  let h = match (l, r) with
    | (ETwo, ETwo) -> 1
    | (ETwo, NTwo (_, _, _, _, n)) | (NTwo (_, _, _, _, n), ETwo) -> 1 + n
    | (NTwo (_, _, _, _, m), NTwo (_, _, _, _, n)) -> 1 + max m n
  in
    NTwo (l, k, v, r, h)

let rotate_L_two = fun l k v r ->
  match r with
    | NTwo (rl, rk, rv, rr, _) -> makeTwo (makeTwo l k v rl) rk rv rr
    | _ -> makeTwo l k v r
	
let rotate_R_two = fun l k v r ->
  match l with
    | NTwo (ll, lk, lv, lr, _) -> makeTwo ll lk lv (makeTwo lr k v r)
    | _ -> makeTwo l k v r

let rotate_RL_two = fun l k v r ->
  match r with
    | NTwo (NTwo (rll, rlk, rlv, rlr, _), rk, rv, rr, _) -> 
	makeTwo (makeTwo l k v rll) rlk rlv (makeTwo rlr rk rv rr)
    | _ -> makeTwo l k v r

let rotate_LR_two = fun l k v r ->
  match l with
    | NTwo (ll, lk, lv, NTwo (lrl, lrk, lrv, lrr, _), _) ->
	makeTwo (makeTwo ll lk lv lrl) lrk lrv (makeTwo lrr k v r)
    | _ -> makeTwo l k v r

let height_two = function
  | ETwo -> 0
  | NTwo (_, _, _, _, n) -> n

let makeBTwo = fun l k v r ->
  let hl = height_two l and hr = height_two r in
    if hl > hr + 1 then
      (match l with
	 | ETwo -> invalid "makeBTwo" UnexpectedPattern
         | NTwo (ll, _, _, lr, _) ->
             if height_two ll >= height_two lr then rotate_R_two l k v r 
	     else rotate_LR_two l k v r
      )
    else if hr > hl + 1 then
      (match r with
	 | ETwo -> invalid "makeBTwo" UnexpectedPattern
         | NTwo (rl, _, _, rr, _) ->
             if height_two rr >= height_two rl then rotate_L_two l k v r 
	     else rotate_RL_two l k v r
      )
    else makeTwo l k v r
      
let rec makeSTwo = fun l k v r ->
  let hl = height_two l and hr = height_two r in
    if hl > hr + 2 then
      (match l with
	 | ETwo -> invalid "makeSTwo" UnexpectedPattern
	 | NTwo (ll, lk, lv, lr, _) -> makeBTwo ll lk lv (makeSTwo lr k v r)
      )
    else if hr > hl + 2 then
      (match r with
	 | ETwo -> invalid "makeSTwo" UnexpectedPattern
	 | NTwo (rl, rk, rv, rr, _) -> makeBTwo (makeSTwo l k v rl) rk rv rr
      )
    else makeBTwo l k v r

let rec append_two = fun a b ->
  match (a, b) with
    | (ETwo, _) -> b
    | (_, ETwo) -> a
    | (NTwo (l1, k1, v1, r1, _), NTwo (l2, k2, v2, r2, _)) ->
        makeBTwo l1 k1 v1 (makeBTwo (append_two r1 l2) k2 v2 r2)

let rec appendS_two = fun a b ->
  match (a, b) with
    | (ETwo, _) -> b
    | (_, ETwo) -> a
    | (NTwo (l1, k1, v1, r1, _), NTwo (l2, k2, v2, r2, _)) ->
        makeSTwo l1 k1 v1 (makeSTwo (appendS_two r1 l2) k2 v2 r2)

let rec map_tree f = function
  | E -> E
  | N (l, k, v, r, s, _) -> makeTree (map_tree f l) k v (map_tree f r) (f s)

let rec map_tree_two f = function
  | ETwo -> ETwo
  | NTwo (l, k, v, r, h) -> 
      NTwo (map_tree_two f l, k, f v, map_tree_two f r, h)


(** Categorial operations **)
(** Oprations catgorielles **)
(** Operaciones categoriales **)


let injection = fun a x v b y -> 
  if x < y then makeTree E x a (makeTree E y b E ETwo) (makeTwo ETwo y v ETwo)
  else makeTree E y a (makeTree E x b E ETwo) (makeTwo ETwo y v ETwo)

let zero = function () -> E

let rec to_list_two x result = function
  | ETwo -> result
  | NTwo (l, y, v, r, _) -> 
      to_list_two x ((x, v, y) :: to_list_two x result r) l

let rec to_list_one result = function
  | E -> result
  | N (l, k, _, r, s, _) -> 
      to_list_one (to_list_two k (to_list_one result r) s) l 

let rec lookup_one k' = function
  | E -> raise Not_found
  | N (l, k, v, r, _, _) ->
      match compare k k' with
	| n when n < 0 -> lookup_one k' l
	| n when n > 0 -> lookup_one k' r
	| _ -> v

let rec insert_value graph = function
  | [] -> []
  | (i, v, j) :: tail ->
      ((i, lookup_one i graph), v, (j, lookup_one j graph)) ::
      insert_value graph tail

let to_list = function graph -> insert_value graph (to_list_one [] graph)

let rec insert_two y v = function
  | ETwo -> makeTwo ETwo y v ETwo
  | NTwo (l, k', v', r, _) ->
      match compare y k' with
	| n when n < 0 -> makeBTwo (insert_two y v l) k' v' r
	| n when n > 0 -> makeBTwo l k' v' (insert_two y v r)
	| _ -> 
	    match compare v v' with
	      | 0 -> raise Found
	      | _ -> makeBTwo l y v r

let rec insert_raise i a j = function 
  | E -> raise Not_found
  | N (l, k, v, r, s, _) ->
      match compare i k with
	 | n when n < 0 -> makeTree (insert_raise i a j l) k v r s
	 | n when n > 0 -> makeTree l k v (insert_raise i a j r) s
	 | _ -> makeTree l k v r (insert_two j a s)

let insert = fun x v y tree ->
  try 
    insert_raise x v y tree 
  with Found -> tree

let rec insert_node_raise k' v' = function
  | E -> makeTree E k' v' E ETwo
  | N (l, k, v, r, s, _) ->
      match compare k' k with
	| n when n < 0 -> makeBTree (insert_node_raise k' v' l) k v r s
	| n when n > 0 -> makeBTree l k v (insert_node_raise k' v' r) s
	| _ ->
	    match compare v v' with
	      | 0 -> raise Found
	      | _ -> makeBTree l k' v' r s

let insert_node = fun k v graph ->
  try
    insert_node_raise k v graph
  with Found -> graph

let rec from_list_rec result = function
  | [] -> result
  | ((i, a), v, (j, b)) :: tail -> 
      from_list_rec (
	insert i v j (
	  insert_node i a (
	    insert_node j b result
	  )
	)
      ) tail

let from_list = function list -> from_list_rec E list

let map f = function tree -> from_list (List.map f (to_list tree)) 

let rec partition_one pivot = function
  | E -> (E, None, E)
  | N (l, k, v, r, s, _) as node ->
      match compare pivot k with
	| n when n < 0 ->
	    let (inf, value, sup) = partition_one pivot l in
	      (inf, value, makeSTree sup k v r s)
	| n when n > 0 ->
	    let (inf, value, sup) = partition_one pivot r in
	      (makeSTree l k v inf s, value, sup)
	| _ -> (l, Some (k, v, s), r)

let rec partition_two pivot = function
  | ETwo -> (ETwo, None, ETwo)
  | NTwo (l, y, v, r, _) as node ->
      match compare pivot y with
	| n when n < 0 ->
	    let (inf, value, sup) = partition_two pivot l in
	      (inf, value, makeSTwo sup y v r)
	| n when n > 0 ->
	    let (inf, value, sup) = partition_two pivot r in
	      (makeSTwo l y v inf, value, sup)
	| _ -> (l, Some (y, v), r)

let rec plus_two = fun a b -> 
  match (a, b) with
    | (ETwo, _) -> b
    | (_, ETwo) -> a
    | (NTwo (l, x, v, r, _), _) -> let (c, _, d) = partition_two x b in 
	makeSTwo (plus_two l c) x v (plus_two r d)

let rec plus = fun a b ->
  match (a, b) with
    | (E, _) -> b
    | (_, E) -> a
    | (N (l, k, v, r, s, _), N _) ->
	match partition_one k b with
	  | (left, None, right) -> makeTree (plus l left) k v (plus r right) s
	  | (left, Some (_, _, s'), right) -> 
	      makeSTree (plus l left) k v (plus r right) (plus_two s s')


(** Binary operations **)
(** Oprations binaires **)
(** Operaciones binrias **)


let union = plus

let rec intersection_two = fun a b ->
  match (a, b) with
    | (ETwo, _) | (_, ETwo) -> ETwo
    | (NTwo (l, k, v, r, _), NTwo _) ->
	match partition_two k b with
	  | (l', None, r') -> 
	      appendS_two (intersection_two l l') (intersection_two r r')
	  | (l', Some _, r') -> 
	      makeSTwo (intersection_two l l') k v (intersection_two r r')

let rec intersection = fun a b ->
  match (a, b) with
    | (E, _) | (_, E) -> E
    | (N (l, k, v, r, s, _), N _) ->
	match partition_one k b with
	  | (l', None, r') -> appendS (intersection l l') (intersection r r')
	  | (l', Some (_, _, s'), r') -> 
	      makeSTree (intersection l l') k v (intersection r r') 
	      (intersection_two s s')

let rec difference_two = fun a b ->
  match (a, b) with
    | (ETwo, _) -> ETwo
    | (_, ETwo) -> a
    | (NTwo (l, k, v, r, _), NTwo _) -> 
	match partition_two k b with
	  | (l', None, r') -> 
	      makeSTwo (difference_two l l') k v (difference_two r r')
	  | (l', Some _, r') -> 
	      appendS_two (difference_two l l') (difference_two r r')
	      
let rec difference = fun a b ->  
  match (a, b) with
    | (E, _) -> E
    | (_, E) -> a
    | (N (l, k, v, r, s, _) , N _) ->
	match partition_one k b with
	  | (l', None, r') -> 
	      makeSTree (difference l l') k v (difference r r') s
	  | (l', Some (_, _, s'), r') -> 
	      makeSTree (difference l l') k v (difference r r') 
	      (difference_two s s')

let rec symetric_difference_two = fun a b ->
  match (a, b) with
    | (ETwo, s) | (s, ETwo) -> s
    | (NTwo (l, k, v, r, _), NTwo _) ->
	match partition_two k b with
	  | (l', None, r') -> makeSTwo (symetric_difference_two l l') k v 
	      (symetric_difference_two r r')
	  | (l', Some _, r') -> appendS_two (symetric_difference_two l l') 
	      (symetric_difference_two r r')

let rec symetric_difference = fun a b ->  
  match (a, b) with
    | (E, s) | (s, E) -> s
    | (N (l, k, v, r, s, _) , N _) ->
	match partition_one k b with
	  | (l', None, r') -> makeSTree (symetric_difference l l') k v
	      (symetric_difference r r') s
	  | (l', Some (_, _, s'), r') ->
	      makeSTree (symetric_difference l l') k v 
	      (symetric_difference r r') (symetric_difference_two s s')

let rec subset_two = fun a b ->
  match (a, b) with
    | (ETwo, _) -> true
    | (_, ETwo) -> false
    | (NTwo (l1, k1, v1, r1, _), NTwo (l2, k2, v2, r2, _)) ->
        match compare k1 k2 with
	  | n when n < 0 -> 
	      subset_two (makeTwo l1 k1 v1 ETwo) l2 && subset_two r1 b
	  | n when n > 0 -> 
	      subset_two (makeTwo ETwo k1 v1 r1) r2 && subset_two l1 b
	  | _ -> v1 = v2 && subset_two l1 l2 && subset_two r1 r2

let rec subset = fun a b ->
  match (a, b) with
    | (E, _) -> true
    | (_, E) -> false
    | (N (l1, k1, v1, r1, s1, _), N (l2, k2, v2, r2, s2, _)) ->
        match compare k1 k2 with
	  | n when n < 0 -> subset (makeTree l1 k1 v1 E s1) l2 && subset r1 b
	  | n when n > 0 -> subset (makeTree E k1 v1 r1 s1) r2 && subset l1 b
	  | _ -> v1 = v2 && subset_two s1 s2 && subset l1 l2 && subset r1 r2

let rec compare_two = fun a b ->
  match (a, b) with
    | ([], []) -> 0
    | ([], _) -> -1
    | (_, []) -> 1
    | (ETwo :: t1, ETwo :: t2) -> compare_two t1 t2
    | (NTwo (ETwo, k1, v1, r1, _) :: t1, NTwo (ETwo, k2, v2, r2, _) :: t2) ->
	(match compare k1 k2 with
	  | n when n <> 0 -> n
	  | _ -> 
	      match compare v1 v2 with
		| 0 ->compare_two (r1 :: t1) (r2 :: t2)
		| other -> other
	)
    | (NTwo (a, k1, v1, r1, _) :: t1, t2) ->
        compare_two (a :: NTwo (ETwo, k1, v1, r1, 0) :: t1) t2
    | (t1, NTwo (b, k2, v2, r2, _) :: t2) ->
	compare_two t1 (b :: NTwo (ETwo, k2, v2, r2, 0) :: t2)

let rec compare_rec = fun a b ->
  match (a, b) with
    | ([], []) -> 0
    | ([], _) -> -1
    | (_, []) -> 1
    | (E :: t1, E :: t2) -> compare_rec t1 t2
    | (N (E, k1, v1, r1, s1, _) :: t1, N (E, k2, v2, r2, s2, _) :: t2) ->
	(match compare k1 k2 with
	   | n when n <> 0 -> n
	   | _ -> match compare_two [s1] [s2] with
	       | 0 -> compare_rec (r1 :: t1) (r2 :: t2)
	       | other -> other
	)
    | (N (a, k1, v1, r1, s1, _) :: t1, t2) ->
        compare_rec (a :: N (E, k1, v1, r1, s1, 0) :: t1) t2
    | (t1, N (b, k2, v2, r2, s2, _) :: t2) ->
	compare_rec t1 (b :: N (E, k2, v2, r2, s2, 0) :: t2)

let equal = fun a b -> compare_rec [a] [b] = 0


(** Observateurs **)
(** Observers **)
(** Observadores **)


let is_empty = function
  | E -> true
  | N _ -> false

let rec nodes_rec total = function
  | E -> total
  | N (l, _, _, r, _, _) -> nodes_rec (nodes_rec (1 + total) l) r

let nodes = function graph -> nodes_rec 0 graph

let rec size_two total = function
  | ETwo -> total
  | NTwo (l, _, _, r, _) -> size_two (size_two (1 + total) l) r

let rec size_one total = function
  | E -> total
  | N (l, _, _, r, s, _) -> size_one (size_one (size_two total s) l) r

let edges = function graph -> size_one 0 graph

let rec labels_two result = function
  | ETwo -> result
  | NTwo (l, _, v, r, _) -> labels_two (labels_two (v :: result) l) r

let rec labels_one result = function
  | E -> result
  | N (l, _, _, r, s, _) -> labels_one (labels_one (labels_two result s) l) r

let edge_labels_list = function graph -> labels_one [] graph

let rec node_labels_rec result = function
  | E -> result
  | N (l, k, v, r, _, _) -> node_labels_rec (node_labels_rec (v :: result) l) r

let node_labels_list = function graph -> node_labels_rec [] graph

let rec nodes_one result = function
  | E -> result
  | N (l, k, v, r, _, _) -> nodes_one (nodes_one ((k, v) :: result) l) r

let rec nodes_two result = function
  | ETwo -> result
  | NTwo (l, k, _, r, _) -> nodes_two (nodes_two (k :: result) l) r

let nodes_list = function graph -> nodes_one [] graph

let rec edges_rec result = function
  | E -> result
  | N (l, k, _, r, s, _) ->
      edges_rec (
	edges_rec (
	  to_list_two k result s
	) l
      ) r

let edges_list = function graph -> edges_rec [] graph

let rec member_two y = function
  | ETwo -> false
  | NTwo (l, k, _, r, _) ->
      match compare y k with
	| n when n < 0 -> member_two y l
	| n when n > 0 -> member_two y r
	| _ -> true

let rec interior_degree_rec total x = function
  | E -> total
  | N (l, _, _, r, s, _) ->
      interior_degree_rec (
	interior_degree_rec (
	  if member_two x s then (1 + total) else total
	) x l
      ) x r
			     
let interior_degree = fun id graph -> interior_degree_rec 0 id graph

let degrees_list = function graph ->
  let rec degrees_rec result = function
    | E -> raise Not_found
    | N (l, k, v, r, s, _) ->
	degrees_rec (
	  degrees_rec (
	    (k, interior_degree_rec 0 k graph, size_two 0 s) :: result
	  ) l
	) r
  in
    degrees_rec [] graph
  
let rec labels_two result = function
  | ETwo -> result
  | NTwo (l, _, v, r, _) ->
      labels_two (labels_two (UnbalancedSet.insert v result) l) r

let rec labels_one result = function
  | E -> result
  | N (l, _, _, r, s, _) -> labels_one (labels_one (labels_two result s) l) r

let labels_tree = function graph -> labels_one Leaf graph

let rec nodes_tree = function
  | E -> Leaf
  | N (l, k, v, r, _, _) -> Node (nodes_tree l, (k, v), nodes_tree r)

let rec edges_tree_two x result = function
  | ETwo -> result
  | NTwo (l, y, v, r, _) ->
      edges_tree_two x (
	edges_tree_two x (
	  UnbalancedSet.insert (x, y, v) result
	) l
      ) r

let rec edges_tree_one result = function
  | E -> result
  | N (l, k, v, r, s, _) ->
      edges_tree_one (
	edges_tree_one (
	  edges_tree_two k result s
	) l
      ) r
  
let edges_tree = function graph -> edges_tree_one Leaf graph

let degrees_tree = function graph ->
  let rec degrees_rec result = function
    | E -> raise Not_found
    | N (l, k, v, r, s, _) ->
	degrees_rec (
	  degrees_rec (
	    UnbalancedSet.insert 
			 (k, interior_degree_rec 0 k graph, size_two 0 s)
			 result
	  ) l
	) r
  in
    degrees_rec Leaf graph


(** Miscellaneous **)
(** Divers **)
(** Micelaneos **)


let rec union_two_with f = fun a b -> 
  match (a, b) with
    | (ETwo, _) -> b
    | (_, ETwo) -> a
    | (NTwo (l, k, v, r, _), _) -> 
	match partition_two k b with
	  | (l', None, r') -> 
	      makeSTwo (union_two_with f l l') k v (union_two_with f r r')
	  | (l', Some (_, v'), r') ->
	      makeSTwo (union_two_with f l l') k (f v v') 
	      (union_two_with f r r')

let rec union_with f = fun a b ->
  match (a, b) with
    | (E, _) -> b
    | (_, E) -> a
    | (N (l, k, v, r, s, _), N _) ->
	match partition_one k b with
	  | (left, None, right) -> 
	      makeTree (union_with f l left) k v (union_with f r right) s
	  | (left, Some (_, _, s'), right) -> 
	      makeSTree (union_with f l left) k v
	      (union_with f r right) (union_two_with f s s')

let rec intersection_with_two f = fun a b ->
  match (a, b) with
    | (ETwo, _) | (_, ETwo) -> ETwo
    | (NTwo (l, k, v, r, _), NTwo _) ->
	match partition_two k b with
	  | (l', None, r') -> 
	      appendS_two (intersection_with_two f l l') 
	      (intersection_with_two f r r')
	  | (l', Some (_, v'), r') -> 
	      makeSTwo (intersection_with_two f l l') k (f v v') 
	      (intersection_with_two f r r')

let rec intersection_with f = fun a b ->
  match (a, b) with
    | (E, _) | (_, E) -> E
    | (N (l, k, v, r, s, _), N _) ->
	match partition_one k b with
	  | (l', None, r') -> 
	      appendS (intersection_with f l l') (intersection_with f r r')
	  | (l', Some (_, _, s'), r') -> 
	      makeSTree (intersection_with f l l') k v
	      (intersection_with f r r') (intersection_with_two f s s')


(** Operations on node **)
(** Oprations sur les noeuds **)
(** Operaciones sobre los nudos **)


let rec member_node x = function
  | E -> false
  | N (l, k, _, r, _, _) ->
      match compare x k with
	| n when n < 0 -> member_node x l
	| n when n > 0 -> member_node x r
	| _ -> true

let rec new_node_rec = fun n graph ->
  if member_node n graph then new_node_rec (n + 1) graph
  else n

let new_node = function graph -> new_node_rec 0 graph

let rec new_nodes_rec n graph = function
  | 0 -> []
  | k when member_node n graph -> new_nodes_rec (n + 1) graph k
  | k -> n :: new_nodes_rec (n + 1) graph (k - 1)

let new_nodes = fun n graph -> new_nodes_rec 0 graph n

let rec insert_node_list = fun list graph ->
  match list with
    | [] -> graph
    | (node, value) :: tail -> 
	insert_node_list tail (insert_node node value graph)

let rec delete_two_raise y = function
  | ETwo -> raise Not_found
  | NTwo (l, k, v, r, _) ->
      match compare y k with
	| n when n < 0 -> makeBTwo (delete_two_raise y l) k v r
	| n when n > 0 -> makeBTwo l k v (delete_two_raise y r)
	| _ -> append_two l r

let delete_two = fun y tree ->
  try
    delete_two_raise y tree
  with Not_found -> tree

let rec delete_node_raise id = function
  | E -> raise Not_found
  | N (l, k, v, r, s, _) ->
      match compare id k with
	| n when n < 0 -> 
	    makeBTree (delete_node_raise id l) k v 
	    (map_tree (delete_two id) r) (delete_two id s)
	| n when n > 0 ->
	    makeBTree (map_tree (delete_two id) l) k v 
	    (delete_node_raise id r) (delete_two id s)
	| _ -> append (map_tree (delete_two id) l) (map_tree (delete_two id) r)

let delete_node = fun id graph ->
  try
    delete_node_raise id graph
  with Not_found -> graph

let rec delete_node_unsafe_raise id = function
  | E -> raise Not_found
  | N (l, k, v, r, s, _) ->
      match compare id k with
	| n when n < 0 -> makeBTree (delete_node_unsafe_raise id l) k v r s
	| n when n > 0 -> makeBTree l k v (delete_node_unsafe_raise id r) s
	| _ -> append l r

let delete_node_unsafe = fun id graph ->
  try
    delete_node_unsafe_raise id graph
  with Not_found -> graph

let rec delete_node_list = fun list graph ->
  match list with
    | [] -> graph
    | node :: tail -> delete_node_list tail (delete_node node graph)

let rec successors_rec result x = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) ->
      match compare x k with
	| n when n < 0 -> successors_rec result x l
	| n when n > 0 -> successors_rec result x r
	| _ -> nodes_two result s

let rec successors x = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) ->
      match compare x k with
	| n when n < 0 -> successors x l
	| n when n > 0 -> successors x r
	| _ -> nodes_two [] s

let rec predecessors_rec result x = function
  | E -> result
  | N (l, k, _, r, s, _) ->
      predecessors_rec (
	predecessors_rec (
	  if member_two x s then k :: result else result
	) x l
      ) x r

let predecessors = fun x graph -> predecessors_rec [] x graph

let rec adjacents_rec (pred, succ) x = function
  | E -> (pred, succ)
  | N (l, k, _, r, s, _) ->
      adjacents_rec (
	adjacents_rec (
	  ((if member_two x s then k :: pred else pred), 
	   if x = k then nodes_two succ s else succ)
	) x l
      ) x r

let adjacents = fun x graph -> adjacents_rec ([], []) x graph

let rec exterior_edges_rec result x = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) ->
      match compare x k with
	| n when n < 0 -> exterior_edges_rec result x l
	| n when n > 0 -> exterior_edges_rec result x r
	| _ -> to_list_two x result s

let rec exterior_edges x = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) ->
      match compare x k with
	| n when n < 0 -> exterior_edges x l
	| n when n > 0 -> exterior_edges x r
	| _ -> to_list_two x [] s

let rec lookup_two y = function
  | ETwo -> raise Not_found
  | NTwo (l, k, v, r, _) ->
      match compare y k with
	| n when n < 0 -> lookup_two y l
	| n when n > 0 -> lookup_two y r
	| _ -> v

let rec interior_edges_rec result x = function
  | E -> result
  | N (l, k, _, r, s, _) ->
      interior_edges_rec (
	interior_edges_rec (
	  try
	    (k, lookup_two x s, x) :: result
	  with Not_found -> result
	) x l
      ) x r

let interior_edges = fun x graph -> interior_edges_rec [] x graph

let rec adjacent_edges_rec (pred, succ) x = function
  | E -> (pred, succ)
  | N (l, k, _, r, s, _) ->
      adjacent_edges_rec (
	adjacent_edges_rec (
	  ((try 
	      (k, lookup_two x s, x) :: pred 
	    with Not_found -> pred), 
	   if x = k then to_list_two x succ s else succ)
	) x l
      ) x r

let adjacent_edges = fun x graph -> adjacent_edges_rec ([], []) x graph

let rec exterior_degree x = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) ->
      match compare x k with
	| n when n < 0 -> exterior_degree x l
	| n when n > 0 -> exterior_degree x r
	| _ -> size_two 0 s

let rec interior_degree_rec total x = function
  | E -> total
  | N (l, _, _, r, s, _) ->
      interior_degree_rec (
	interior_degree_rec (
	  if member_two x s then (1 + total) else total
	) x l
      ) x r
			     
let interior_degree = fun id graph -> interior_degree_rec 0 id graph

let degree = fun x graph -> (interior_degree x graph, exterior_degree x graph)

let rec filter_nodes_two f result = function
  | ETwo -> result
  | NTwo (l, k, v, r, _) ->
      filter_nodes_two f (
	filter_nodes_two f (
	  if f k then insert_two k v result 
	  else result
	) l
      ) r

let rec filter_nodes f = function
  | E -> E
  | N (l, k, v, r, s, _) ->
      if f k then makeSTree (filter_nodes f l) k v (filter_nodes f r) 
	(filter_nodes_two f ETwo s)
      else appendS (filter_nodes f l) (filter_nodes f r)

let rec filter_lt_two y = function
  | ETwo -> ETwo
  | NTwo (l, k, v, r, _) ->
      match compare y k with
	| n when n < 0 -> filter_lt_two y l
	| n when n > 0 -> makeSTwo l k v (filter_lt_two y r)
	| _ -> l

let rec filter_gt_two y = function
  | ETwo -> ETwo
  | NTwo (l, k, v, r, _) ->
      match compare y k with
	| n when n < 0 -> makeSTwo (filter_gt_two y l) k v r
	| n when n > 0 -> filter_gt_two y r
	| _ -> r

let rec filter_nodes_lt x = function
  | E -> E
  | N (l, k, v, r, s, _) ->
      match compare x k with
	| n when n < 0 -> filter_nodes_lt x l
	| n when n > 0 -> makeSTree (map_tree (filter_lt_two x) l) k v 
	    (filter_nodes_lt x r) (filter_lt_two x s)
	| _ -> map_tree (filter_lt_two x) l

let rec filter_nodes_gt x = function
  | E -> E
  | N (l, k, v, r, s, _) ->
      match compare x k with
	| n when n < 0 -> makeSTree (filter_nodes_gt x l) k v 
	    (map_tree (filter_gt_two x) r) (filter_gt_two x s)
	| n when n > 0 -> filter_nodes_gt x r
	| _ -> map_tree (filter_gt_two x) r

let rec filter_le_two x = function
  | ETwo -> ETwo
  | NTwo (l, k, v, r, _) ->
      match compare x k with
	| n when n < 0 -> filter_le_two x l
	| _ -> makeSTwo l k v (filter_le_two x r)

let rec filter_ge_two x = function
  | ETwo -> ETwo
  | NTwo (l, k, v, r, _) ->
      match compare x k with
	| n when n <= 0 -> makeSTwo (filter_ge_two x l) k v r
	| _ -> filter_ge_two x r

let rec filter_nodes_le x = function
  | E -> E
  | N (l, k, v, r, s, _) ->
      match compare x k with
	| n when n < 0 -> filter_nodes_le x l
	| _ -> makeSTree (map_tree (filter_le_two x) l) k v 
	    (filter_nodes_le x r) (filter_le_two x s)

let rec filter_nodes_ge x = function
  | E -> E
  | N (l, k, v, r, s, _) ->
      match compare x k with
	| n when n <= 0 -> makeSTree (filter_nodes_ge x l) k v 
	    (map_tree (filter_le_two x) r) (filter_ge_two x s)
	| _ -> filter_nodes_ge x r

let rec range_two min max = function
  | ETwo -> ETwo
  | NTwo (l, k, v, r, _) ->
      if compare k min < 0 then range_two min max r
      else if compare k max > 0 then range_two min max l
      else makeSTwo (filter_ge_two min l) k v (filter_le_two max r)

let rec range_nodes min max = function
  | E -> E
  | N (l, k, v, r, s, _) ->
      if compare k min < 0 then range_nodes min max r
      else if compare k max > 0 then range_nodes min max l
      else makeSTree (filter_nodes_ge min l) k v (filter_nodes_le max r) 
	(range_two min max s)

let rec look_for_node f = function
  | E -> raise Not_found
  | N (_, k, _, _, _, _) when f k -> k
  | N (l, _, _, r, _, _) -> 
      try look_for_node f l 
      with Not_found -> look_for_node f r

let choose_node = function
  | E -> raise Empty
  | N (_, k, _, _, _, _) -> k

let rec min_node = function
  | E -> raise Not_found
  | N (E, k, _, _, _, _) -> k
  | N (l, _, _, _, _, _) -> min_node l

let rec max_node = function
  | E -> raise Not_found
  | N (_, k, _, E, _, _) -> k
  | N (_, _, _, r, _, _) -> max_node r

let rec next_node x = function
  | E -> raise Not_found
  | N (l, k, _, r, _, _) ->
      match compare x k with
	| n when n < 0 -> (try next_node x l with Not_found -> k)
	| n when n > 0 -> next_node x r
	| _ -> min_node r (* raises [Not_found] *)

let rec previous_node x = function
  | E -> raise Not_found
  | N (l, k, _, r, _, _) ->
      match compare x k with
	| n when n < 0 -> previous_node x l
	| n when n > 0 -> (try previous_node x r with Not_found -> k)
	| _ -> max_node l (* raises [Not_found] *)
	      
let rec ceil_node x = function
  | E -> raise Not_found
  | N (l, k, _, r, _, _) ->
      match compare x k with
	| n when n < 0 -> (try ceil_node x l with Not_found -> k)
	| n when n > 0 -> ceil_node x r
	| _ -> x

let rec floor_node x = function
  | E -> raise Not_found
  | N (l, k, _, r, _, _) ->
      match compare x k with
	| n when n < 0 -> floor_node x l
	| n when n > 0 -> (try floor_node x r with Not_found -> k)
	| _ -> x

let rec for_all_nodes f = function
  | E -> true
  | N (l, k, _, r, _, _) -> f k && for_all_nodes f l && for_all_nodes f r

let rec exists_node f = function
  | E -> false
  | N (l, k, _, r, _, _) -> f k || exists_node f l || exists_node f r

let rec iter_nodes f = function
  | E -> ()
  | N (l, k, _, r, _, _) -> iter_nodes f l ; f k ; iter_nodes f r

let rec fold_left_nodes f e = function
  | E -> e
  | N (l, k, _, r, _, _) -> fold_left_nodes f (f (fold_left_nodes f e l) k) r

let rec fold_right_nodes = fun f set e ->
  match set with
    | E -> e 
    | N (l, k, _, r, _, _) -> 
	fold_right_nodes f l (f k (fold_right_nodes f r e))

let fold_nodes = fold_right_nodes


(** Miscellaneous **)
(** Divers **)
(** Micelaneos **)


let rec for_all_contexts = fun f graph ->
  match graph with
    | E -> true
    | N (l, k, _, r, s, _) -> 
	let 
	  succ = to_list_two k [] s and 
	  pred = interior_edges k graph 
	in
	  f pred k succ && for_all_contexts f l && for_all_contexts f r

let rec exists_context = fun f graph ->
  match graph with
    | E -> false
    | N (l, k, _, r, s, _) -> 
	let
	  succ = to_list_two k [] s and 
	  pred = interior_edges k graph 
	in
	  f pred k succ || exists_context f l || exists_context f r

let rec for_all_exterior_context f = function
  | E -> true
  | N (l, k, _, r, s, _) ->
      f k (to_list_two k [] s) && 
      for_all_exterior_context f l &&
      for_all_exterior_context f r

let rec exists_exterior_context f = function
  | E -> false
  | N (l, k, _, r, s, _) ->
      f k (to_list_two k [] s) ||
      exists_exterior_context f l ||
      exists_exterior_context f r


(** Operations on edges **)
(** Operations sur les arcs **)
(** Operaciones sobre los arcos **)


let insert_edge = insert

let insert_edge_raise = insert_raise

let rec insert_edge_list = fun list graph -> 
  match list with
    | [] -> graph
    | (i, v, j) :: tail -> insert_edge_list tail (insert_edge i v j graph)

let rec delete_edge_raise x y = function
  | E -> raise Not_found
  | N (l, k, v, r, s, _) ->
      match compare x k with
	| n when n < 0 -> makeTree (delete_edge_raise x y l) k v r s
	| n when n > 0 -> makeTree l k v (delete_edge_raise x y r) s
	| _ -> makeTree l k v r (delete_two_raise y s)

let delete_edge = fun x y tree ->
  try 
    delete_edge_raise x y tree
  with Not_found -> tree

let rec delete_edge_list = fun list set ->
  match list with
    | [] -> set
    | (x, y) :: tail -> delete_edge_list tail (delete_edge x y set)

let rec edge_label x y = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) ->
      match compare x k with
	| n when n < 0 -> edge_label x y l
	| n when n > 0 -> edge_label x y r
	| _ -> lookup_two y s

let rec map_images_two f = function
  | ETwo -> ETwo
  | NTwo (l, k, v, r, h) -> 
      NTwo (map_images_two f l, k, f v, map_images_two f r, h)

let rec map_labels f = function
  | E -> E
  | N (l, k, v, r, s, h) ->
      N (map_labels f l, k, v, map_labels f r, map_images_two f s, h)

let rec member_edge x y = function
  | E -> false
  | N (l, k, _, r, s, _) ->
      match compare x k with
	 | n when n < 0 -> member_edge x y l
	 | n when n > 0 -> member_edge x y r
	 | _ -> member_two y s

let rec look_for_two f = function
  | ETwo -> raise Not_found
  | NTwo (_, k, v, _, _) when f v k -> (v, k)
  | NTwo (l, _, _, r, _) -> 
      (try look_for_two f l with Not_found -> look_for_two f r)

let rec look_for_edge f = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) ->
      try let (v, k') = look_for_two (f k) s in (k, v, k')
      with Not_found ->
	try look_for_edge f l 
	with Not_found -> look_for_edge f r
  
let choose_edge = function graph -> look_for_edge (fun i v j -> true) graph

let rec min_element_two = function
  | ETwo -> raise Not_found
  | NTwo (ETwo, k, v, _, _) -> (k, v)
  | NTwo (l, _, _, _, _) -> min_element_two l

let rec min_edge = function
  | E -> raise Not_found
  | N (E, x, _, _, s, _) -> let (y, v) = min_element_two s in (x, v, y)
  | N (l, _, _, _, _, _) -> min_edge l

let rec max_element_two = function
  | ETwo -> raise Not_found
  | NTwo (_, k, v, ETwo, _) -> (k, v)
  | NTwo (_, _, _, r, _) -> max_element_two r

let rec max_edge = function
  | E -> raise Not_found
  | N (_, x, _, E, s, _) -> let (y, v) = max_element_two s in (x, v, y)
  | N (_, _, _, l, _, _) -> max_edge l

let rec next_two y = function
  | ETwo -> raise Not_found
  | NTwo (l, k, v, r, _) -> 
      match compare y k with
	| n when n < 0 -> (try next_two y l with Not_found -> (k, v))
	| n when n > 0 -> next_two y r
	| _ -> min_element_two r (* [Not_found] *)

let rec next_edge x y = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) -> 
      match compare x k with
	| n when n < 0 -> 
	    (try next_edge x y l 
	     with Not_found -> let (k', v) = min_element_two s in (k, v,  k'))
	| n when n > 0 -> next_edge x y r
	| _ ->
	    try let (k', v) = next_two y s in (k, v, k') 
	    with Not_found -> min_edge r

let rec previous_two y = function
  | ETwo -> raise Not_found
  | NTwo (l, k, v, r, _) ->
      match compare y k with
	| n when n < 0 -> previous_two y l
	| n when n > 0 -> (try previous_two y r with Not_found -> (k, v))
	| _ -> max_element_two l (* [Not_found] *)

let rec previous_edge x y = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) ->
      match compare x k with
	| n when n < 0 -> previous_edge x y l
	| n when n > 0 -> 
	    (try previous_edge x y r 
	     with Not_found -> let (k', v) = max_element_two s in (k, v, k'))
	| _ -> 
	    try let (k', v) = previous_two y s in (k, v, k')
	    with Not_found -> max_edge l
	
let rec ceil_two y = function
  | ETwo -> raise Not_found
  | NTwo (l, k, v, r, _) -> 
      match compare y k with
	| n when n < 0 -> (try ceil_two y l with Not_found -> (k, v))
	| n when n > 0 -> ceil_two y r
	| _ -> (k, v)

let rec ceil_edge x y = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) -> 
      match compare x k with
	| n when n < 0 -> 
	    (try ceil_edge x y l 
	     with Not_found -> let (k', v) = min_element_two s in (k, v, k'))
	| n when n > 0 -> ceil_edge x y r
	| _ -> 
	    try let (k', v) = ceil_two y s in (k, v, k') 
	    with Not_found -> min_edge r

let rec floor_two y = function
  | ETwo -> raise Not_found
  | NTwo (l, k, v, r, _) ->
      match compare y k with
	| n when n < 0 -> floor_two y l
	| n when n > 0 -> (try floor_two y r with Not_found -> (k, v))
	| _ -> (k, v)

let rec floor_edge x y = function
  | E -> raise Not_found
  | N (l, k, _, r, s, _) ->
      match compare x k with
	| n when n < 0 -> floor_edge x y l
	| n when n > 0 -> 
	    (try floor_edge x y r 
	     with Not_found -> let (k', v) = max_element_two s in (k, v, k'))
	| _ -> 
	    try let (k', v) = floor_two y s in (k, v, k') 
	    with Not_found -> max_edge l

let rec exists_two f = function
  | ETwo -> false
  | NTwo (l, y, v, r, _) -> f v y || exists_two f l || exists_two f r

let rec exists_edge f = function
  | E -> false
  | N (l, k, _, r, s, _) -> 
      exists_two (f k) s || exists_edge f l || exists_edge f r

let rec for_all_two f = function
  | ETwo -> true
  | NTwo (l, y, v, r, _) -> f v y && for_all_two f l && for_all_two f r

let rec for_all_edges f = function
  | E -> true
  | N (l, k, _, r, s, _) -> 
      for_all_two (f k) s && for_all_edges f l && for_all_edges f r

let rec iter_two f = function
  | ETwo -> ()
  | NTwo (l, y, v, r, _) -> iter_two f l ; f v y ; iter_two f r

let rec iter_edges f = function
  | E -> ()
  | N (l, k, _, r, s, _) -> iter_edges f l ; iter_two (f k) s ; iter_edges f r

let rec fold_left_two f e = function
  | ETwo -> e
  | NTwo (l, k, v, r, _) -> fold_left_two f (f (fold_left_two f e l) v k) r

let rec fold_left_edges f e = function
  | E -> e
  | N (l, x, _, r, s, _) -> 
      fold_left_edges f 
      (fold_left_two (fun a y -> f a x y) (fold_left_edges f e l) s) r

let rec fold_right_two = fun f set e ->
  match set with
    | ETwo -> e 
    | NTwo (l, k, v, r, _) -> fold_right_two f l (f v k (fold_right_two f r e))

let rec fold_right_edges = fun f set e ->
  match set with
    | E -> e
    | N (l, x, _, r, s, _) -> 
	fold_right_edges f l (fold_right_two (f x) s (fold_right_edges f r e))

let fold_edges = fold_right_edges


(** Views **)
(** Vues **)
(** Vistas **)


(* faster versions soon *)
(* versions plus rapides ultrieurement *)
(* mejoras pronto *)
    
let node_view = fun node graph ->
  let 
    pred = interior_edges node graph and
    succ = exterior_edges node graph and
    value = lookup_one node graph
  in
    (pred, node, value, succ, delete_node node graph)

let any_node_view = function graph ->
  match graph with
    | E -> raise Empty
    | N (_, node, _, _, _, _) -> 
	let 
	  pred = interior_edges node graph and
	  succ = exterior_edges node graph and
	  value = lookup_one node graph
	in
	  (pred, node, value, succ, delete_node node graph)


(** Graph traversals **)
(** Parcours de graphes **)
(** Recorridos de grafos **)


let rec dfs f e graph = function
  | [] -> e
  | x :: tail ->
      let reduced = delete_node_unsafe x graph in
	try
	  dfs f (f x e) reduced (successors_rec tail x graph)
	with Not_found -> dfs f e reduced tail

let rec dfs' f e graph = function
  | [] -> e
  | (i, v, j) :: tail ->
      let reduced = delete_node_unsafe j graph in
	try 
	  dfs' f (f i v j e) reduced (exterior_edges_rec tail j graph)
	with Not_found -> dfs' f e reduced tail
	  
let depth_first_fold = fun f e graph x -> dfs f e graph [x]

let depth_first_fold' = fun f e graph x -> 
  dfs' f e graph (exterior_edges x graph)

let rec dfs_list result graph = function
  | [] -> result
  | x :: tail ->
      let reduced = delete_node_unsafe x graph in
	try
	  dfs_list (x :: result) reduced (successors_rec tail x graph)
	with Not_found -> dfs_list result reduced tail
      
let rec dfs_list' result graph = function
  | [] -> result
  | (_, _, j) as head :: tail ->
      let reduced = delete_node_unsafe j graph in
	try
	  dfs_list' (head :: result) reduced (exterior_edges_rec tail j graph)
	with Not_found -> dfs_list' result reduced tail

let depth_first_list = fun graph x -> List.rev (dfs_list [] graph [x])

let depth_first_list' = fun graph x -> 
  List.rev (dfs_list' [] 
	      (delete_node_unsafe x graph) 
	      (List.rev (exterior_edges x graph))
	   )

let rec 
  bfs_notail f e graph = function
    | [] -> e
    | x :: tail ->
	let reduced = delete_node_unsafe x graph in
	  try
	    bfs_tail f (f x e) reduced tail (successors x graph)
	  with Not_found -> bfs_notail f e reduced tail
and
  bfs_tail f e = fun graph hd tl ->
    match hd with
      | [] ->
	  (match tl with
	     | [] -> e
	     | _ -> bfs_notail f e graph (List.rev tl)
	  )
      | x :: tail ->
	  let reduced = delete_node_unsafe x graph in
	    try
	      bfs_tail f (f x e) reduced tail (successors_rec tl x graph)
	    with Not_found -> bfs_tail f e reduced tail tl

let rec 
  bfs_notail' f e graph = function
    | [] -> e
    | (i, v, j) :: tail ->
	let reduced = delete_node_unsafe j graph in
	  try
	    bfs_tail' f (f i v j e) reduced tail (exterior_edges j graph)
	  with Not_found -> bfs_notail' f e reduced tail
and
  bfs_tail' f e = fun graph hd tl ->
    match hd with
      | [] ->
	  (match tl with
	     | [] -> e
	     | _ -> bfs_notail' f e graph (List.rev tl)
	  )
      | (i, v, j) :: tail ->
	  let reduced = delete_node_unsafe j graph in
	    try
	      bfs_tail' f (f i v j e) reduced tail 
		(exterior_edges_rec tl j graph)
	    with Not_found -> bfs_tail' f e reduced tail tl

let breadth_first_fold = fun f e graph x -> bfs_notail f e graph [x]

let breadth_first_fold' = fun f e graph x -> 
  bfs_notail' f e (delete_node_unsafe x graph) (exterior_edges x graph)

let rec 
  bfs_list_notail result graph = function
    | [] -> result
    | x :: tail ->
	let reduced = delete_node_unsafe x graph in
	  try
	    bfs_list_tail (x :: result) reduced tail (successors x graph)
	  with Not_found -> bfs_list_notail result reduced tail
and
  bfs_list_tail result = fun graph hd tl ->
    match hd with
      | [] ->
	  (match tl with
	     | [] -> result
	     | _ -> bfs_list_notail result graph (List.rev tl)
	  )
      | x :: tail ->
	  let reduced = delete_node_unsafe x graph in
	    try
	      bfs_list_tail (x :: result) reduced tail 
		(successors_rec tl x graph)
	    with Not_found -> bfs_list_tail result reduced tail tl

let rec 
  bfs_list_notail' result graph = function
    | [] -> result
    | (_, _, j) as head :: tail ->
	let reduced = delete_node_unsafe j graph in
	  try
	    bfs_list_tail' (head :: result) reduced tail 
	      (exterior_edges j graph)
	  with Not_found -> bfs_list_notail' result reduced tail
and
  bfs_list_tail' result = fun graph hd tl ->
    match hd with
      | [] ->
	  (match tl with
	     | [] -> result
	     | _ -> bfs_list_notail' result graph (List.rev tl)
	  )
      | (_, _, j) as head :: tail ->
	  let reduced = delete_node_unsafe j graph in
	    try
	      bfs_list_tail' (head :: result) reduced tail 
		(exterior_edges_rec tl j graph)
	    with Not_found -> bfs_list_tail' result reduced tail tl

let breadth_first_list = fun graph x -> List.rev (bfs_list_notail [] graph [x])

let breadth_first_list' = fun graph x -> 
  List.rev (bfs_list_notail' [] 
	      (delete_node_unsafe x graph) 
	      (exterior_edges x graph)
	   )

exception Cyclic

let rec topological_rec = function
  | E -> raise Not_found
  | N (_, k, _, _, ETwo, _) -> k
  | N (l, _, _, r, _, _) ->
      try topological_rec l with Not_found -> topological_rec r

let rec topological_list_rec result = function
  | E -> result
  | graph ->
     try
       let x = topological_rec graph in
	 topological_list_rec (x :: result) (delete_node x graph)
     with Not_found -> raise Cyclic

let topological_list = function graph -> topological_list_rec [] graph

let rec topological_fold f e = function
  | E -> e
  | graph ->
      try
	let x = topological_rec graph in
	  topological_fold f (f x e) (delete_node x graph)
      with Not_found -> raise Cyclic


(** Debug **)

(** Objets **)
(** Ojects **)
(** Objetos **)

class ['a, 'b] o_graph = fun graph ->

object (self : 'c)

  val contents : ('a, 'b) graph = graph

  method to_graph = contents

  (* Categorial operations *)
  (* Oprations categorielles *)
  (* Operaciones categoriales *)
		      
  method map f = {< contents = map f contents >}
		   
  method plus = function (graph : 'c) -> 
    {< contents = plus contents graph#to_graph >} 

  method to_list = to_list contents

  (* Binary operations *)
  (* Oprations binaires *)
  (* Operaciones binrias *)
		     
  method union = function (graph : 'c) -> 
    {< contents = plus contents graph#to_graph >}
  
  method intersection = function (graph : 'c) -> 
    {< contents = intersection contents graph#to_graph >}
  
  method difference = function (graph : 'c) -> 
    {< contents = difference contents graph#to_graph >}
  
  method symetric_difference = function (graph : 'c) -> 
    {< contents = symetric_difference contents graph#to_graph >}
    
  (* View *)
  (* Vues *)
  (* Vistas *)
   
  method node_view = function node -> 
    let (pred, node, value, succ, graph) = node_view node contents in
      (pred, node, value, succ, {< contents = graph >})

  method any_node_view =
    let (pred, node, value, succ, graph) = any_node_view contents in
      (pred, node, value, succ, {< contents = graph >})

  method is_empty =
    match contents with
      | E -> true
      | N _ -> false

  method nodes = nodes contents

  method edges = edges contents

  method edge_labels_list = edge_labels_list contents

  method node_labels_list = node_labels_list contents

  method nodes_list = nodes_list contents

  method edges_list = edges_list contents

  method degrees_list = degrees_list contents

  (* Traversals *)
  (* Parcours *)
  (* Recorridos *)

  method breadth_first_list = function node -> breadth_first_list contents node

  method depth_first_list = function node -> depth_first_list contents node

  method topological_list = topological_list contents

  method breadth_first_list' = function node -> 
    breadth_first_list' contents node

  method depth_first_list' = function node -> 
    depth_first_list' contents node

  method breadth_first_fold : 'd. (node -> 'd -> 'd) -> 'd -> node -> 'd 
    = fun f e node -> breadth_first_fold f e contents node

  method breadth_first_fold' : 
    'd. (node -> 'b -> node -> 'd -> 'd) -> 'd -> node -> 'd
    = fun f e node -> breadth_first_fold' f e contents node

  method depth_first_fold : 'd. (node -> 'd -> 'd) -> 'd -> node -> 'd
    = fun f e node -> depth_first_fold f e contents node

  method depth_first_fold' : 
    'd. (node -> 'b -> node -> 'd -> 'd) -> 'd -> node -> 'd
    = fun f e node -> depth_first_fold' f e contents node

  method topological_fold : 'd. (node -> 'd -> 'd) -> 'd -> 'd
    = fun f e -> topological_fold f e contents

  (* Operations on nodes *)
  (* Oprations sur les noeuds *)
  (* Operaciones sobre los nudos *)

  method new_node = new_node contents

  method new_nodes = function n -> new_nodes n contents

  method insert_node = fun node value -> 
    {< contents = insert_node node value contents >}
    
  method delete_node = function node ->
    {< contents = delete_node node contents >}

  method insert_node_list = function list ->
    {< contents = insert_node_list list contents >}

  method delete_node_list = function list ->
    {< contents = delete_node_list list contents >}

  method successors = function node -> successors node contents

  method predecessors = function node -> predecessors node contents

  method adjacents = function node -> adjacents node contents

  method exterior_edges = function node -> exterior_edges node contents

  method interior_edges = function node -> interior_edges node contents

  method adjacent_edges = function node -> adjacent_edges node contents

  method exterior_degree = function node -> exterior_degree node contents

  method interior_degree = function node -> interior_degree node contents

  method degree = function node -> degree node contents

  method member_node = function node -> member_node node contents

  method look_for_node = function f -> look_for_node f contents

  method exists_node = function f -> exists_node f contents

  method for_all_nodes = function f -> for_all_nodes f contents

  method iter_nodes : (node -> unit) -> unit 
    = function f -> iter_nodes f contents

  method fold_nodes : 'd. (node -> 'd -> 'd) -> 'd -> 'd 
    = fun f e -> fold_nodes f contents e

  method fold_right_nodes : 'd. (node -> 'd -> 'd) -> 'd -> 'd
    = fun f e -> fold_right_nodes f contents e

  method fold_left_nodes : 'd. ('d -> node -> 'd) -> 'd -> 'd
    = fun f e -> fold_left_nodes f e contents

  (* Operations on edges *)
  (* Operations sur les arcs *)
  (* Operaciones sobre los arcos *)

  method insert_edge = fun i v j -> {< contents = insert_edge i v j contents >}

  method insert_edge_list = function list -> 
    {< contents = insert_edge_list list contents >}

  method delete_edge = fun i j -> {< contents = delete_edge i j contents >}

  method delete_edge_list = function list ->
    {< contents = delete_edge_list list contents >}

  method edge_label = fun i j -> edge_label i j contents

  method member_edge = fun i j -> member_edge i j contents

  method look_for_edge = function f -> look_for_edge f contents

  method exists_edge = function f -> exists_edge f contents

  method for_all_edges = function f -> for_all_edges f contents

  method iter_edges : (node -> 'b -> node -> unit) -> unit 
    = function f -> iter_edges f contents

  method fold_edges : 'd. (node -> 'b -> node -> 'd -> 'd) -> 'd -> 'd 
    = fun f e -> fold_edges f contents e

  method fold_right_edges : 'd.  (node -> 'b -> node -> 'd -> 'd) -> 'd -> 'd 
    = fun f e -> fold_right_edges f contents e

  method fold_left_edges : 'd. ('d -> node -> 'b -> node -> 'd) -> 'd -> 'd
    = fun f e -> fold_left_edges f e contents

end

(* Constructors *)
(* Constructeurs *)
(* Constructores *)

let o_injection = fun i a v j b -> new o_graph (injection i a v j b)
let o_map f = function a -> new o_graph (map f a#to_graph)
let o_plus = fun a b -> new o_graph (plus a#to_graph b#to_graph)
let o_zero = function () -> new o_graph E

let o_from_list = function list -> new o_graph (from_list list)
let o_from_graph = function graph -> new o_graph graph

(* Binary operations *)
(* Oprations binaires *)
(* Operaciones binrias *)
  
let o_union = fun a b -> new o_graph (plus a#to_graph b#to_graph)
let o_intersection = fun a b -> 
  new o_graph (intersection a#to_graph b#to_graph)
let o_difference = fun a b -> new o_graph (difference a#to_graph b#to_graph)
let o_symetric_difference = fun a b -> 
  new o_graph (symetric_difference a#to_graph b#to_graph)



(* let compare = fun a b -> compare_rec [a] [b] *)


