type ('a, 'b) graph

type node = int
type 'b edge = node * 'b * node

val moduleName : string

(** Oprations catgorielles **)
(** Categorial operiations **)
(** Operaciones de categora **)

val injection : 'a -> node -> 'b -> 'a -> node -> ('a, 'b) graph
val map : ((node * 'a) * 'b * (node * 'a) -> (node * 'c) * 'd * (node * 'c)) ->
  ('a, 'b) graph -> ('c, 'd) graph

val plus : ('a, 'b) graph -> ('a, 'b) graph -> ('a, 'b) graph
val zero : unit -> ('a, 'b) graph

val from_list : ((node * 'a) * 'b * (node * 'a)) list -> ('a, 'b) graph
val to_list : ('a, 'b) graph -> ((node * 'a) * 'b * (node * 'a)) list

(** Oprations binaires **)
(** Binairy operations **)
(** Operaciones binrias **)

val union        : ('a, 'b) graph -> ('a, 'b) graph -> ('a, 'b) graph
val intersection : ('a, 'b) graph -> ('a, 'b) graph -> ('a, 'b) graph
val difference   : ('a, 'b) graph -> ('a, 'b) graph -> ('a, 'b) graph
val symetric_difference : ('a, 'b) graph -> ('a, 'b) graph -> ('a, 'b) graph

(** Views **)
(** Vues **)
(** Vistas **)

val node_view : node -> ('a, 'b) graph -> 
  'b edge list * node * 'a * 'b edge list * ('a, 'b) graph

val any_node_view : ('a, 'b) graph -> 
  'b edge list * node * 'a * 'b edge list * ('a, 'b) graph

(** Observateurs **)
(** Observers **)
(** Observadores **)

val is_empty : ('a, 'b) graph -> bool
 
val nodes : ('a, 'b) graph -> int
val edges : ('a, 'b) graph -> int

val edge_labels_list : ('a, 'b) graph -> 'b list
val node_labels_list : ('a, 'b) graph -> 'a list

val nodes_list   : ('a, 'b) graph -> (node * 'a) list
val edges_list   : ('a, 'b) graph -> 'b edge list
val degrees_list : ('a, 'b) graph -> (node * int * int) list

val labels_tree  : ('a, 'b) graph -> 'b BairePervasives.tree
val nodes_tree   : ('a, 'b) graph -> (node * 'a) BairePervasives.tree
val edges_tree   : ('a, 'b) graph -> (node * node * 'b) BairePervasives.tree
val degrees_tree : ('a, 'b) graph -> (node * int * int) BairePervasives.tree

(** Miscellaneous **)
(** Divers **)
(** Micelaneos **)

val union_with : ('b -> 'b -> 'b) -> 
  ('a, 'b) graph -> ('a, 'b) graph -> ('a, 'b) graph

val intersection_with : ('b -> 'b -> 'b) -> 
  ('a, 'b) graph -> ('a, 'b) graph -> ('a, 'b) graph

val exists_context : 
  ('b edge list -> node -> 'b edge list -> bool) -> ('a, 'b) graph -> bool

val for_all_contexts : 
  ('b edge list -> node -> 'b edge list -> bool) -> ('a, 'b) graph -> bool

val exists_exterior_context :
  (node -> 'b edge list -> bool) -> ('a, 'b) graph -> bool

val for_all_exterior_context :
  (node -> 'b edge list -> bool) -> ('a, 'b) graph -> bool

(** Traversals **)
(** Parcours **)
(** Recorridos **)

val breadth_first_list  : ('a, 'b) graph -> node -> node list
val depth_first_list    : ('a, 'b) graph -> node -> node list

val topological_list    : ('a, 'b) graph -> node list           (* [Cyclic] *)

val breadth_first_list' : ('a, 'b) graph -> node -> 'b edge list
val depth_first_list'   : ('a, 'b) graph -> node -> 'b edge list

val breadth_first_fold   : (node -> 'c -> 'c) -> 
  'c -> ('a, 'b) graph -> node -> 'c
val breadth_first_fold'  : (node -> 'b -> node -> 'c -> 'c) -> 
  'c -> ('a, 'b) graph -> node -> 'c

val depth_first_fold   : (node -> 'c -> 'c) -> 
  'c -> ('a, 'b) graph -> node -> 'c
val depth_first_fold'  : (node -> 'b -> node -> 'c -> 'c) ->
  'c -> ('a, 'b) graph -> node -> 'c

val topological_fold   : (node -> 'c -> 'c) -> 'c -> ('a, 'b) graph -> 'c
  (* [Cyclic] *)


(** Operations on nodes **)
(** Oprations sur les noeuds **)
(** Operaciones sobre los nudos **)


val new_node  : ('a, 'b) graph -> node
val new_nodes : int -> ('a, 'b) graph -> node list

val insert_node : node -> 'a -> ('a, 'b) graph -> ('a, 'b) graph
val delete_node : node -> ('a, 'b) graph -> ('a, 'b) graph

val insert_node_list : (node * 'a) list -> ('a, 'b) graph -> ('a, 'b) graph
val delete_node_list : node list -> ('a, 'b) graph -> ('a, 'b) graph

val insert_node_raise : node -> 'a -> ('a, 'b) graph -> ('a, 'b) graph 
  (* [Found] *)

val delete_node_raise : node -> ('a, 'b) graph -> ('a, 'b) graph 
  (* [Not_found] *)

val successors   : node -> ('a, 'b) graph -> node list
val predecessors : node -> ('a, 'b) graph -> node list
val adjacents    : node -> ('a, 'b) graph -> node list * node list

val exterior_edges  : node -> ('a, 'b) graph -> 'b edge list
val interior_edges  : node -> ('a, 'b) graph -> 'b edge list
val adjacent_edges  : node -> ('a, 'b) graph -> 'b edge list * 'b edge list

val exterior_degree : node -> ('a, 'b) graph -> int
val interior_degree : node -> ('a, 'b) graph -> int
val degree          : node -> ('a, 'b) graph -> int * int

val filter_nodes : (node -> bool) -> ('a, 'b) graph -> ('a, 'b) graph
val filter_nodes_lt : node -> ('a, 'b) graph -> ('a, 'b) graph
val filter_nodes_le : node -> ('a, 'b) graph -> ('a, 'b) graph
val filter_nodes_gt : node -> ('a, 'b) graph -> ('a, 'b) graph
val filter_nodes_ge : node -> ('a, 'b) graph -> ('a, 'b) graph

val member_node   : node -> ('a, 'b) graph -> bool
val look_for_node : (node -> bool) -> ('a, 'b) graph -> node

val next_node     : node -> ('a, 'b) graph -> node
val previous_node : node -> ('a, 'b) graph -> node
val ceil_node     : node -> ('a, 'b) graph -> node
val floor_node    : node -> ('a, 'b) graph -> node

val choose_node : ('a, 'b) graph -> node
val min_node    : ('a, 'b) graph -> node
val max_node    : ('a, 'b) graph -> node

val exists_node   : (node -> bool) -> ('a, 'b) graph -> bool
val for_all_nodes : (node -> bool) -> ('a, 'b) graph -> bool

val iter_nodes : (node -> unit) -> ('a, 'b) graph -> unit

val fold_nodes       : (node -> 'b -> 'b) -> ('a, 'b) graph -> 'b -> 'b
val fold_right_nodes : (node -> 'b -> 'b) -> ('a, 'b) graph -> 'b -> 'b
val fold_left_nodes  : ('b -> node -> 'b) -> 'b -> ('a, 'b) graph -> 'b


(** Operations on edges **)
(** Operations sur les arcs **)
(** Operaciones sobre los arcos **)


val insert_edge : node -> 'b -> node -> ('a, 'b) graph -> ('a, 'b) graph 
  (* [Not_found] *)

val insert_edge_list : 'b edge list -> ('a, 'b) graph -> ('a, 'b) graph

val delete_edge : node -> node -> ('a, 'b) graph -> ('a, 'b) graph
val delete_edge_list : (node * node) list -> ('a, 'b) graph -> ('a, 'b) graph

val insert_edge_raise : node -> 'b -> node -> ('a, 'b) graph -> ('a, 'b) graph
(* [Not_found] | [Found] *)

val delete_edge_raise : node -> node -> ('a, 'b) graph -> ('a, 'b) graph 
  (* [Not_found] *)

val edge_label : node -> node -> ('a, 'b) graph -> 'b
val map_labels : ('b -> 'c) -> ('a, 'b) graph -> ('a, 'c) graph

val member_edge : node -> node -> ('a, 'b) graph -> bool

val look_for_edge : (node -> 'b -> node -> bool) -> 
  ('a, 'b) graph -> 'b edge
  
val next_edge     : node -> node -> ('a, 'b) graph -> 'b edge
val previous_edge : node -> node -> ('a, 'b) graph -> 'b edge
val ceil_edge     : node -> node -> ('a, 'b) graph -> 'b edge
val floor_edge    : node -> node -> ('a, 'b) graph -> 'b edge

val choose_edge : ('a, 'b) graph -> 'b edge (* [Empty] *)
val min_edge    : ('a, 'b) graph -> 'b edge (* [Not_found] *)
val max_edge    : ('a, 'b) graph -> 'b edge (* [Not_found] *)

val exists_edge   : (node -> 'b -> node -> bool) -> ('a, 'b) graph -> bool
val for_all_edges : (node -> 'b -> node -> bool) -> ('a, 'b) graph -> bool

val iter_edges : (node -> 'b -> node -> unit) -> ('a, 'b) graph -> unit

val fold_edges : 
  (node -> 'b -> node -> 'b -> 'b) -> ('a, 'b) graph -> 'b -> 'b

val fold_right_edges : 
  (node -> 'b -> node -> 'b -> 'b) -> ('a, 'b) graph -> 'b -> 'b

val fold_left_edges : 
  ('b -> node -> 'b -> node -> 'b) -> 'b -> ('a, 'b) graph -> 'b


(* to be used with care *)
(* oprations  utiliser avec prcaution *)
(* a usar con cautela *)

val delete_node_unsafe : node -> ('a, 'b) graph -> ('a, 'b) graph
val delete_node_unsafe_raise : node -> ('a, 'b) graph -> ('a, 'b) graph


(** Object wrapper **)
(** Capsule objet **)
(** Componente objeto **)


class type ['a, 'b] o_graph =
object ('c)
  
  method map : 
    ((node * 'a) * 'b * (node * 'a) -> (node * 'a) * 'b * (node * 'a)) -> 'c
  method plus : 'c -> 'c

  method to_list  : ((node * 'a) * 'b * (node * 'a)) list
  method to_graph : ('a, 'b) graph

  method union : 'c -> 'c
  method intersection : 'c -> 'c
  method difference : 'c -> 'c
  method symetric_difference : 'c -> 'c

  method node_view : node -> 'b edge list * node * 'a * 'b edge list * 'c

  method any_node_view : 'b edge list * node * 'a * 'b edge list * 'c

  method is_empty : bool

  method nodes : int
  method edges : int

  method edge_labels_list : 'b list
  method node_labels_list : 'a list
  method nodes_list   : (node * 'a) list
  method edges_list   : 'b edge list
  method degrees_list : (node * int * int) list

  method breadth_first_list  : node -> node list
  method depth_first_list    : node -> node list
    
  method topological_list    : node list
    
  method breadth_first_list' : node -> 'b edge list
  method depth_first_list'   : node -> 'b edge list
    
  method breadth_first_fold  : (node -> 'd -> 'd) -> 'd -> node -> 'd
  method breadth_first_fold' : (node -> 'b -> node -> 'd -> 'd) -> 
    'd -> node -> 'd
    
  method depth_first_fold  : (node -> 'd -> 'd) -> 'd -> node -> 'd
  method depth_first_fold' : (node -> 'b -> node -> 'd -> 'd) ->
    'd -> node -> 'd
    
  method topological_fold   : (node -> 'd -> 'd) -> 'd -> 'd

  method new_node : node
  method new_nodes : int -> node list

  method insert_node : node -> 'a -> 'c
  method delete_node : node -> 'c

  method insert_node_list : (node * 'a ) list -> 'c
  method delete_node_list : node list -> 'c

  method successors   : node -> node list
  method predecessors : node -> node list
  method adjacents    : node -> node list * node list

  method exterior_edges : node -> 'b edge list
  method interior_edges : node -> 'b edge list
  method adjacent_edges : node -> 'b edge list * 'b edge list
 
  method exterior_degree : node -> int
  method interior_degree : node -> int
  method degree          : node -> int * int

  method member_node   : node -> bool
  method look_for_node : (node -> bool) -> node

  method exists_node   : (node -> bool) -> bool
  method for_all_nodes : (node -> bool) -> bool

  method iter_nodes : (node -> unit) -> unit

  method fold_nodes       : (node -> 'd -> 'd) -> 'd -> 'd
  method fold_right_nodes : (node -> 'd -> 'd) -> 'd -> 'd
  method fold_left_nodes  : ('d -> node -> 'd) -> 'd -> 'd

  method insert_edge : node -> 'b -> node -> 'c
  method insert_edge_list : 'b edge list -> 'c

  method delete_edge : node -> node -> 'c
  method delete_edge_list : (node * node) list -> 'c

  method edge_label : node -> node -> 'b

  method member_edge : node -> node -> bool

  method look_for_edge : (node -> 'b -> node -> bool) -> 'b edge

  method exists_edge   : (node -> 'b -> node -> bool) -> bool
  method for_all_edges : (node -> 'b -> node -> bool) -> bool

  method iter_edges : (node -> 'b -> node -> unit) -> unit
    
  method fold_edges       : (node -> 'b -> node -> 'd -> 'd) -> 'd -> 'd
  method fold_right_edges : (node -> 'b -> node -> 'd -> 'd) -> 'd -> 'd
  method fold_left_edges  : ('d -> node -> 'b -> node -> 'd) -> 'd -> 'd

end

(** Constructors **)
(** Constructeurs **)
(** Constructores **)

val o_injection : 'a -> node -> 'b -> 'a -> node -> ('a, 'b) o_graph

val o_map : 
  ((node * 'a) * 'b * (node * 'a) -> (node * 'c) * 'd * (node * 'c)) ->
  ('a, 'b) o_graph -> ('c, 'd) o_graph

val o_plus : ('a, 'b) o_graph -> ('a, 'b) o_graph -> ('a, 'b) o_graph
val o_zero : unit -> ('a, 'b) o_graph

val o_from_list : ((node * 'a) * 'b * (node * 'a)) list -> ('a, 'b) o_graph
val o_from_graph : ('a, 'b) graph -> ('a, 'b) o_graph

(** Binary operations **)
(** Oprations binaires **)
(** Operaciones binrias **)

val o_union        : ('a, 'b) o_graph -> ('a, 'b) o_graph -> ('a, 'b) o_graph
val o_intersection : ('a, 'b) o_graph -> ('a, 'b) o_graph -> ('a, 'b) o_graph
val o_difference   : ('a, 'b) o_graph -> ('a, 'b) o_graph -> ('a, 'b) o_graph
val o_symetric_difference : ('a, 'b) o_graph -> ('a, 'b) o_graph -> 
  ('a, 'b) o_graph

