val moduleName : string

(** Oprations catgorielles *)
(** Categorial operations *)
(** Operaciones categoriales *)


val injection : 'a -> 'a list
  (* Create a singleton sequence *)
  
val map : ('a -> 'b) -> 'a list -> 'b list 
  (* return the result of applying a function to every element of a sequence *)
  
val join : 'a list list -> 'a list 
  (* flatten a sequence of sequences into a simple sequence *)

val plus : 'a list -> 'a list -> 'a list 
  (* catenate to sequences with the first argument on the left 
   * see also list aliases 
   *)

val zero : unit -> 'a list
  (* the empty sequence *)
  
val make : int -> 'a -> 'a list 
  (* create a sequence contanining n copies of the given element *)

val get : 'a list -> int -> 'a 
  (* return the element at the given index 
   * raise [Out_of_bound n] where (n + 1) is the number of elements lacking
   * if the list is too short
   *)

val set : 'a list -> int -> 'a -> 'a list 
  (* replace the element at the given index
   * raise [Out_of_bound n] where (n + 1) is the number of elements lacking
   * if the list is too short
   *)

val from_list : 'a list -> 'a list
  (* convert a list to a sequence *)

val to_list : 'a list -> 'a list
  (* convert a sequence to a list *)


(** Acces *)
(** Accs *)
(** Acceso *)


val cons : 'a -> 'a list -> 'a list 
  (* add a new element to the left of a sequence *)

val snoc : 'a list -> 'a -> 'a list 
  (* add a new element to the right of a sequence *)

val init : int -> (int -> 'a) -> 'a list 
  (* create a sequence containing the result of applying the given function
   * to the integers 0 .. n - 1 
   *)

val filter : ('a -> bool) -> 'a list -> 'a list 
  (* extract the elements of a sequence that satisfy a given predicate *)

val take : int -> 'a list -> 'a list
  (* extract a prefix of length i from a sequence *)

val drop : int -> 'a list -> 'a list 
  (* delete a prefix of length i from a sequence *)

val split_at : int -> 'a list -> 'a list * 'a list 
  (* split a sequence into prefix of length i and the remaining sequence *)

val sub : int -> int -> 'a list -> 'a list 
  (* extract a sequence from a sequence
   * the integer arguments are start index and _end_ index
   *)

val take_while : ('a -> bool) -> 'a list -> 'a list 
  (* extract the maximal prefix of elements satisfying the given predicate *)

val drop_while : ('a -> bool) -> 'a list -> 'a list 
  (* delete the maximal prefix of elements satisfying the given predicate *)

val split_while : ('a -> bool) -> 'a list -> 'a list * 'a list
  (* split a sequence into the maximal prefix of elements satisfying
   * the given predicate, and the remaining sequence *)


(** View *)
(** Vues *)
(** Vistas *)


val partition : ('a -> bool) -> 'a list -> 'a list * 'a list 
  (* separate the elements of a sequence into those that satisfy
   * the given predicate and those that do not *)

val left_view : 'a list -> 'a * 'a list 
  (* separate a sequence into its first element and the remaining sequence *)

val right_view : 'a list -> 'a list * 'a 
  (* separate a sequence into its last element and the remaining sequence *)

val left_head : 'a list -> 'a 
  (* return the first element of a sequence *)

val left_tail : 'a list -> 'a list 
  (* delete the first element of a sequence *)

val right_head : 'a list -> 'a 
  (* return the last element of a sequence *)

val right_tail : 'a list -> 'a list 
  (* delete the last element of a sequence *)


(** Observateurs *)
(** Observadores *)
(** Observers **)


val is_empty : 'a list -> bool 
  (* return true if the sequence is empty and false otherwise *)

val size : 'a list -> int 
  (* return the length of a sequence *)

val exists : ('a -> bool) -> 'a list -> bool 
  (* return true if there is at least on element of a sequence 
   * that satisfies the given predicate
   *)

val for_all : ('a -> bool) -> 'a list -> bool 
  (* return true if all elements of a sequence satisfy the given predicate *)

val look_for : ('a -> bool) -> 'a list -> 'a 
  (* return the first element of a sequence satisfying a given predicate *)


(** Reversals *)      
(** Renversements *)
(** Inversiones *)


val reverse : 'a list -> 'a list 
  (* reverse the order of a sequence *)

val append_reverse : 'a list -> 'a list -> 'a list
  (* append a sequence to a reversed sequence
   * append_reverse a b = a @ (reverse b)
   *)

val reverse_append : 'a list -> 'a list -> 'a list
  (* reverse and append a sequence to another 
   * reverse_append a b = (reverse a) @ b
   *)

val reverse_map : ('a -> 'b) -> 'a list -> 'b list 
  (* apply a function and reverse the result
   * reverse_map f x = reverse (map f x) 
   *)

val reverse_map_append :  ('a -> 'b) -> 'b list -> 'a list -> 'b list
  (* apply a function, reverse the result and append it 
   * reverse_map_append f a b = (reverse (map f b)) @ a
   *)

val reverse_take : int -> 'a list -> 'a list
  (* extract a prefix of length i from a sequence and reverse it 
   * reverse_take n a = reverse (take n a)
   *)

val reverse_unzip : ('a * 'b) list -> 'a list * 'b list
  (* transpose a sequence of pairs into a pair of reversed sequences 
   * let (a', b') = unzip c
   * reverse_unzip c = (reverse a', reverse b')
   *)


(** Traversals *)
(** Parcours *)
(** Recorridos *)


val iter : ('a -> unit) -> 'a list -> unit 
  (* apply a function to all elements of a sequence form left to right *)

val fold_right : ('a -> 'b -> 'b) -> 'b -> 'a list -> 'b
  (* combine all the elements of a sequence into a single value
   * given a right-associative combining function and an initial value
   * fold_right (+) e [a, b, ..., z] = a + (b + (... + (z + e)))
   *) 

val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a 
  (* combine all the elements of a sequence into a single value
   * given a left-associative combinition function and an initial value
   * fold_left (+) e [a, b, ..., z] = ((((e + a) + b) + ... ) + z)
   *)

val fold : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a 
  (* combines all the elements in a collection to a single value
   * given a combining funciton and an initial value
   * processes the elements in an unspecified order
   *)

(** Miscellaneous *)
(** Divers *)
(** Micelaneos *)


val bind : 'a list -> ('a -> 'b list) -> 'b list 
  (* apply a sequence producing function to every element of a sequence
   * and flatten the result *)

val concat_map : ('a -> 'b list) -> 'a list -> 'b list 
  (* apply a sequence producing function to every element of a sequence
   * and flatten the result *)


(** Index based operations *)
(** Operations indices *)
(** Operaciones con indices *)


val map_with_index : (int -> 'a -> 'b) -> 'a list -> 'b list 
  (* see map *)

val iter_with_index : (int -> 'a -> 'b) -> 'a list -> unit 
  (* see iter *)

val look_for_with_index : (int -> 'a -> bool) -> 'a list -> 'a
  (* see find *)

val filter_with_index : (int -> 'a -> bool) -> 'a list -> 'a list 
  (* see filter *)

val fold_right_with_index : (int -> 'a -> 'b -> 'b) -> 'b -> 'a list -> 'b
  (* see fold_right *)

val fold_left_with_index : (int -> 'a -> 'b -> 'a) -> 'a -> 'b list -> 'a
  (* see fold_left *)

val fill : 'a list -> int -> int -> 'a -> 'a list 
  (* fill a sequence from indexes start to end with a given element *)

val blit : 'a list -> int -> 'a list -> int -> int -> 'a list 


(** Multiple lists operations **)


val zip : 'a list -> 'b list -> ('a * 'b) list 
  (* combine two sequences into a sequence of pairs *)

val unsafe_zip : 'a list -> 'b list -> ('a * 'b) list

val unzip : ('a * 'b) list -> 'a list * 'b list 
  (* transpose a sequence of pairs into a pair of sequences *)

val iter2 : ('a -> 'b -> 'c) -> 'a list -> 'b list -> unit 

val map2 : ('a -> 'b -> 'c) -> 'a list -> 'b list -> 'c list 

val reverse_map2 : ('a -> 'b -> 'c) -> 'a list -> 'b list -> 'c list 

val fold_left2 : ('a -> 'b -> 'c -> 'a) -> 'a -> 'b list -> 'c list -> 'a

val fold_right2 : ('a -> 'b -> 'c -> 'c) -> 'c -> 'a list -> 'b list -> 'c

val for_all2 : ('a -> 'b -> bool) -> 'a list -> 'b list -> bool 

val exists2 : ('a -> 'b -> bool) -> 'a list -> 'b list -> bool 


(** Aliases **)


val empty : unit -> 'a list
val append : 'a list -> 'a list -> 'a list
val (@) : 'a list -> 'a list -> 'a list


(** Collections **)


(** Categorical operations **)


val injection : 'a -> 'a list 

val map : ('a -> 'b) -> 'a list -> 'b list 

val plus : 'a list -> 'a list -> 'a list 

val zero : unit -> 'a list

val join : 'a list list -> 'a list 

val make : int -> 'a -> 'a list 

val from_list : 'a list -> 'a list

val to_list : 'a list -> 'a list


(** Constructors **)

val insert : 'a -> 'a list -> 'a list 
  (* add an element to a collection *)

val insert_list : 'a list -> 'a list -> 'a list 
  (* add a list of elements to a collection *)

val union : 'a list -> 'a list -> 'a list 
  (* merge two collections *)


(** Deletions **)


val delete : 'a -> 'a list -> 'a list 
  (* delete a single occurence of the given element from a collection *)

val delete_q : 'a -> 'a list -> 'a list 
  (* delete a single occurence of the given element form a collection
   * uses physical equality
   *)

val delete_all : 'a -> 'a list -> 'a list 
  (* delete all occurences of the given element from a collection *)

val delete_all_q : 'a -> 'a list -> 'a list 
  (* delete all occurences of the given element from a collection
   * uses physical equality
   *)

val delete_min : 'a list -> 'a list
  (* delete the minimun element of a collection *)

val delete_max : 'a list -> 'a list
  (* delete the maximum element of a collection *)


(** Observers **)


val is_empty : 'a list -> bool 

val size : 'a list -> int 

val member : 'a -> 'a list -> bool 
  (* test whether the given element is in a collection *)

val member_q : 'a -> 'a list -> bool 
  (* test whether the given element is in a collection
   * uses physical equality
   *)

val count : 'a -> 'a list -> int 
  (* count how many copies of the given element are in a collection *)

val count_q : 'a -> 'a list -> int 
  (* count how many copies of the given element are in a collection
   * uses physical equality
   *)

val min_element : 'a list -> 'a
  (* return the minimum element of a collection *)

val max_element : 'a list -> 'a
  (* return the maximum element of a collection *)

val min_view : 'a list -> ('a * 'a list)
  (* separate a collection into its minimum element 
   * and the remaining collection *)

val max_view : 'a list -> ('a * 'a list)
  (* separate a collection into its maximum element
   * and the remaining collection *)


(** Other operations **)


val exists : ('a -> bool) -> 'a list -> bool 

val for_all : ('a -> bool) -> 'a list -> bool 

val filter : ('a -> bool) -> 'a list -> 'a list 

val partition : ('a -> bool) -> 'a list -> 'a list * 'a list 

val fold : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a 
  (* combines all the elements in a collection to a single value
   * given a combining funciton and an initial value
   * processes the elements in an unspecified order
   *)

val bind : 'a list -> ('a -> 'b list) -> 'b list 


(** Sets **)


val injection_set : 'a -> 'a list

val map_set : ('a -> 'b) -> 'a list -> 'b list

val plus_set : 'a list -> 'a list -> 'a list 

val join_set : 'a list list -> 'a list 

val zero_set : unit -> 'a list

(** Observers **)

val khi_set : 'a list -> 'a -> bool
  (* return the caracteristic function of a set *)

val from_list_set : 'a list -> 'a list 

val is_empty_set : 'a list -> bool

val size_set : 'a list -> int

val member_set : 'a -> 'a list -> bool 

val count_set : 'a -> 'a list -> int 

val filter_set : ('a -> bool) -> 'a list -> 'a list 

val partition_set : ('a -> bool) -> 'a list -> 'a list * 'a list 

val union_set : 'a list -> 'a list -> 'a list 

val insert_set : 'a -> 'a list -> 'a list 

val insert_list_set : 'a list -> 'a list -> 'a list 

val delete_set : 'a -> 'a list -> 'a list

val delete_list_set : 'a list -> 'a list -> 'a list

(** Two sets operations **)

val intersection_set : 'a list -> 'a list -> 'a list 
  (* computes the intersection of two sets *)

val difference_set : 'a list -> 'a list -> 'a list 
  (* computes the differece of two sets *)

val subset_set : 'a list -> 'a list -> bool 
  (* test whether the first set is a subset of the second *)


(** Associative collections **)


type ('k, 'a) assoc = ('k * 'a) list

(** Monadic operations **)

val injection_assoc : 'k -> 'a -> ('k * 'a) list
  (* create an associative collection with a single bindind *)

val map_assoc : ('a -> 'b) -> ('k * 'a) list -> ('k * 'b) list 

val bind_assoc : ('k * 'a) list -> ('a -> ('k * 'b) list) -> ('k * 'b) list 

val zero_assoc : unit -> ('k * 'a)  list
  (* the empty associative collection *)

val plus_assoc : ('k * 'a) list -> ('k * 'a) list -> ('k * 'a) list 
  (* merge two associative collections *)


(** Constructors **)


val from_list_assoc : ('k * 'a) list -> ('k * 'a) list
  (* convert a list of bindings to an associative collection *)

val insert_assoc : 'k -> 'a -> ('a * 'k) list -> ('a * 'k) list 
  (* add a binding to an associative collection *)

val insert_list_assoc : 'k list -> 'k list -> 'k list 
  (* add a list of bindings to an associative collection *)


(** Observers **)


val is_empty_assoc : ('k * 'a) list -> bool
  (* test whether the associative collection is empty *)

val size_assoc : ('k * 'a) list -> int
  (* return the number of bindings in the associative collection *)

val to_list : ('k * 'a) list -> ('k * 'a) list
  (* convert an associative collection to a list of bindings *)

val member_assoc : ('k * 'a) list -> 'k -> bool 
  (* test whether the given key is bound in the associative collection *)

val member_q_assoc : ('k * 'a) list -> 'k -> bool 
  (* test whether the given key is bound in the associative collection 
   * uses physical equality
   *)

val count_assoc : ('k * 'a) list -> 'k -> int 
  (* return the number of bindings with a given key *)

val count_q_assoc : ('k * 'a) list -> 'k -> int 
  (* return the number of bindings with a given key 
   * uses physical equality
   *)

val lookup_assoc : ('k * 'a) list -> 'k -> 'a 
  (* find the first element associated with a given key *)

val lookup_q_assoc : ('k * 'a) list -> 'k -> 'a 
  (* find the first element associated with a given key
   * uses physical equality
   *)

val lookup_all_assoc : ('k * 'a) list -> 'k -> 'a list 
  (* find all the elements associated with a given key *)

val lookup_all_q_assoc : ('k * 'a) list -> 'k -> 'a list 
  (* find all the elements associated with a given key 
   * uses physical equality
   *)

val elements_assoc : ('k * 'a) list -> 'a list 
  (* return all elements in a given association list *)


(** Deletions **)

val delete_assoc : ('k * 'a) list -> 'k -> ('k * 'a) list 
  (* delete the first binding with a given key *)

val delete_q_assoc : ('k * 'a) list -> 'k -> ('k * 'a) list 
  (* delete the first binding with a given key
   * uses physical equality
   *)

val delete_all_assoc : ('k * 'a) list -> 'k -> ('k * 'a) list 
  (* delete all bindings with a given key *)

val delete_all_q_assoc : ('k * 'a) list -> 'k -> ('k * 'a) list 
  (* delete all bindings with a give key
   * uses physical equality *)

val delete_list_assoc : ('k * 'a) list -> 'k list -> ('k * 'a) list
  (* delete all bindings of list of keys *)


(** Modifiers **)


val adjust_assoc : ('a -> 'a) -> 'k -> ('k * 'a) list -> ('k * 'a) list

val adjust_all_assoc : ('a -> 'a) -> 'k -> ('k * 'a) list -> ('k * 'a) list


(** Maps and folds **)

val fold_assoc : ('a -> 'b -> 'a) -> 'a -> ('k * 'b) list -> 'a

val fold_with_key_assoc : ('a -> 'k -> 'b -> 'a) -> 'a -> ('k * 'b) list -> 'a
  
val map_with_key_assoc : ('k -> 'a -> 'b) -> ('k * 'a) list -> ('k * 'b) list


(** Filters and partitions **)


val filter_elements_assoc : ('a -> bool) -> ('k * 'a) list -> ('k * 'a) list

val filter_keys_assoc : ('k -> bool) -> ('k * 'a) list -> ('k * 'a) list

val partition_elements_assoc :
  ('a -> bool) -> ('k * 'a) list -> ('k * 'a) list * ('k * 'a) list 

val partition_keys_assoc :
  ('k -> bool) -> ('k * 'a) list -> ('k * 'a) list * ('k * 'a) list 
