(* Diego Olivier Fernandez Pons
 * Universit Pierre et Marie Curie, Paris VI
 * diego-olivier.fernandez-pons@cicrp.jussieu.fr
 * 
 * Baire Pervasives
 * 
 *)

(* library exceptions *)
(* exceptions utilises par la librarie *)
(* excepciones usadas por la librera *)

exception Empty
exception Found
exception Out_of_bounds of int
exception Not_working of string

(* Caml predefined exceptions used by the library
 * Exceptions prdfinies de Caml utilises par la librairie
 * Exepciones predefinidas en Caml que usa la librera
 *
 * exception Not_found
 * 
 *)

(* zippers exceptions *)
(* exceptions utilises par les zippers *)
(* exepciones usadas por los zippers *)

exception Bottom
exception Top

(* error handling *)
(* traitement des erreurs *)
(* manejo de los errores *)

type error =
  | Unknown
  | UnexpectedPattern
  | Negative
  | OutOfBounds
  | SameLength
  | Special of string

let error_message_fr = function
  | Unknown -> "erreur d'origine inconnue"
  | UnexpectedPattern -> 
      "cas inattendu, erreur dans la librairie ou dans son usage"
  | Negative -> "argument ngatif"
  | OutOfBounds -> "valeur en dehors des bornes admissibles"
  | SameLength -> "les arguments doivent tre de mme longueur"
  | Special message -> "message special : " ^ message

let error_message_en = function
  | Unknown -> "unknown error"
  | UnexpectedPattern -> 
      "unexpected pattern, error in the library or in it usage"
  | Negative -> "negative argument"
  | OutOfBounds -> "out of bounds value"
  | SameLength -> "arguments must have the same length"
  | Special message -> "special message : " ^ message

let error_message_es = function
  | Unknown -> "error de origen desconocido"
  | UnexpectedPattern -> "caso inesperado, error en la librera o en su uso"
  | Negative -> "argumento negativo"
  | OutOfBounds -> "argumento fuera de lmites"
  | SameLength -> "los argumentos tienen que ser de mismo tamao"
  | Special message -> "mensaje especial : " ^ message

let error_message_de = function
  | Unknown -> "Unbekannter Irrtum"
  | UnexpectedPattern -> "Unbekannter Fall"
  | Negative -> "Negatives Argument"
  | OutOfBounds -> "aus Grenzen heraus"
  | SameLength -> "Die Argumente mssen die selbe Lnge haben"
  | Special message -> "Spezialeil Botschaft"

let error_message = error_message_fr

let invalid_argument = fun module_name function_name error_type ->
  raise (Invalid_argument 
	   (module_name ^ "." 
	    ^ function_name ^ " : " 
	    ^ error_message error_type))

(* lazy aliases *)
(* fonctions synonymes pour les valeurs dfres *)

let force = Lazy.force_val
let strict = Lazy.lazy_from_val

(* predefined data types *)
(* types prdfinis *)
(* tipos predefinidos *)

type 'a tree = Leaf | Node of 'a tree * 'a * 'a tree

(* predefined functions *)
(* fonctions prdfinies *)
(* funciones predefinidas *)

let id = function x -> x

let sym f = fun x y -> f y x

(* predefined modules *)
(* modules prdfinis *)
(* mdulos predefinidos *)

module Integer = struct
  type t = int
  let compare : int -> int -> int = Pervasives.compare
end

module Real = struct
  type t = float
  let compare : float -> float -> int = Pervasives.compare
end
