type 'a graph

type node = int
type 'a edge = node * 'a * node

val moduleName : string

(** Oprations catgorielles **)
(** Categorial operiations **)
(** Operaciones de categora **)

val injection : node -> 'a -> node -> 'a graph
val map : ('a edge -> 'b edge) -> 'a graph -> 'b graph
val plus : 'a graph -> 'a graph -> 'a graph
val zero : unit -> 'a graph

val from_list : 'a edge list -> 'a graph
val to_list : 'a graph -> 'a edge list

(** Oprations binaires **)
(** Binairy operations **)
(** Operaciones binrias **)

val union        : 'a graph -> 'a graph -> 'a graph
val intersection : 'a graph -> 'a graph -> 'a graph
val difference   : 'a graph -> 'a graph -> 'a graph
val symetric_difference : 'a graph -> 'a graph -> 'a graph

(** Views **)
(** Vues **)
(** Vistas **)

(** Observateurs **)
(** Observers **)
(** Observadores **)

val is_empty : 'a graph -> bool

val nodes : 'a graph -> int
val edges : 'a graph -> int

val labels_list  : 'a graph -> 'a list
val nodes_list   : 'a graph -> node list
val edges_list   : 'a graph -> 'a edge list
val degrees_list : 'a graph -> (node * int * int) list

(** Miscellaneous **)
(** Divers **)
(** Micelaneos **)

val union_with        : ('a -> 'a -> 'a) -> 'a graph -> 'a graph -> 'a graph
val intersection_with : ('a -> 'a -> 'a) -> 'a graph -> 'a graph -> 'a graph

(** Traversals **)
(** Parcours **)
(** Recorridos **)


(** Operations on nodes **)
(** Oprations sur les noeuds **)
(** Operaciones sobre los nudos **)


val successors   : node -> 'a graph -> node list
val predecessors : node -> 'a graph -> node list
val adjacents    : node -> 'a graph -> node list * node list

val exterior_edges  : node -> 'a graph -> 'a edge list
val interior_edges  : node -> 'a graph -> 'a edge list
val adjacent_edges  : node -> 'a graph -> 'a edge list * 'a edge list

val exterior_degree : node -> 'a graph -> int
val interior_degree : node -> 'a graph -> int
val degree          : node -> 'a graph -> int * int

val member_node   : node -> 'a graph -> bool

val exists_node   : (node -> bool) -> 'a graph -> bool
val for_all_nodes : (node -> bool) -> 'a graph -> bool

val iter_nodes : (node -> unit) -> 'a graph -> unit


(** Operations on edges **)
(** Operations sur les arcs **)
(** Operaciones sobre los arcos **)


val insert_edge : node -> 'a -> node -> 'a graph -> 'a graph

val delete_edge : node -> node -> 'a graph -> 'a graph

val member_edge : node -> node -> 'a graph -> bool

val look_for_edge : (node -> 'a -> node -> bool) -> 'a graph -> 'a edge
