type ('a, 'b) symbol =
  | Terminal of 'a
  | NonTerminal of 'b

type nt = E | S | S' | T

type lex = (char, nt) symbol

let inj = function
  | 'E' -> NonTerminal E
  | 'S' -> NonTerminal S
  | 'Z' -> NonTerminal S'
  | 'T' -> NonTerminal T
  | other -> Terminal other

let epsilon =
  [
    (0, 1);
    (1, 4);
    (4, 5);
    (5, 7);
    (7, 8);
    (8, 7);
    (10, 14);
    (7, 12);
    (12, 14);
    (14, 15);
    (14, 17);
    (18, 7)
  ];;

let no_epsilon = 
  [
    (1, 'S', 2);
    (2, '#', 3);
    (5, 'E', 6);
    (8, 'E', 9);
    (9, '-', 10);
    (10, 'T', 11);
    (12, 'T', 13);
    (15, 'n', 16);
    (17, '(', 18);
    (18, 'E', 19);
    (19, ')', 20)
  ]

let rec none = function
  | [] -> []
  | (x, y) :: tail -> (x, None, y) :: none tail

let rec some f = function
  | [] -> []
  | (x, v, y) :: tail -> (x, Some (f v), y) :: some f tail

let build = fun epsilon no_epsilon ->
  from_list ((none epsilon) @ (some inj no_epsilon))
  
