type 'a graph

type node = int
type 'a edge = node * 'a * node

val moduleName : string

(** Oprations catgorielles **)
(** Categorial operiations **)
(** Operaciones de categora **)

val injection : node -> 'a -> node -> 'a graph
val zero : unit -> 'a graph

val from_list : 'a edge list -> 'a graph
val to_list   : 'a graph -> 'a edge list

(** Observateurs **)
(** Observers **)
(** Observadores **)

val is_empty : 'a graph -> bool

val edges : 'a graph -> int
val nodes : 'a graph -> int

val labels_list  : 'a graph -> 'a list
val nodes_list   : 'a graph -> node list
val edges_list   : 'a graph -> 'a edge list

(** Miscellaneous **)
(** Divers **)
(** Micelaneos **)

val next : node -> 'a -> 'a graph -> node
val next_list : node -> 'a list -> 'a graph -> node


(** Operations on nodes **)
(** Oprations sur les noeuds **)
(** Operaciones sobre los nudos **)

val delete_node : node -> 'a graph -> 'a graph

val delete_node_list : node list -> 'a graph -> 'a graph

val successors   : node -> 'a graph -> node list
val predecessors : node -> 'a graph -> node list
val adjacents    : node -> 'a graph -> node list * node list

val exterior_edges  : node -> 'a graph -> 'a edge list
val interior_edges  : node -> 'a graph -> 'a edge list
val adjacent_edges  : node -> 'a graph -> 'a edge list * 'a edge list

val exterior_degree : node -> 'a graph -> int
val interior_degree : node -> 'a graph -> int
val degree          : node -> 'a graph -> int * int

val member_node   : node -> 'a graph -> bool

val exists_node   : (node -> bool) -> 'a graph -> bool
val for_all_nodes : (node -> bool) -> 'a graph -> bool

val iter_nodes : (node -> unit) -> 'a graph -> unit

val fold_nodes       : (node -> 'b -> 'b) -> 'a graph -> 'b -> 'b
val fold_right_nodes : (node -> 'b -> 'b) -> 'a graph -> 'b -> 'b
val fold_left_nodes  : ('b -> node -> 'b) -> 'b -> 'a graph -> 'b


(** Operations on edges **)
(** Operations sur les arcs **)
(** Operaciones sobre los arcos **)


val insert_edge : node -> 'a -> node -> 'a graph -> 'a graph
val insert_edge_list : 'a edge list -> 'a graph -> 'a graph

val member_edge : node -> node -> 'a graph -> bool

val look_for_edge : (node -> 'a -> node -> bool) -> 'a graph -> 'a edge

val choose_edge : 'a graph -> 'a edge
