type 'a graph

type node = int
type 'a edge = node * 'a * node

val moduleName : string

(** Oprations catgorielles **)
(** Categorial operiations **)
(** Operaciones de categora **)

val injection : node -> 'a -> node -> 'a graph
val map : ('a edge -> 'b edge) -> 'a graph -> 'b graph
val plus : 'a graph -> 'a graph -> 'a graph
val zero : unit -> 'a graph

val from_list : 'a edge list -> 'a graph
val to_list : 'a graph -> 'a edge list

(** Oprations binaires **)
(** Binairy operations **)
(** Operaciones binrias **)

val union        : 'a graph -> 'a graph -> 'a graph
val intersection : 'a graph -> 'a graph -> 'a graph

(** Observateurs **)
(** Observers **)
(** Observadores **)

val is_empty : 'a graph -> bool

val nodes : 'a graph -> int
val edges : 'a graph -> int

val labels_list  : 'a graph -> 'a list
val nodes_list   : 'a graph -> node list
val edges_list   : 'a graph -> 'a edge list
val degrees_list : 'a graph -> (node * int * int) list

(** Traversals **)
(** Parcours **)
(** Recorridos **)

val breadth_first_list  : 'a graph -> node -> node list
val depth_first_list    : 'a graph -> node -> node list

val topological_list    : 'a graph -> node list             (* [Cyclic] *)
val topological_list'   : 'a graph -> node list             (* [Cyclic] *)

val breadth_first_list' : 'a graph -> node -> 'a edge list
val depth_first_list'   : 'a graph -> node -> 'a edge list

val breadth_first_fold  : (node -> 'b -> 'b) -> 
  'b -> 'a graph -> node -> 'b
val breadth_first_fold' : (node -> 'a -> node -> 'b -> 'b) -> 
  'b -> 'a graph -> node -> 'b

val depth_first_fold  : (node -> 'b -> 'b) -> 
  'b -> 'a graph -> node -> 'b
val depth_first_fold' : (node -> 'a -> node -> 'b -> 'b) ->
  'b -> 'a graph -> node -> 'b

val topological_fold  : (node -> 'b -> 'b) -> 'b -> 'a graph -> 'b 
  (* [Cyclic] *)


(** Operations on nodes **)
(** Oprations sur les noeuds **)
(** Operaciones sobre los nudos **)


val new_node  : 'a graph -> node
val new_nodes : int -> 'a graph -> node list

val insert_node : node -> 'a graph -> 'a graph
val delete_node : node -> 'a graph -> 'a graph

val insert_node_list : node list -> 'a graph -> 'a graph
val delete_node_list : node list -> 'a graph -> 'a graph

val insert_node_raise : node -> 'a graph -> 'a graph        (* [Found] *)
val delete_node_raise : node -> 'a graph -> 'a graph        (* [Not_found] *)

val successors   : node -> 'a graph -> node list
val predecessors : node -> 'a graph -> node list
val adjacents    : node -> 'a graph -> node list * node list

val exterior_edges  : node -> 'a graph -> 'a edge list
val interior_edges  : node -> 'a graph -> 'a edge list
val adjacent_edges  : node -> 'a graph -> 'a edge list * 'a edge list

val exterior_degree : node -> 'a graph -> int
val interior_degree : node -> 'a graph -> int
val degree          : node -> 'a graph -> int * int

val filter_nodes : (node -> bool) -> 'a graph -> 'a graph
val filter_nodes_lt : node -> 'a graph -> 'a graph
val filter_nodes_le : node -> 'a graph -> 'a graph
val filter_nodes_gt : node -> 'a graph -> 'a graph
val filter_nodes_ge : node -> 'a graph -> 'a graph

val member_node    : node -> 'a graph -> bool
val look_for_node  : (node -> bool) -> 'a graph -> node

val next_node     : node -> 'a graph -> node
val previous_node : node -> 'a graph -> node
val ceil_node     : node -> 'a graph -> node
val floor_node    : node -> 'a graph -> node

val choose_node : 'a graph -> node
val min_node    : 'a graph -> node
val max_node    : 'a graph -> node

val exists_node   : (node -> bool) -> 'a graph -> bool
val for_all_nodes : (node -> bool) -> 'a graph -> bool

val iter_nodes : (node -> unit) -> 'a graph -> unit

val fold_nodes       : (node -> 'b -> 'b) -> 'a graph -> 'b -> 'b
val fold_right_nodes : (node -> 'b -> 'b) -> 'a graph -> 'b -> 'b
val fold_left_nodes  : ('b -> node -> 'b) -> 'b -> 'a graph -> 'b


(** Operations on edges **)
(** Operations sur les arcs **)
(** Operaciones sobre los arcos **)


val insert_edge : node -> 'a -> node -> 'a graph -> 'a graph
val insert_edge_list : 'a edge list -> 'a graph -> 'a graph

val delete_edge : node -> node -> 'a graph -> 'a graph
val delete_edge_list : (node * node) list -> 'a graph -> 'a graph

val insert_edge_raise : node -> 'a -> node -> 'a graph -> 'a graph
(* [Not_found] | [Found] *)
val delete_edge_raise : node -> node -> 'a graph -> 'a graph (* [Not_found] *)

val edge_label : node -> node -> 'a graph -> 'a
val map_edge_labels : ('a -> 'b) -> 'a graph -> 'b graph

val member_edge : node -> node -> 'a graph -> bool

val look_for_edge : (node -> 'a -> node -> bool) -> 'a graph -> 'a edge

val choose_edge : 'a graph -> 'a edge (* [Empty] *)

val exists_edge   : (node -> 'a -> node -> bool) -> 'a graph -> bool
val for_all_edges : (node -> 'a -> node -> bool) -> 'a graph -> bool

val iter_edges : (node -> 'a -> node -> unit) -> 'a graph -> unit

val fold_edges       : (node -> 'a -> node -> 'b -> 'b) -> 'a graph -> 'b -> 'b
val fold_right_edges : (node -> 'a -> node -> 'b -> 'b) -> 'a graph -> 'b -> 'b
val fold_left_edges  : ('b -> node -> 'a -> node -> 'b) -> 'b -> 'a graph -> 'b
