type node = int

type 'a graph

val moduleName : string

(** Oprations catgorielles *)
(** Categorial operiations *)
(** Operaciones de categora *)

val injection : node -> 'a option -> node -> 'a graph
val map : (node * 'a option * node -> node * 'b option * node) -> 
  'a graph -> 'b graph

val zero : unit -> 'a graph

val from_list : (node * 'a option * node) list -> 'a graph
val to_list : 'a graph -> (node * 'a option * node) list

(** Oprations binaires *)
(** Binairy operations *)
(** Operaciones binrias *)






(** Views *)
(** Vues *)
(** Vistas *)





(** Observateurs *)
(** Observers *)
(** Observadores *)

val is_empty : 'a graph -> bool

val nodes : 'a graph -> int
val edges : 'a graph -> int

val labels_list  : 'a graph -> 'a option list
val nodes_list   : 'a graph -> node list
val edges_list   : 'a graph -> (node * 'a option * node) list
val degrees_list : 'a graph -> (node * int * int) list

(** Miscellaneous *)
(** Divers *)
(** Micelaneos *)

val next : node -> 'a -> 'a graph -> node list (* [Not_found] *)

val next_no_epsilon_loop : node -> 'a -> 'a graph -> node list 
  (* [Not_found] | [Stack_overflow] *)

val next_list_df : node -> 'a list -> 'a graph -> node list (* [Not_found] *)
val next_list_bf : node -> 'a list -> 'a graph -> node list (* [Not_found] *)

(** Traversals *)
(** Parcours *)
(** Recorridos *)

val breadth_first_list  : 'a graph -> node -> node list
val depth_first_list    : 'a graph -> node -> node list

val topological_list    : 'a graph -> node list               (* [Cyclic] *)

val breadth_first_list' : 'a graph -> node -> (node * 'a option * node) list
val depth_first_list'   : 'a graph -> node -> (node * 'a option * node) list

val breadth_first_fold   : (node -> 'b -> 'b) -> 
  'b -> 'a graph -> node -> 'b
val breadth_first_fold'  : (node -> 'a option -> node -> 'b -> 'b) -> 
  'b -> 'a graph -> node -> 'b

val depth_first_fold   : (node -> 'b -> 'b) -> 
  'b -> 'a graph -> node -> 'b
val depth_first_fold'  : (node -> 'a option -> node -> 'b -> 'b) ->
  'b -> 'a graph -> node -> 'b

val topological_fold   : (node -> 'b -> 'b) -> 'b -> 'a graph -> 'b
  (* [Cyclic] *)


(** Operations on nodes *)
(** Oprations sur les noeuds *)
(** Operaciones sobre los nudos *)


val new_node  : 'a graph -> node
val new_nodes : int -> 'a graph -> node list

val insert_node : node -> 'a graph -> 'a graph
val delete_node : node -> 'a graph -> 'a graph

val insert_node_list : node list -> 'a graph -> 'a graph
val delete_node_list : node list -> 'a graph -> 'a graph

val insert_node_raise : node -> 'a graph -> 'a graph (* [Found] *)
val delete_node_raise : node -> 'a graph -> 'a graph (* [Not_found] *)

val successors   : node -> 'a graph -> node list
val predecessors : node -> 'a graph -> node list
val adjacents    : node -> 'a graph -> node list * node list

val exterior_edges  : node -> 'a graph -> (node * 'a option * node) list
val interior_edges  : node -> 'a graph -> (node * 'a option * node) list
val adjacent_edges  : node -> 'a graph -> (node * 'a option * node) list * 
  (node * 'a option * node) list

val exterior_degree : node -> 'a graph -> int
val interior_degree : node -> 'a graph -> int
val degree          : node -> 'a graph -> int * int

val member_node   : node -> 'a graph -> bool
val look_for_node : (node -> bool) -> 'a graph -> node

val next_node     : node -> 'a graph -> node
val previous_node : node -> 'a graph -> node
val ceil_node     : node -> 'a graph -> node
val floor_node    : node -> 'a graph -> node

val choose_node : 'a graph -> node
val min_node    : 'a graph -> node
val max_node    : 'a graph -> node

val exists_node   : (node -> bool) -> 'a graph -> bool
val for_all_nodes : (node -> bool) -> 'a graph -> bool

val iter_nodes : (node -> unit) -> 'a graph -> unit

val fold_nodes       : (node -> 'b -> 'b) -> 'a graph -> 'b -> 'b
val fold_right_nodes : (node -> 'b -> 'b) -> 'a graph -> 'b -> 'b
val fold_left_nodes  : ('b -> node -> 'b) -> 'b -> 'a graph -> 'b


(** Operations on edges *)
(** Operations sur les arcs *)
(** Operaciones sobre los arcos *)


val insert_edge : node -> 'a option -> node -> 'a graph -> 'a graph 
  (* [Not_found] *)

val insert_edge_list : (node * 'a option * node) list -> 'a graph -> 'a graph

val delete_edge : node -> node -> 'a graph -> 'a graph
val delete_edge_list : (node * node) list -> 'a graph -> 'a graph

val insert_edge_raise : node -> 'a option -> node -> 'a graph -> 'a graph
(* [Not_found] | [Found] *)

val delete_edge_raise : node -> node -> 'a graph -> 'a graph (* [Not_found] *)

val edge_label : node -> node -> 'a graph -> 'a option

val member_edge : node -> node -> 'a graph -> bool

val exists_edge   : (node -> 'a option -> node -> bool) -> 'a graph -> bool
val for_all_edges : (node -> 'a option -> node -> bool) -> 'a graph -> bool

val iter_edges : (node -> 'a option -> node -> unit) -> 'a graph -> unit

val fold_edges       : (node -> 'a option -> node -> 'b -> 'b) -> 
  'a graph -> 'b -> 'b

val fold_right_edges : (node -> 'a option  -> node -> 'b -> 'b) -> 
  'a graph -> 'b -> 'b

val fold_left_edges  : ('b -> node -> 'a option -> node -> 'b) -> 
  'b -> 'a graph -> 'b


(* to be used with care *)
(* oprations  utiliser avec prcaution *)
(* a usar con cautela *)

val delete_node_unsafe : node -> 'a graph -> 'a graph
val delete_node_unsafe_raise : node -> 'a graph -> 'a graph

