(*
 * Ranked AVL heaps
 *)

open BairePervasives

(* Ce code est bas sur le rapport de recherche suivant :
 * 
 * A General Technique for Implementation of Efficient Priority Queues
 * Peter Hyer
 * Department of mathematics and computer science
 * Odense University, Denmark
 *)

module Rlist = SparseRandomAccesList

type 'a tree = E | N of 'a tree * 'a * 'a tree
type 'a pennant = Pennant of 'a * int * 'a tree


(** Utils **)


let root = function Pennant (x, _, _) -> x

let rec tree_rank = function
  | E -> 1
  | N (_, _, r) -> 1 + tree_rank r

let rank = function Pennant (_, r, _) -> r + 1

let link = fun (Pennant (x, r1, t1)) (Pennant (y, r2, t2)) ->
  match compare x y with
    | n when n < 0 -> Pennant (x, r2 + 2, N (t1, y, t2))
    | _ -> Pennant (y, r1 + 2, N (t2, x, t1))

let cut = function
  | E | N (E, _, _) -> raise (Unexpected_pattern "RankedAvlHeap.cut")
  | N (N _ as tree, v, lt) -> (tree, Pennant (v, tree_rank lt, lt))

let left_rotation = function tree ->
  match tree with
    | E | N (_, _, E) -> tree
    | N (l, v, N (rl, rv, rr)) -> N (N (l, v, rl), rv, rr)

let right_rotation = function tree ->
  match tree with
    | E | N (E, _, _) -> tree
    | N (N (ll, lv, lr), v, r) ->
	match compare lv v with
	  | n when n < 0 -> N (ll, lv, N (lr, v, r))
	  | _ -> N (ll, v, N (r, lv, lr))
 
let left_swap = function tree ->
  match tree with
    | E | N (E, _, _) -> tree
    | N (N (ll, lv, lr), v, r) -> N (N (ll, v, r), lv, lr)

let right_swap = function tree ->
  match tree with
    | E | N (_, _, E) -> tree
    | N (l, v, N (rl, rv, rr)) -> N (l, rv, N (rr, v, rl))

(* Un lment crucial dans le dveloppement de cette structure de donnes
 * est la sparation des oprations sur la strucuture du tas et des
 * oprations d'quilibrage.
 * Les premires se font avec les fonctions [link] et [cut] tandis que les
 * secondes se font au moyen des rotations [left_rotation, right_rotation] 
 * et des changes [left_swap, right_swap]
 *)

let rec 
  link_first_pennant forest = function
    | [] -> (forest, [])
    | (p, p') :: tail -> 
	let linked = link p p' in 
	  (add_pennant linked (rank linked) forest, tail)
and
  add_pennant = fun p rank (forest, tree_list) ->
    let result = ref None in
    let rec add_pennant_rec = fun p k forest ->
      match k with
	| 0 ->
	    (match forest with
	       | [] -> One p
	       | Zero :: tail -> One p :: tail
	       | One p' :: tail -> result := (p, p') ; Zero :: tail
	    )
	| _ ->
	    match forest with
	      | [] -> Zero :: add_pennant_rec p (k - 1) []
	      | head :: tail -> head :: add_pennant_rec p (k - 1) tail
    in
    let new_forest = add_pennant_rec p rank forest in
      link_first_pennant 
      
	
