open BairePervasives

(* Ces listes  concatnation rapide sont crites  partir de files
 * reprsentes par deux listes comme dans le module SimpleQueue
 * En vitant l'indirection, nous gagnons en efficacit mais perdons
 * en simplicit. Un foncteur gnrique y rmdiera nonobstant.
 * 
 * La signature gnrique est
 * 'a seq = E | N of 'a * 'a seq queue
 *)

type 'a seq =
  | E
  | N of 'a seq list * 'a * 'a seq list

(** Utils **)

let link = fun a b ->
  match a with
    | E -> b
    | N ([], x, _) -> N ([b], x, [])
    | N (front, x, rear) -> N (front, x, b :: rear)

let rec link_all = fun a b ->
  match a with
    | [] -> raise Empty
    | [x] when b = [] -> x
    | [x] -> link x (link_all (List.reverse b) [])
    | head :: tail -> link head (link_all tail b)

let unit = function x -> N ([], x, [])

let rec map f = function
  | E -> E
  | N (a, x, b) -> N (List.map (map f) a, f x, List.map (map f) b)

let plus = fun a b ->
  match (a, b) with
    | (E, _) -> b
    | (_, E) -> a
    | _ -> link a b

let rec join = function
  | E -> E
  | N ([], x, _) -> x
  | N (a, x, b) ->
      match x with
	| E -> raise (Unexpected_pattern "catenableList.join")
	| N (a', x', b') -> 
	    N (a', x', List.map join b @ List.reverse_map join a @ b')

let zero = function () -> E

let make = fun n x ->
  match n with
    | 0 -> E
    | n -> 
	N (List.make (n / 2) (unit x), x, List.make (n - 1 - n / 2) (unit x))

let rec 
  get_rec = fun n list ->
    match list with
      | [] -> raise (Out_of_bounds n)
      | head :: tail ->
	  try get head n 
	  with Out_of_bounds k -> get_rec k tail
and
  get = fun catlist n ->
    match catlist with
      | E -> raise (Out_of_bounds n)
      | N (_, x, _) when n = 0 -> x
      | N (a, _, b) ->
	  try get_rec (n - 1) a 
	  with Out_of_bounds k -> get_rec k (List.reverse b)

let cons x = function
  | E -> unit x
  | list -> N ([list], x, [])

let snoc = fun list x ->
  match list with
    | E -> unit x
    | N ([], y, _) -> N ([N ([], x, [])], y, [])
    | N (a, y, b) -> N (a, y, N ([], x, []) :: b)

let rec from_list = function
  | [] -> E
  | head :: tail -> N (List.map unit tail, head, [])

let rec init = fun f n ->
  match List.init f n with
    | [] -> E
    | x :: succ -> N (List.map unit succ, x, []) 

let left_view = function
  | E -> raise Empty
  | N ([], x, _) -> (x, E)
  | N (a, x, b) -> (x, link_all a b)

let left_head = function
  | E -> raise Empty
  | N (_, x, _) -> x

let left_tail = function
  | E -> raise Empty
  | N ([], _, _) -> E
  | N (a, _, b)  -> link_all a b

let rec right_head = function
  | E -> raise Empty
  | N ([], x, _) -> x
  | N (front, _, []) -> right_head (List.right_head front)
  | N (_, _, rhead :: _) -> right_head rhead

let is_empty = function 
  | E -> true 
  | _ -> false

let rec size_rec result = function
  | E -> result
  | N (front, _, rear) -> 
      List.fold_left size_rec (List.fold_left size_rec (1 + result) front) rear

let size = function catlist -> size_rec 0 catlist

let rec to_list = function
  | E -> []
  | N (a, x, b) -> x :: List.concat_map to_list a @ 
      List.concat_map to_list (List.reverse b)

