open BairePervasives

type 'a seq = 
  | E 
  | N of int * 'a * 'a seq * 'a seq * 'a seq

let size = function
  | E -> 0
  | N (n, _, _, _, _) -> n

let makeList = fun x a b c -> N (1 + size a + size b + size c, x, a, b, c)

let unit = function x -> N (1, x, E, E, E)

let rec map f = function
  | E -> E
  | N (n, x, fr, r, s) -> N (n, f x, map f fr, map f r, map f s)

let plus = fun a b ->
  if b = E then a else
    match a with
      | E -> b
      | N (n, x, f, r, _) -> N (n + size b, x, f, r, b)

let zero = function () -> E

let rec make_rec x a b = function
  | 1 -> makeList x a b E
  | n when n mod 2 = 0 -> make_rec x (makeList x a a E) b (n / 2)
  | n -> make_rec x (makeList x a a E) (makeList x a b E) ((n - 1) / 2)

let make = fun n x -> make_rec x E E n

let rec get = fun catlist n ->
  match catlist with
    | E -> raise (Out_of_bounds n)
    | N (_, x, _, _, _) when n = 0 -> x
    | N (_, _, f, r, s) ->
	if n <= size f then get f (n - 1)
	else if n <= size f + size r then get r (n - size f - 1)
	else get s (n - size f - size r + 1)

let cons = fun x a -> makeList x a E E

let snoc = fun a x ->
  match a with
    | E -> unit x
    | N (_, y, E, _, _) -> makeList y (makeList x E E E) E E
    | N (_, y, f, r, _) -> makeList y f (makeList x E E r) E 

let rec from_list = function
  | [] -> E
  | head :: tail -> makeList head (from_list tail) E E
      
let rec
  to_list = function
    | E -> []
    | N (_, x, f, r, s) -> 
	x :: to_list f @ to_list_rev (to_list s) r
and
  to_list_rev result = function
    | E -> result
    | N (_, x, f, r, s) -> 
	to_list_rev (to_list r @ to_list_rev (x :: result) f) s

let rec init_rec f n = function
  | k when k = n  -> makeList (f k) E E E
  | k -> makeList (f k) (init_rec f n (k + 1)) E E

let init = fun f n -> init_rec f n 0

(*-- revoir srieusement cette fonction --*)
let rec link_all = fun f r s ->
  match f with
    | E -> s
    | N (_, fx, ff, fr, E) when s = E -> makeList fx ff fr r
    | N (_, fx, ff, fr, E) -> makeList fx ff fr (link_all r s E)
    | N (_, fx, ff, fr, fs) -> makeList fx ff fr (link_all fs r s)

let left_view = function
  | E -> raise Empty
  | N (_, x, E, _, s) -> (x, s)
  | N (_, x, f, r, s) -> (x, link_all f r s)
