type 'a seq
  
val copy : 'a seq -> 'a seq

val unit : 'a -> 'a seq
val map : ('a -> 'b) -> 'a seq -> 'b seq
val plus : 'a seq -> 'a seq -> 'a seq
val join : 'a seq seq -> 'a seq
val bind : 'a seq -> ('a -> 'b seq) -> 'b seq
val zero : 'a seq
val make : int -> 'a -> 'a seq
val get : 'a seq -> int -> 'a
val set : 'a seq -> int -> 'a -> 'a seq

val cons : 'a -> 'a seq -> 'a seq
val snoc : 'a seq -> 'a -> 'a seq
val from_list : 'a list -> 'a seq
val init : int -> (int -> 'a) -> 'a seq

val reverse_onto : 'a seq -> 'a seq -> 'a seq
val reverse_append : 'a seq -> 'a seq -> 'a seq
val reverse : 'a seq -> 'a seq
val reverse_map : ('a -> 'b) -> 'a seq -> 'b seq

val left_view : 'a seq -> 'a * 'a seq
val left_head : 'a seq -> 'a
val left_tail : 'a seq -> 'a seq
val right_view : 'a seq -> 'a seq * 'a
val right_head : 'a seq -> 'a
val right_tail : 'a seq -> 'a seq

val is_empty : 'a seq -> bool
val size : 'a seq -> int
val to_list : 'a seq -> 'a list

val concat_map : ('a -> 'b seq) -> 'a seq -> 'b seq
val fold_right : ('a -> 'b -> 'b) -> 'b -> 'a seq -> 'b
val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b seq -> 'a
val reverse_fold_left : ('a -> 'b -> 'a) -> 'a -> 'b seq -> 'a
val reverse_fold_right : ('a -> 'b -> 'b) -> 'b -> 'a seq -> 'b

val iter : ('a -> 'b) -> 'a seq -> unit
val take : int -> 'a seq -> 'a seq
val drop : int -> 'a seq -> 'a seq
val split_at : int -> 'a seq -> 'a seq * 'a seq
val sub : int -> int -> 'a seq -> 'a seq

val exists : ('a -> bool) -> 'a seq -> bool
val for_all : ('a -> bool) -> 'a seq -> bool
val find : ('a -> bool) -> 'a seq -> 'a
val filter : ('a -> bool) -> 'a seq -> 'a seq
val partition : ('a -> bool) -> 'a seq -> 'a seq * 'a seq

val take_while : ('a -> bool) -> 'a seq -> 'a seq
val drop_while : ('a -> bool) -> 'a seq -> 'a seq
val split_while : ('a -> bool) -> 'a seq -> 'a seq * 'a seq

val map_with_index : (int -> 'a -> 'b) -> 'a seq -> 'b seq
val iter_with_index : (int -> 'a -> 'b) -> 'a seq -> unit
val filter_with_index : (int -> 'a -> bool) -> 'a seq -> 'a seq
val find_with_index : (int -> 'a -> bool) -> 'a seq -> 'a
val fold_right_with_index : (int -> 'a -> 'b -> 'b) -> 'b -> 'a seq -> 'b
val fold_left_with_index : (int -> 'a -> 'b -> 'a) -> 'a -> 'b seq -> 'a
val fill : 'a seq -> int -> int -> 'a -> 'a seq
val blit : 'a seq -> int -> 'a seq -> int -> int -> 'a seq

val unsafe_zip : 'a seq -> 'b seq -> ('a * 'b) seq
val zip : 'a seq -> 'b seq -> ('a * 'b) seq
val unzip : ('a * 'b) seq -> 'a seq * 'b seq

val empty : 'a seq
val append : 'a seq -> 'a seq -> 'a seq

val zero_side : unit -> 'a seq
val set_side : int -> 'a -> 'a seq -> 'a seq
val reverse_side : 'a seq -> 'a seq
val adjust_side : ('a -> 'a) -> 'a seq -> 'a seq
val adjust_with_index_side : (int -> 'a -> 'a) -> 'a seq -> 'a seq
val fill_side : 'a seq -> int -> int -> 'a -> 'a seq

val insert : 'a -> 'a seq -> 'a seq
val union : 'a seq -> 'a seq -> 'a seq
val delete : 'a -> 'a seq -> 'a seq
val delete_q : 'a -> 'a seq -> 'a seq
val delete_all : 'a -> 'a seq -> 'a seq
val delete_all_q : 'a -> 'a seq -> 'a seq
val member : 'a -> 'a seq -> bool
val member_q : 'a -> 'a seq -> bool
val count : 'a -> 'b -> 'a seq -> int
val count_q_rec : 'a -> int -> 'a seq -> int
val fold : ('a -> 'b -> 'a) -> 'a -> 'b seq -> 'a

type ('a, 'b) assoc

val unit_assoc : 'a -> 'b -> ('a * 'b) seq
val map_assoc : ('a -> 'b) -> ('c * 'a) seq -> ('c * 'b) seq
val zero_assoc : 'a seq
val plus_assoc : 'a seq -> 'a seq -> 'a seq

val insert_assoc : 'a -> 'b -> ('b * 'a) seq -> ('b * 'a) seq
val insert_list_assoc : 'a list -> 'a seq -> 'a seq
val union_assoc : 'a seq -> 'a seq -> 'a seq
val delete_assoc : ('a * 'b) seq -> 'a -> ('a * 'b) seq
val delete_q_assoc : ('a * 'b) seq -> 'a -> ('a * 'b) seq
val delete_all_assoc : ('a * 'b) seq -> 'a -> ('a * 'b) seq

val is_empty_assoc : 'a seq -> bool
val size_assoc : 'a seq -> int
val member_assoc : ('a * 'b) seq -> 'a -> bool
val member_q_assoc : ('a * 'b) seq -> 'a -> bool
val count_assoc : ('a * 'b) seq -> 'a -> int
val count_q_assoc : ('a * 'b) seq -> 'a -> int
val lookup_assoc : ('a * 'b) seq -> 'a -> 'b
val lookup_q_assoc : ('a * 'b) seq -> 'a -> 'b
val lookup_all_assoc : ('a * 'b) seq -> 'a -> 'b seq
val lookup_all_q_assoc : ('a * 'b) seq -> 'a -> 'b seq

val elements_assoc : ('a * 'b) seq -> 'b seq
val adjust_assoc : ('a -> 'a) -> 'b -> ('b * 'a) seq -> ('b * 'a) seq
val adjust_all_assoc : ('a -> 'a) -> 'b -> ('b * 'a) seq -> ('b * 'a) seq
val fold_assoc : ('a -> 'b -> 'a) -> 'a -> ('c * 'b) seq -> 'a
val filter_keys_assoc : ('a -> bool) -> ('a * 'b) seq -> ('a * 'b) seq
val filter_elements_assoc : ('a -> bool) -> ('b * 'a) seq -> ('b * 'a) seq
val delete_list_assoc : ('a * 'b) seq -> 'a list -> ('a * 'b) seq
val partition_keys_assoc :
  ('a -> bool) -> ('a * 'b) seq -> ('a * 'b) seq * ('a * 'b) seq

val adjust_assoc_side : ('a -> 'a) -> 'b -> ('b * 'a) seq -> ('b * 'a) seq
val adjust_all_assoc_side : 
  ('a -> 'a) -> 'b -> ('b * 'a) seq -> ('b * 'a) seq
