type 'a seq

val unit : 'a -> 'a seq
val map : ('a -> 'b) -> 'a seq -> 'b seq
val plus : 'a seq -> 'a seq -> 'a seq
val join : 'a seq seq -> 'a seq
val bind : 'a seq -> ('a -> 'b seq) -> 'b seq
val make : int -> 'a -> 'a seq
val get : int -> 'a seq -> 'a
val set : int -> 'a -> 'a seq -> 'a seq

val cons : 'a -> 'a seq -> 'a seq
val snoc : 'a seq -> 'a -> 'a seq
val from_list : 'a list -> 'a seq
val init : int -> (int -> 'a) -> 'a seq

val reverse : 'a seq -> 'a seq
val reverse_onto : 'a seq -> 'a seq -> 'a seq
val reverse_append : 'a seq -> 'a seq -> 'a seq

val left_view : 'a seq -> 'a * 'a seq
val left_head : 'a seq -> 'a
val left_tail : 'a seq -> 'a seq
val right_view : 'a seq -> 'a seq * 'a
val right_head : 'a seq -> 'a
val right_tail : 'a seq -> 'a seq

val is_empty : 'a seq -> bool
val size : 'a seq -> int
val to_list : 'a seq -> 'a list

val concat_map : ('a -> 'b seq) -> 'a seq -> 'b seq
val fold_right : ('a -> 'b -> 'b) -> 'b -> 'a seq -> 'b
val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b seq -> 'a
val iter : ('a -> 'b) -> 'a seq -> unit

val split_at : int -> 'a seq -> 'a seq * 'a seq
val sub : int -> int -> 'a seq -> 'a seq

val exists : ('a -> bool) -> 'a seq -> bool
val for_all : ('a -> bool) -> 'a seq -> bool
val filter : ('a -> bool) -> 'a seq -> 'a seq
val partition : ('a -> bool) -> 'a seq -> 'a seq * 'a seq

val take_while : ('a -> bool) -> 'a seq -> 'a seq
val drop_while : ('a -> bool) -> 'a seq -> 'a seq
val split_while : ('a -> bool) -> 'a seq -> 'a seq * 'a seq

val map_with_index : (int -> 'a -> 'b ref) -> 'a seq -> 'b seq
val iter_with_index : (int -> 'a -> 'b) -> 'a seq -> unit
val filter_with_index : (int -> 'a -> bool) -> 'a seq -> 'a seq
val fold_right_with_index : (int -> 'a -> 'b -> 'b) -> 'b -> 'a seq -> 'b
val fold_left_with_index : (int -> 'a -> 'b -> 'a) -> 'a -> 'b seq -> 'a
val fill : 'a seq -> int -> int -> 'a ref -> 'a seq
val blit : 'a seq -> int -> 'a seq -> int -> int -> 'a seq

val unsafe_zip : 'a seq -> 'b seq -> ('a * 'b) seq

val zip : 'a seq -> 'b seq -> ('a * 'b) seq
val unzip : ('a * 'b) seq -> 'a seq * 'b seq
