val moduleName : string

(** Oprations catgorielles **)
(** Categorial operiations **)
(** Operaciones de categora **)

val injection : 'a -> 'b -> ('a * 'b) list
val map : ('a -> 'b) -> 'a list -> 'b list
val plus : ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list
val zero : unit -> 'a list

val to_list : 'a list -> 'a list
val from_list : 'a list -> 'a list

val to_tree : 'a list -> 'a BairePervasives.tree
val from_tree : 'a BairePervasives.tree -> 'a list

(** Oprations binaires **)
(** Binairy operations **)
(** Operaciones binrias **)

val union        : ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list
val intersection : ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list
val difference   : ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list
val symetric_difference : ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list

(** Accs **)
(** Acces **)
(** Accesos **)

val insert : 'a -> 'b -> ('a * 'b) list -> ('a * 'b) list

val delete : 'a -> ('a * 'b) list -> ('a * 'b) list
val delete_min : ('a * 'b) list -> ('a * 'b) list
val delete_max : ('a * 'b) list -> ('a * 'b) list

val filter : ('a -> 'b -> bool) -> ('a * 'b) list -> ('a * 'b) list
val filter_lt : 'a -> ('a * 'b) list -> ('a * 'b) list
val filter_le : 'a -> ('a * 'b) list -> ('a * 'b) list
val filter_gt : 'a -> ('a * 'b) list -> ('a * 'b) list
val filter_ge : 'a -> ('a * 'b) list -> ('a * 'b) list

val range : 'a -> 'a -> ('a * 'b) list -> ('a * 'b) list

(** Vues **)
(** Views **)
(** Vistas **)

val min_view : ('a * 'b) list -> ('a * 'b) * ('a * 'b) list

val partition : (('a * 'b) -> bool) -> 
  ('a * 'b) list -> ('a * 'b) list * ('a * 'b) list

val partition_inf_sup :
  'a -> ('a * 'b) list -> ('a * 'b) list * ('a * 'b) option * ('a * 'b) list

(** Observateurs **)
(** Observers **)
(** Observadores **)

val is_empty : ('a * 'b) list -> bool
val size : ('a * 'b) list -> int

val keys   : ('a * 'b) list -> 'a list
val images : ('a * 'b) list -> 'b list

val member : 'a -> ('a * 'b) list -> bool

val lookup : 'a -> ('a * 'b) list -> 'b
val look_for : ('a -> 'b -> bool) -> ('a * 'b) list -> 'a * 'b

val choose : ('a * 'b) list -> 'a * 'b
val min_element : ('a * 'b) list -> 'a * 'b
val max_element : ('a * 'b) list -> 'a * 'b

val previous : 'a -> ('a * 'b) list -> 'a * 'b
val next     : 'a -> ('a * 'b) list -> 'a * 'b
val floor    : 'a -> ('a * 'b) list -> 'a * 'b
val ceil     : 'a -> ('a * 'b) list -> 'a * 'b

val rank : 'a -> ('a * 'b) list -> int
val get : ('a * 'b) list -> int -> 'a * 'b

val for_all : ('a -> 'b -> bool) -> ('a * 'b) list -> bool
val exists  : ('a -> 'b -> bool) -> ('a * 'b) list -> bool

(** Traversals **)
(** Parcours **)
(** Recorridos **)

val iter : ('a -> 'b -> unit) -> ('a * 'b) list -> unit

val fold       : ('a -> 'b -> 'c -> 'c) -> ('a * 'b) list -> 'c -> 'c
val fold_left  : ('c -> 'a -> 'b -> 'c) -> 'c -> ('a * 'b) list -> 'c
val fold_right : ('a -> 'b -> 'c -> 'c) -> ('a * 'b) list -> 'c -> 'c

(** Miscellaneous **)
(** Divers **)
(** Micelaneos **)

val insert_with :
  ('b -> 'b -> 'b) -> 'a -> 'b -> ('a * 'b) list -> ('a * 'b) list

val union_with :
  ('b -> 'b -> 'b) -> ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list

val intersection_with :
  ('b -> 'b -> 'b) -> ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list

val map_images : ('b -> 'c) -> ('a * 'b) list -> ('a * 'c) list

val map_images_with_index : 
  ('a -> 'b -> 'c) -> ('a * 'b) list -> ('a * 'c) list

(** Compatibility with the standard Caml library **)
(** Compatibilit avec la librairie standard **)
(** Compatibilidad con la librera estandar **)

val empty : ('a * 'b) list
val mem : 'a -> ('a * 'b) list -> bool
val add : 'a -> 'b -> ('a * 'b) list -> ('a * 'b) list
val singleton : 'a -> 'b -> ('a * 'b) list
val remove : 'a -> ('a * 'b) list -> ('a * 'b) list
val inter : ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list
val diff : ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list
val cardinal : ('a * 'b) list -> int
val elements : ('a * 'b) list -> ('a * 'b) list
val min_elt : ('a * 'b) list -> 'a * 'b             (* [Not_found] *)
val max_elt : ('a * 'b) list -> 'a * 'b             (* [Not_found] *)
val find : 'a -> ('a * 'b) list -> 'b               (* [Not_found] *)
val mapi : ('a -> 'b -> 'c) -> ('a * 'b) list -> ('a * 'c) list
