type ('a, 'b) map

val moduleName : string


(** Oprations catgorielles **)
(** Categorial operiations **)
(** Operaciones de categora **)

val injection : 'a -> 'b -> ('a, 'b) map
val map : ('a * 'b -> 'c * 'd) -> ('a, 'b) map -> ('c, 'd) map
val plus : ('a, 'b) map -> ('a, 'b) map -> ('a, 'b) map
val zero : unit -> ('a, 'b) map

val from_list : ('a * 'b) list -> ('a, 'b) map
val to_list : ('a, 'b) map -> ('a * 'b) list

val from_tree : ('a * 'b) BairePervasives.tree -> ('a, 'b) map
val to_tree : ('a, 'b) map -> ('a * 'b) BairePervasives.tree

(** Oprations binaires **)
(** Binairy operations **)
(** Operaciones binrias **)

val union        : ('a, 'b) map -> ('a, 'b) map -> ('a, 'b) map
val intersection : ('a, 'b) map -> ('a, 'b) map -> ('a, 'b) map
val difference   : ('a, 'b) map -> ('a, 'b) map -> ('a, 'b) map


val subset  : ('a, 'b) map -> ('a, 'b) map -> bool
val equal   : ('a, 'b) map -> ('a, 'b) map -> bool
val compare : ('a, 'b) map -> ('a, 'b) map -> int

(** Accs **)
(** Acces **)
(** Accesos **)

val insert  : 'a -> 'b -> ('a, 'b) map -> ('a, 'b) map
val insert' : 'a -> 'b -> ('a, 'b) map -> ('a, 'b) map
val insert_list : ('a * 'b) list -> ('a, 'b) map -> ('a, 'b) map

val insert_raise  : 'a -> 'b -> ('a, 'b) map -> ('a, 'b) map (* [Found] *)
val insert_raise' : 'a -> 'b -> ('a, 'b) map -> ('a, 'b) map (* [Found] *)

val delete : 'a -> ('a, 'b) map -> ('a, 'b) map
val delete_min : ('a, 'b) map -> ('a, 'b) map
val delete_max : ('a, 'b) map -> ('a, 'b) map
val delete_list : 'a list -> ('a, 'b) map -> ('a, 'b) map

val delete_raise : 'a -> ('a, 'b) map -> ('a, 'b) map (* [Not_found] *)

val filter : ('a -> 'b -> bool) -> ('a, 'b) map -> ('a, 'b) map
val filter_lt : 'a -> ('a, 'b) map -> ('a, 'b) map
val filter_gt : 'a -> ('a, 'b) map -> ('a, 'b) map
val filter_le : 'a -> ('a, 'b) map -> ('a, 'b) map
val filter_ge : 'a -> ('a, 'b) map -> ('a, 'b) map

val range : 'a -> 'a -> ('a, 'b) map -> ('a, 'b) map

(** Vues **)
(** Views **)
(** Vistas **)

val min_view : ('a, 'b) map -> ('a * 'b) * ('a, 'b) map (* [Empty] *)
val max_view : ('a, 'b) map -> ('a, 'b) map * ('a * 'b) (* [Empty] *)

val partition : ('a -> 'b -> bool) -> ('a, 'b) map ->
  ('a, 'b) map * ('a, 'b) map 

val partition_inf_sup : 'a -> ('a, 'b) map ->
  ('a, 'b) map * ('a * 'b) option * ('a, 'b) map

(** Observateurs **)
(** Observers **)
(** Observadores **)

val is_empty : ('a, 'b) map -> bool
val size : ('a, 'b) map -> int

val keys   : ('a, 'b) map -> 'a list
val images : ('a, 'b) map -> 'b list

val member  : 'a -> ('a, 'b) map -> bool
val member' : 'a -> ('a, 'b) map -> bool

val lookup : 'a -> ('a, 'b) map -> 'b (* [Not_found] *)
val look_for : ('a -> 'b -> bool) -> ('a, 'b) map -> 'a * 'b (* [Not_found] *)

val choose      : ('a, 'b) map -> 'a * 'b (* [Empty] *) 
val min_element : ('a, 'b) map -> 'a * 'b (* [Empty] *)
val max_element : ('a, 'b) map -> 'a * 'b (* [Empty] *)

val previous : 'a -> ('a, 'b) map -> ('a * 'b) (* [Not_found] *)
val next     : 'a -> ('a, 'b) map -> ('a * 'b) (* [Not_found] *)
val floor    : 'a -> ('a, 'b) map -> ('a * 'b) (* [Not_found] *)
val ceil     : 'a -> ('a, 'b) map -> ('a * 'b) (* [Not_found] *)

val rank : 'a -> ('a, 'b) map -> int     (* [Not_found] *)
val get : int -> ('a, 'b) map -> 'a * 'b (* [Out_of_bounds] *)

val for_all : ('a -> 'b -> bool) -> ('a, 'b) map -> bool
val exists  : ('a -> 'b -> bool) -> ('a, 'b) map -> bool

(** Traversals **)
(** Parcours **)
(** Recorridos **)

val iter : ('a -> 'b -> unit) -> ('a, 'b) map -> unit

val fold : ('a -> 'b -> 'c -> 'c) -> ('a, 'b) map -> 'c -> 'c
val fold_left  : ('c -> 'a -> 'b -> 'c) -> 'c -> ('a, 'b) map -> 'c 
val fold_right : ('a -> 'b -> 'c -> 'c) -> ('a, 'b) map -> 'c -> 'c

(** Miscellaneous **)
(** Divers **)
(** Micelaneos **)

val insert_with : ('b -> 'b -> 'b) -> 'a -> 'b -> ('a, 'b) map -> ('a, 'b) map

val union_with : 
  ('b -> 'b -> 'b) -> ('a, 'b) map -> ('a, 'b) map -> ('a, 'b) map

val intersection_with : 
  ('b -> 'b -> 'b) -> ('a, 'b) map -> ('a, 'b) map -> ('a, 'b) map

val map_images : ('b -> 'c) -> ('a, 'b) map -> ('a, 'c) map
val map_images_with_index : ('a -> 'b -> 'c) -> ('a, 'b) map -> ('a, 'c) map

(** Compatibility with the standard Caml library **)
(** Compatibilit avec la librairie standard **)
(** Compatibilidad con la librera estandar **)

val empty : ('a, 'b) map
val mem : 'a -> ('a, 'b) map -> bool
val add : 'a -> 'b -> ('a, 'b) map -> ('a, 'b) map
val singleton : 'a -> 'b -> ('a, 'b) map
val remove : 'a -> ('a, 'b) map -> ('a, 'b) map
val inter : ('a, 'b) map -> ('a, 'b) map -> ('a, 'b) map
val diff : ('a, 'b) map -> ('a, 'b) map -> ('a, 'b) map
val cardinal : ('a, 'b) map -> int
val elements : ('a, 'b) map -> ('a * 'b) list
val min_elt : ('a, 'b) map -> 'a * 'b (* [Not_found] *)
val max_elt : ('a, 'b) map -> 'a * 'b (* [Not_found] *)
val find : 'a -> ('a, 'b) map -> 'b (* [Not_found] *)
val mapi : ('a -> 'b -> 'c) -> ('a, 'b) map -> ('a, 'c) map

(** Compatibility problems **)
(** Problmes de compatibilit **)
(** Problemas de compatibilidad **)

val std_map : ('b -> 'c) -> ('a, 'b) map -> ('a, 'c) map

(** Object wrapper **)
(** Capsule objet **)
(** Componente objeto **)

class type ['a, 'b] o_map =
  object ('c)

    method map : ('a * 'b -> 'a * 'b) -> 'c
    method plus : 'c -> 'c
    method to_list : ('a * 'b) list
    method to_map : ('a, 'b) map

    method union : 'c -> 'c
    method intersection : 'c -> 'c
    method difference : 'c -> 'c
    method symetric_difference : 'c -> 'c

    method subset  : 'c -> bool
    method equal   : 'c -> bool
    method compare : 'c -> int

    method insert  : 'a -> 'b -> 'c
    method insert' : 'a -> 'b -> 'c
    method insert_list : ('a * 'b) list -> 'c

    method insert_raise  : 'a -> 'b -> 'c               (* [Found] *)
    method insert_raise' : 'a -> 'b -> 'c               (* [Found] *)

    method delete : 'a -> 'c
    method delete_list : 'a list -> 'c
    method delete_max : 'c
    method delete_min : 'c
    method delete_raise : 'a -> 'c                      (* [Not_found] *)
  
    method filter : ('a -> 'b -> bool) -> 'c
    method filter_ge : 'a -> 'c
    method filter_gt : 'a -> 'c
    method filter_le : 'a -> 'c
    method filter_lt : 'a -> 'c

    method range : 'a -> 'a -> 'c

    method min_view : ('a * 'b) * 'c                    (* [Empty] *)
    method max_view : 'c * ('a * 'b)                    (* [Empty] *)

    method partition : ('a -> 'b -> bool) -> 'c * 'c
    method partition_inf_sup : 'a -> 'c * ('a * 'b) option * 'c

    method is_empty : bool
    method size : int

    method images : 'b list
    method keys : 'a list

    method member  : 'a -> bool
    method member' : 'a -> bool

    method lookup : 'a -> 'b                            (* [Not_found] *)
    method look_for : ('a -> 'b -> bool) -> 'a * 'b     (* [Not_found] *)

    method choose : 'a * 'b                             (* [Empty] *)
    method max_element : 'a * 'b                        (* [Not_found] *)
    method min_element : 'a * 'b                        (* [Not_found] *)

    method next     : 'a -> 'a * 'b                     (* [Not_found] *)
    method previous : 'a -> 'a * 'b                     (* [Not_found] *)
    method ceil     : 'a -> 'a * 'b                     (* [Not_found] *)
    method floor    : 'a -> 'a * 'b                     (* [Not_found] *)

    method get : int -> 'a * 'b                         (* [Out_of_bounds] *)
    method rank : 'a -> int                             (* [Not_found] *)

    method exists : ('a -> 'b -> bool) -> bool
    method for_all : ('a -> 'b -> bool) -> bool

    method iter : ('a -> 'b -> unit) -> unit
    method fold : ('a -> 'b -> 'd -> 'd) -> 'd -> 'd
    method fold_left : ('d -> 'a -> 'b -> 'd) -> 'd -> 'd
    method fold_right : ('a -> 'b -> 'd -> 'd) -> 'd -> 'd

    method insert_with : ('b -> 'b -> 'b) -> 'a -> 'b -> 'c
    method union_with  : ('b -> 'b -> 'b) -> 'c -> 'c
    method intersection_with : ('b -> 'b -> 'b) -> 'c -> 'c

  end

(** Constructors **)
(** Constructeurs **)
(** Constructores **)

val o_injection : 'a -> 'b -> ('a, 'b) o_map
val o_map : ('a * 'b -> 'c * 'd) -> ('a, 'b) o_map -> ('c, 'd) o_map
val o_plus : ('a, 'b) o_map -> ('a, 'b) o_map -> ('a, 'b) o_map
val o_zero : unit -> ('a, 'b) o_map
val o_from_list : ('a * 'b) list -> ('a, 'b) o_map
val o_from_map : ('a, 'b) map -> ('a, 'b) o_map

(** Binary operations **)
(** Oprations binaires **)
(** Operaciones binrias **)

val o_union        : ('a, 'b) o_map -> ('a, 'b) o_map -> ('a, 'b) o_map
val o_intersection : ('a, 'b) o_map -> ('a, 'b) o_map -> ('a, 'b) o_map
val o_difference   : ('a, 'b) o_map -> ('a, 'b) o_map -> ('a, 'b) o_map
val o_symetric_difference : ('a, 'b) o_map -> ('a, 'b) o_map -> ('a, 'b) o_map

val o_subset  : ('a, 'b) o_map -> ('a, 'b) o_map -> bool
val o_equal   : ('a, 'b) o_map -> ('a, 'b) o_map -> bool
val o_compare : ('a, 'b) o_map -> ('a, 'b) o_map -> int

val o_union_with : ('b -> 'b -> 'b) -> 
  ('a, 'b) o_map -> ('a, 'b) o_map -> ('a, 'b) o_map

val o_intersection_with : ('b -> 'b -> 'b) ->
  ('a, 'b) o_map -> ('a, 'b) o_map -> ('a, 'b) o_map

module type Map =
sig
  
  type key
  type 'a t

  val injection : key -> 'a -> 'a t
  val map : (key * 'a -> key * 'b) -> 'a t -> 'b t
  val plus : 'a t -> 'a t -> 'a t
  val zero : unit -> 'a t
  val from_list : (key * 'a) list -> 'a t
  val to_list : 'a t -> (key * 'a) list
    
  val union        : 'a t -> 'a t -> 'a t
  val intersection : 'a t -> 'a t -> 'a t
  val difference   : 'a t -> 'a t -> 'a t

  val subset  : 'a t -> 'a t -> bool
  val equal   : 'a t -> 'a t -> bool
  val compare : 'a t -> 'a t -> int

  val insert  : key -> 'a -> 'a t -> 'a t
  val insert' : key -> 'a -> 'a t -> 'a t
  val insert_list : (key * 'a) list -> 'a t -> 'a t

  val insert_raise  : key -> 'a -> 'a t -> 'a t (* [Found] *)
  val insert_raise' : key -> 'a -> 'a t -> 'a t (* [Found] *)

  val delete : key -> 'a t -> 'a t
  val delete_min : 'a t -> 'a t
  val delete_max : 'a t -> 'a t
  val delete_list : key list -> 'a t -> 'a t

  val delete_raise : key -> 'a t -> 'a t (* [Not_found] *)

  val filter : (key -> 'a -> bool) -> 'a t -> 'a t
  val filter_lt : key -> 'a t -> 'a t
  val filter_gt : key -> 'a t -> 'a t
  val filter_le : key -> 'a t -> 'a t
  val filter_ge : key -> 'a t -> 'a t

  val range : key -> key -> 'a t -> 'a t

  val min_view : 'a t -> (key * 'a) * 'a t (* [Empty] *)
  val max_view : 'a t -> 'a t * (key * 'a) (* [Empty] *)

  val partition : (key -> 'a -> bool) -> 'a t -> 'a t * 'a t 
  val partition_inf_sup : key -> 'a t -> 'a t * (key * 'a) option * 'a t

  val is_empty : 'a t -> bool
  val size : 'a t -> int

  val keys   : 'a t -> key list
  val images : 'a t -> 'a list

  val member  : key -> 'a t -> bool
  val member' : key -> 'a t -> bool

  val lookup : key -> 'a t -> 'a (* [Not_found] *)
  val look_for : (key -> 'a -> bool) -> 'a t -> key * 'a (* [Not_found] *)

  val choose      : 'a t -> key * 'a (* [Empty] *) 
  val min_element : 'a t -> key * 'a (* [Empty] *)
  val max_element : 'a t -> key * 'a (* [Empty] *)

  val previous : key -> 'a t -> (key * 'a) (* [Not_found] *)
  val next     : key -> 'a t -> (key * 'a) (* [Not_found] *)
  val floor    : key -> 'a t -> (key * 'a) (* [Not_found] *)
  val ceil     : key -> 'a t -> (key * 'a) (* [Not_found] *)

  val rank : key -> 'a t -> int      (* [Not_found] *)
  val get  : int -> 'a t -> key * 'a (* [Out_of_bounds] *)

  val for_all : (key -> 'a -> bool) -> 'a t -> bool
  val exists  : (key -> 'a -> bool) -> 'a t -> bool

  val iter : (key -> 'a -> unit) -> 'a t -> unit

  val fold : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
  val fold_left  : ('b -> key -> 'a -> 'b) -> 'b -> 'a t -> 'b 
  val fold_right : (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b

  val insert_with : ('a -> 'a -> 'a) -> key -> 'a -> 'a t -> 'a t
  val union_with  : ('a -> 'a -> 'a) -> 'a t -> 'a t -> 'a t
  val intersection_with : ('a -> 'a -> 'a) -> 'a t -> 'a t -> 'a t

  val empty : 'a t
  val mem : key -> 'a t -> bool
  val add : key -> 'a -> 'a t -> 'a t
  val singleton : key -> 'a -> 'a t
  val remove : key -> 'a t -> 'a t
  val inter : 'a t -> 'a t -> 'a t
  val diff : 'a t -> 'a t -> 'a t
  val cardinal : 'a t -> int
  val elements : 'a t -> (key * 'a) list
  val min_elt : 'a t -> key * 'a (* [Not_found] *)
  val max_elt : 'a t -> key * 'a (* [Not_found] *)
  val find : key -> 'a t -> 'a (* [Not_found] *)
  val mapi : (key -> 'a -> 'b) -> 'a t -> 'b t

  val std_map : ('a -> 'b) -> 'a t -> 'b t

class type ['a] o_t =
  object ('b)

    method map : (key * 'a -> key * 'a) -> 'b
    method plus : 'b -> 'b
    method to_list : (key * 'a) list
    method to_map : 'a t

    method union : 'b -> 'b
    method intersection : 'b -> 'b
    method difference : 'b -> 'b
    method symetric_difference : 'b -> 'b

    method subset  : 'b -> bool
    method equal   : 'b -> bool
    method compare : 'b -> int

    method insert  : key -> 'a -> 'b
    method insert' : key -> 'a -> 'b
    method insert_list : (key * 'a) list -> 'b

    method insert_raise  : key -> 'a -> 'b               (* [Found] *)
    method insert_raise' : key -> 'a -> 'b               (* [Found] *)

    method delete : key -> 'b
    method delete_list : key list -> 'b
    method delete_max : 'b
    method delete_min : 'b
    method delete_raise : key -> 'b                      (* [Not_found] *)
  
    method filter : (key -> 'a -> bool) -> 'b
    method filter_ge : key -> 'b
    method filter_gt : key -> 'b
    method filter_le : key -> 'b
    method filter_lt : key -> 'b

    method range : key -> key -> 'b

    method min_view : (key * 'a) * 'b                    (* [Empty] *)
    method max_view : 'b * (key * 'a)                    (* [Empty] *)

    method partition : (key -> 'a -> bool) -> 'b * 'b
    method partition_inf_sup : key -> 'b * (key * 'a) option * 'b

    method is_empty : bool
    method size : int

    method images : 'a list
    method keys : key list

    method member  : key -> bool
    method member' : key -> bool

    method lookup : key -> 'a                            (* [Not_found] *)
    method look_for : (key -> 'a -> bool) -> key * 'a    (* [Not_found] *)

    method choose : key * 'a                             (* [Empty] *)
    method max_element : key * 'a                        (* [Not_found] *)
    method min_element : key * 'a                        (* [Not_found] *)

    method next     : key -> key * 'a                    (* [Not_found] *)
    method previous : key -> key * 'a                    (* [Not_found] *)
    method ceil     : key -> key * 'a                    (* [Not_found] *)
    method floor    : key -> key * 'a                    (* [Not_found] *)

    method get : int -> key * 'a                         (* [Out_of_bounds] *)
    method rank : key -> int                             (* [Not_found] *)

    method exists : (key -> 'a -> bool) -> bool
    method for_all : (key -> 'a -> bool) -> bool

    method iter : (key -> 'a -> unit) -> unit
    method fold : (key -> 'a -> 'c -> 'c) -> 'c -> 'c
    method fold_left : ('c -> key -> 'a -> 'c) -> 'c -> 'c
    method fold_right : (key -> 'a -> 'c -> 'c) -> 'c -> 'c

    method insert_with : ('a -> 'a -> 'a) -> key -> 'a -> 'b
    method union_with  : ('a -> 'a -> 'a) -> 'b -> 'b
    method intersection_with : ('a -> 'a -> 'a) -> 'b -> 'b

  end

(** Constructors **)
(** Constructeurs **)
(** Constructores **)

val o_injection : key -> 'a -> 'a o_t
val o_map : (key * 'a -> key * 'b) -> 'a o_t -> 'b o_t
val o_plus : 'a o_t -> 'a o_t -> 'a o_t
val o_zero : unit -> 'a o_t
val o_from_list : (key * 'a) list -> 'a o_t
val o_from_map : 'a t -> 'a o_t

(** Binary operations **)
(** Oprations binaires **)
(** Operaciones binrias **)

val o_union        : 'a o_t -> 'a o_t -> 'a o_t
val o_intersection : 'a o_t -> 'a o_t -> 'a o_t
val o_difference   : 'a o_t -> 'a o_t -> 'a o_t
val o_symetric_difference : 'a o_t -> 'a o_t -> 'a o_t

val o_subset  : 'a o_t -> 'a o_t -> bool
val o_equal   : 'a o_t -> 'a o_t -> bool
val o_compare : 'a o_t -> 'a o_t -> int

val o_union_with : ('a -> 'a -> 'a) -> 'a o_t -> 'a o_t -> 'a o_t
val o_intersection_with : ('a -> 'a -> 'a) -> 'a o_t -> 'a o_t -> 'a o_t

end

module type OrderedType = 
sig 
  type t 
  val compare : t -> t -> int 
end

module type RelaxationInformation = sig
  val relaxation_coefficient : int
end

module Make (Ord : OrderedType) : Map with type key = Ord.t

module MakeRelaxed
  (Inf : RelaxationInformation) 
  (Ord : OrderedType) : Map with type key = Ord.t


(** Debug **)

val stat : ('a, 'b) map -> int * int * int * int

