(* Diego Olivier Fernandez Pons
 * Universit Pierre et Marie Curie, Paris VI
 * diego-olivier.fernandez-pons@cicrp.jussieu.fr
 * 
 * Shortest path
 * Recherche de plus courts chemins
 * Busqueda de cminos ms cortos
 * 
 *)

open BairePervasives

let moduleName = "ShortestPath"

(* [reduce] ajoute au tas un couple (i, v) ou met  jour sa priorit si la
 * nouvelle priorit est infrieure  l'existante
 *)

let rec insert_succ = fun d heap list ->
  match list with
    | [] -> heap
    | (_, v, j) :: tail -> 
	insert_succ d (CartesianHeap.reduce j (d + v) heap) tail

let rec find_one = fun heap visited ->
  match CartesianHeap.min_view heap with
    | ((x, _), new_heap) when AvlMap.member x visited -> 
	find_one new_heap visited
    | other -> other

let rec dijkstra_rec = fun heap visited graph ->
  if CartesianHeap.is_empty heap then AvlMap.to_list visited
  else
    let ((x, v), heap) = find_one heap visited in
    let  
      new_heap = insert_succ v heap (graph#exterior_edges x) and
      new_visited = AvlMap.insert x v visited and
      new_graph = graph#delete_node x
    in
      dijkstra_rec new_heap new_visited new_graph
    
(* requiert un graphe valu par des entiers *)

let dijkstra = fun node graph ->
  let 
    cocycle = CartesianHeap.zero () and
    visited = AvlMap.injection node 0
  in
    dijkstra_rec 
      (insert_succ 0 cocycle (graph#exterior_edges node))
      visited
      graph
