type 'a seq

val unit : 'a -> 'a seq
val map : ('a -> 'b) -> 'a seq -> 'b seq
val join : 'a seq seq -> 'a seq
val bind : 'a seq -> ('a -> 'b seq) -> 'b seq
val plus : 'a seq -> 'a seq -> 'a seq
val zero : unit -> 'a seq
val make : int -> 'a -> 'a seq
val get : 'a seq -> int -> 'a
val set : 'a seq -> int -> 'a -> 'a seq

val compare : 'a seq -> 'a seq -> int

val cons : 'a -> 'a seq -> 'a seq
val snoc : 'a seq -> 'a -> 'a seq
val init : int -> (int -> 'a) -> 'a seq
val from_list : 'a list -> 'a seq

val left_view : 'a seq -> 'a * 'a seq
val left_head : 'a seq -> 'a
val left_tail : 'a seq -> 'a seq
val right_view : 'a seq -> 'a seq * 'a
val right_head : 'a seq -> 'a
val right_tail : 'a seq -> 'a seq

val to_list : 'a seq -> 'a list
val is_empty : 'a seq -> bool
val size : 'a seq -> int

val reverse : 'a seq -> 'a seq
val reverse_append : 'a seq -> 'a seq -> 'a seq
val reverse_onto : 'a seq -> 'a seq -> 'a seq

val concat_map : ('a -> 'b seq) -> 'a seq -> 'b seq
val fold_right : ('a -> 'b -> 'b) -> 'b -> 'a seq -> 'b
val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b seq -> 'a
val iter : ('a -> unit) -> 'a seq -> unit

val take : int -> 'a seq -> 'a seq
val drop : int -> 'a seq -> 'a seq
val split_at : int -> 'a seq -> 'a seq * 'a seq
val sub : int -> int -> 'a seq -> 'a seq

val for_all : ('a -> bool) -> 'a seq -> bool
val exists : ('a -> bool) -> 'a seq -> bool
val find : ('a -> bool) -> 'a seq -> 'a
val filter : ('a -> bool) -> 'a seq -> 'a seq
val partition : ('a -> bool) -> 'a seq -> 'a seq * 'a seq

val take_while : ('a -> bool) -> 'a seq -> 'a seq
val drop_while : ('a -> bool) -> 'a seq -> 'a seq
val split_while : ('a -> bool) -> 'a seq -> 'a seq * 'a seq

val map_with_index :  (int -> 'a -> 'b) -> 'a seq -> 'b seq
val iter_with_index : (int -> 'a -> 'b) -> 'a seq -> unit
val find_with_index : (int -> 'a -> bool) -> 'a seq -> 'a
val filter_with_index : (int -> 'a -> bool) -> 'a seq -> 'a seq
val fold_right_with_index : (int -> 'a -> 'b -> 'b) -> 'b -> 'a seq -> 'b
val fold_left_with_index : (int -> 'a -> 'b -> 'a) -> 'a -> 'b seq -> 'a
val fill : 'a seq -> int -> int -> 'a -> 'a seq
val blit : 'a seq -> int -> 'a seq -> int -> int -> 'a seq

val zip : 'a seq -> 'b seq -> ('a * 'b) seq
val unzip : ('a * 'b) seq  -> 'a seq * 'b seq

val empty : unit -> 'a seq
val append : 'a seq -> 'a seq -> 'a seq
