(*
 * Arrays
 * 
 *)

open BairePervasives

type 'a seq = 'a array

(* fonctions C externes *)

external size : 'a array -> int = "%array_length"
external unsafe_get: 'a array -> int -> 'a = "%array_unsafe_get"
external unsafe_set: 'a array -> int -> 'a -> unit = "%array_unsafe_set"
external make: int -> 'a -> 'a array = "make_vect"

(* Utils *)

exception Index of int

let unsafe_copy = function array ->
  let len = size array in
  let result = make len (unsafe_get array 0) in
    for i = 0 to (len - 1) do
      unsafe_set result i (unsafe_get array i)
    done ;
    result


(* Signature monadique *)

let unit = function x -> make 1 x

let map f = function array ->
  let len = size array in
    match len with
      | 0 -> [||]
      | _ -> 
	  let result = make len (f (unsafe_get array 0)) in
	    for i = 1 to len - 1 do
	      unsafe_set result i (f (unsafe_get array i))
	    done ;
	    result

let join = function array ->
  let 
    total = ref 0 and
    len = size array and
    index = ref 0 
  in
    for i = 0 to len - 1 do 
      let temp = size (unsafe_get array i) in
	if (temp <> 0) then index := i ; total := !total + temp
    done ;
    
    if (!total = 0) then [||] 
    else
      let 
	result = make !total (unsafe_get (unsafe_get array 0) !index) 
      and
	compteur = ref 0
      in
	for i = 0 to len - 1 do
	  for j = 0 to size (unsafe_get array i) - 1 do
	    unsafe_set result !compteur 
	      (unsafe_get (unsafe_get array i) j) ;
	    compteur := !compteur + 1
	  done
	done ;
	result

let bind = fun array f -> join (map f array)

let zero = function () -> [||]

let plus = fun arrayA arrayB ->
  let 
    lenA = size arrayA and
    lenB = size arrayB 
  in
    if (lenA = 0 && lenB = 0) then [||]
    else 
      let result = make (lenA + lenB) 
		       (unsafe_get (if lenA > 0 then arrayA else arrayB) 0) in
	for i = 0 to lenA - 1 do 
	  unsafe_set result i (unsafe_get arrayA i) 
	done ;  
	for i = 0 to lenB - 1 do 
	  unsafe_set result (i + lenA) (unsafe_get arrayB i)
	done ;
	result

let set = fun array n x ->
  let len = size array in
    if (n < 0) then invalid_arg "ArraySeq.set : argument ngatif"
    else if (n >= len) then raise (Out_of_bounds (len - n))
    else if (len = 0) then [||]
    else
      let result = make len (unsafe_get array 0) in
	for i = 1 to len - 1 do
	  unsafe_set result i (unsafe_get array i)
	done ;
	unsafe_set result n x ;
	result

let get = fun array n ->
  let len = size array in
    if (n < 0) then invalid_arg "ArraySeq.get : argument ngatif"
    else if (n >= len) then raise (Out_of_bounds (len - n))
    else unsafe_get array n

let make_matrix = fun sx sy init ->
  let result = make sx [||] in
    for x = 0 to pred sx do
      unsafe_set result x (make sy init)
    done ;
    result

(* Constructors *)

let cons x = function array ->
  let len = size array in
  let result = make (len + 1) x in
    for i = 1 to len do 
      unsafe_set result i (unsafe_get array (i - 1)) 
    done ;
    result

let snoc = fun array x ->
  let len = size array in
  let result = make (len + 1) x in
    for i = 0 to (len - 1) do 
      unsafe_set result i (unsafe_get array i) 
    done ;
    result

let from_list = function
  | [] -> [||]
  | head :: tail as list ->
      let result = make (List.size list) head in
      let rec list_rec k = function
	| [] -> result
	| head :: tail -> unsafe_set result k head ; list_rec (k + 1) tail
      in
	list_rec 1 tail

let init = fun n f ->
  match n with
    | 0 -> [||]
    | _ -> let result = make n (f 0) in
	for i = 1 to (n - 1) do unsafe_set result i (f i) done ;
	result
	  
(* Destructors *)

let left_view = function array ->
  let len = size array in
    match len with
      | 0 | 1 -> raise Empty
      | _ ->
	  let tail = make (len - 1) (unsafe_get array 1) in
	    for i = 0 to (len - 2) do
	      unsafe_set tail i (unsafe_get array (i + 1))
	    done ;
	    (unsafe_get array 0, tail)

let left_head = function
  | [||] -> raise Empty
  | array -> unsafe_get array 0

let left_tail = function array ->
  let len = size array in
    match len with
      | 0 | 1 -> raise Empty
      | _ ->
	  let tail = make (len - 1) (unsafe_get array 1) in
	    for i = 0 to (len - 2) do
	      unsafe_set tail i (unsafe_get array (i + 1))
	    done ;
	    tail

let right_view = function array ->
  let len = size array in
    match len with
      | 0 | 1 -> raise Empty
      | _ ->
	  let tail = make (len - 1) (unsafe_get array (len - 1)) in
	    for i = 0 to (len - 2) do
	      unsafe_set tail i (unsafe_get array i)
	    done ;
	    (tail, unsafe_get array (len - 1))

let right_head = function
  | [||] -> raise Empty
  | array ->
      let len = size array in
	unsafe_get array (len - 1)

let right_tail = function array ->
  let len = size array in
    match len with
      | 0 | 1 -> raise Empty
      | _ ->
	  let tail = make (len - 1) (unsafe_get array (len - 1)) in
	    for i = 0 to (len - 2) do
	      unsafe_set tail i (unsafe_get array i)
	    done ;
	    tail

(* Observers *)

let is_empty = function
  | [||] -> true
  | _ -> false

let to_list = function array ->
  let rec to_list_rec result = function
    | k when (k < 0) -> result
    | k ->  to_list_rec (unsafe_get array k :: result) (k - 1)
  in
    to_list_rec [] (size array - 1)

(* Reverse *)
      
let reverse = function array ->
  let len = size array in
  let result = make len (unsafe_get array 0) in
    for i = 0 to len - 1 do
      unsafe_set result i (unsafe_get array (len - i - 1))
    done ;
    result

let reverse_onto = fun array_rev array ->
  let 
    len = size array and
    len_rev = size array_rev
  in
    if (len <> 0) then
      let result = make (len + len_rev) (unsafe_get array 0) in
	if (len_rev <> 0) then
	  for i = 0 to len_rev - 1 do
	    unsafe_set result i (unsafe_get array_rev (len_rev - i - 1))
	  done 
	else () ;
	for i = 0 to len - 1 do
	  unsafe_set result (len_rev + i) (unsafe_get array i)
	done ;
	result
    else reverse array_rev

(* Maps et folds *)

let concat_map = fun f array -> join (map f array)

let fold_left f e = function array ->
  let result = ref e in
    for i = 0 to size array - 1 do
      result := f !result (unsafe_get array i)
    done ;
    !result
      
let fold_right f e = function array ->
  let result = ref e in
    for i = size array - 1 downto 0 do
      result := f (unsafe_get array i) !result 
    done ;
    !result
      
let iter f = function array ->
  for i = 0 to size array - 1 do f (unsafe_get array i) done
  
(* Subsequences *)

let copy = function array ->
  let len = size array in
    if (len = 0) then [||]
    else 
      let result = make len (unsafe_get array 0) in
	for i = 1 to len do
	  unsafe_set result i (unsafe_get array i)
	done ;
	result

let take n = function array ->
  let len = size array in
    if n > len then raise (Out_of_bounds (len - n))
    else
      let result = make n (unsafe_get array 0) in
	for i = 0 to n - 1 do
	  unsafe_set result i (unsafe_get array i)
	done ;
	result

let drop n = function array ->
  let len = size array in
    if n > len then raise (Out_of_bounds (len - n))
    else
      let result = make (len - n) (unsafe_get array 0) in
	for i = 0 to len - n - 1 do
	  unsafe_set result i (unsafe_get array (i + n))
	done ;
	result

let split_at n = function array ->
  let len = size array  - 1 in
  let 
    a = make n (unsafe_get array 0) and
    b = make (len - n + 1) (unsafe_get array 0)
  in
    for i = 0 to (n - 1) do
      unsafe_set a i (unsafe_get array i)
    done ;
    for i = 0 to (len - n) do
      unsafe_set b i (unsafe_get array (i + n))
    done ;
    (a, b)
    
let sub = fun array dbut fin ->
  let result = make (fin - dbut) (unsafe_get array 0) in
    for i = dbut to fin - 1 do
      unsafe_set result (i - dbut) (unsafe_get array i)
    done ;
    result

(* Predicates *)

let for_all f = function array ->
  let rec forall_rec = function 
    | k when (k < 0) -> true
    | k when f (unsafe_get array k) -> forall_rec (k - 1)
    | k -> false
  in
    forall_rec (size array - 1)

let exists f = function array ->
  let rec exists_rec = function
    | k when (k < 0) -> false
    | k when f (unsafe_get array k) -> true
    | k -> exists_rec (k - 1)
  in
    exists_rec (size array - 1)



let find f = function array ->
  let len = size array in
    try
      for i = 0 to len - 1 do
	if f (unsafe_get array i) then raise (Index i)
      done ;
      raise Not_found
    with Index k -> unsafe_get array k

let filter f = function array ->
  let len = size array in
  let temp  = make len (unsafe_get array 0) in
  let rec filter_rec n = function
    | k when k = len -> n
    | k when f (unsafe_get array k) ->
	unsafe_set temp n (unsafe_get array k) ;
	filter_rec (n + 1) (k + 1)
    | k -> filter_rec n (k + 1)
  in
  let taille = filter_rec 0 0 in
  let result = make taille (unsafe_get temp 0) in
    for i = 0 to taille - 1 do
      unsafe_set result i (unsafe_get temp i)
    done ;
    result

let partition f = function array ->
  let len = size array in
    if (len <> 0) then
      let element = unsafe_get array 0 in
      let
	true_ = make len element and
	false_ = make len element in
      let rec part_rec m n = function
	| k when (k = len) -> (m, n)
	| k when f (unsafe_get array k) ->
	    unsafe_set true_ m (unsafe_get array k) ;
	    part_rec (m + 1) n (k + 1)
	| k -> unsafe_set false_ n (unsafe_get array k) ;
	    part_rec m (n + 1) (k + 1)
      in
      let (m, n) = part_rec 0 0 0 in
      let 
	a = make m element and
	b = make n element
      in
	for i = 0 to m - 1 do
	  unsafe_set a i (unsafe_get true_ i)
	done ;
	for i = 0 to n - 1 do
	  unsafe_set b i (unsafe_get false_ i)
	done ;
	(a, b)
    else ([||], [||])

let partition_with_temp_list f = function array ->
  let len = size array in
    if (len <> 0) then
      let rec part_rec k m n = function 
	| result when k = len -> (m, n, result)
	| (true_, false_) when f (unsafe_get array k) ->
	    part_rec (k + 1) (m + 1) n (unsafe_get array k :: true_, false_)
	| (true_, false_) -> 
	    part_rec (k + 1) m (n + 1) (true_, unsafe_get array k :: false_)
      in
      let (m, n, lists) = part_rec 0 0 0 ([], []) in
      let
	true_ = make m (unsafe_get array 0) and
	false_ = make n (unsafe_get array 0)
      in
      let rec fill_rec k = function
	| (headV :: tailV, headF :: tailF) ->
	    unsafe_set true_ (m - 1 - k) headV ;
	    unsafe_set false_ (n - 1 - k) headF ;
	    fill_rec (k + 1) (tailV, tailF)
	| (head :: tail, []) -> unsafe_set true_ (m - 1 - k) head ;
	    fill_rec (k + 1) (tail, [])
	| ([], head :: tail) -> unsafe_set false_ (n - 1 - k) head ;
	    fill_rec (k + 1) ([], tail)
	| ([], []) -> ()
      in
	fill_rec 0 lists; (true_, false_)
    else ([||], [||])

let take_while f = function array ->
  let len = size array in
  let rec take_rec = function
    | k when k = len -> len
    | k when f (unsafe_get array k) -> take_rec (k + 1)
    | k -> k
  in
    sub array 0 (take_rec 0)

let drop_while f = function array ->
  let len = size array in
  let rec drop_rec = function
    | k when k = len -> len
    | k when f (unsafe_get array k) -> drop_rec (k + 1)
    | k -> k
  in
    sub array (drop_rec 0) len

let split_while f = function  array ->
  let len = size array in
  let rec split_rec = function
    | k when k = len -> len
    | k when f (unsafe_get array k) -> split_rec (k + 1)
    | k -> k
  in
    split_at (split_rec 0) array

(* Indexes *)
      
let map_with_index f = function array ->
  let len = size array in
    match len with
      | 0 -> [||]
      | _ -> 
	  let result = make len (f 0 (unsafe_get array 0)) in
	    for i = 1 to (len - 1) do
	      unsafe_set result i (f i (unsafe_get array i))
	    done ;
	    result

let iter_with_index f = function array ->
  for i = 0 to size array - 1 do
    f i (unsafe_get array i)
  done

let filter_with_index f = function array ->
  let len = size array in
  let temp  = make len (unsafe_get array 0) in
  let rec filter_rec n = function
    | k when k = len -> n
    | k when f k (unsafe_get array k) ->
	unsafe_set temp n (unsafe_get array k) ;
	filter_rec (n + 1) (k + 1)
    | k -> filter_rec n (k + 1)
  in
  let taille = filter_rec 0 0 in
  let result = make taille (unsafe_get temp 0) in
    for i = 0 to taille - 1 do
      unsafe_set result i (unsafe_get temp i)
    done ;
    result

let find_with_index f = function array ->
  let len = size array in
    try
      for i = 0 to len - 1 do
	if f i (unsafe_get array i) then raise (Index i)
      done ;
      raise Not_found
    with Index k -> unsafe_get array k


let fold_left_with_index  f e = function array ->
  let result = ref e in
    for i = 0 to size array - 1 do
      result := f i !result (unsafe_get array i)
    done ;
    !result
      
let fold_right_with_index  f e = function array ->
  let result = ref e in
    for i = size array - 1 downto 0 do
      result := f i (unsafe_get array i) !result 
    done ;
    !result


let fill = fun array dbut fin motif ->
  let result = unsafe_copy array in
    for i = dbut to fin do
      unsafe_set result i motif
    done ;
    result

let blit = fun arrayC indexC array index taille ->
  let result = unsafe_copy array in
    for i = 0 to taille do
      unsafe_set result (index + i) (unsafe_get arrayC (indexC + i))
    done ;
    result

(* Zips et unzips *)

let zip = fun arrayA arrayB ->
  match (size arrayA, size arrayB) with
    | (0,_) | (_, 0) -> [||]
    | (a, b) ->
	let len = min a b in
	let result = 
	  make len (unsafe_get arrayA 0, unsafe_get arrayB 0) in
	  for i = 0 to len - 1 do
	    unsafe_set result i 
	      (unsafe_get arrayA i, unsafe_get arrayB i)
	  done ;
	  result

let unzip = function array ->
  let len = size array in
    if len = 0 then ([||], [||]) else
      let (x, y) = unsafe_get array 0 in
      let 
	a = make len x and
	b = make len y 
      in
	for i = 0 to len - 1 do
	  let (x,y) = unsafe_get array i in
	    unsafe_set a i x ;
	    unsafe_set b i y
	done ;
	(a, b)

let set_side = fun array n x ->
  let len = size array in
    if (n < 0) then 
      raise (Invalid_argument "set_side : argument ngatif")
    else if (n >= len) then raise (Out_of_bounds n) 
    else unsafe_set array n x

let reverse_side = function array ->
  let len = size array in
    if len <> 0 then
      let middle = (len / 2) in
      let temp = make (middle + 1) (unsafe_get array 0) in
	for i = 0 to middle do
	  unsafe_set temp i (unsafe_get array i)
	done ;
	for i = 0 to len - middle do
	  unsafe_set array i (unsafe_get array (len - 1 - i))
	done ;
	for i = 0 to middle do
	  unsafe_set array (len - i - 1) (unsafe_get temp i)
	done

let adjust_side f = function array ->
  let len = size array in
    for i = 0 to len - 1 do
      unsafe_set array i (f (unsafe_get array i))
    done

let adjust_with_index_side f = function array ->
  let len = size array in
    for i = 0 to len - 1 do
      unsafe_set array i (f i (unsafe_get array i))
    done

let fill_side = fun array dbut fin motif ->
  for i = dbut to fin do
    unsafe_set array i motif
  done

let blit_side = fun arrayC indexC array index taille ->
  for i = 0 to taille do
    unsafe_set array (index + i) (unsafe_get arrayC (indexC + i))
  done

