(*
 * Global rebuilding queue
 *)

type 'a rotation_state =
  | Idle
  | Reversing of int * 'a list * 'a list * 'a list * 'a list
  | Appending of int * 'a list * 'a list
  | Done of 'a list
      
type 'a seq = int * 'a list * 'a rotation_state * int * 'a list
		  
let exec = function
  | Reversing (ok, x :: f, f', y :: r, r') ->
      Reversing (ok + 1, f, x :: f', r, y :: r')
  | Reversing (ok, [], f', [y], r') -> Appending (ok, f', y :: r')
  | Appending (0, _, r') -> Done r'
  | Appending (ok, x :: f', r') -> Appending (ok - 1, f', x :: r')
  | state -> state
      
let invalidate = function
  | Reversing (ok, f, f', r, r') -> Reversing (ok - 1, f, f', r, r')
  | Appending (0, _, _ :: r') -> Done r'
  | Appending (ok, f', r') -> Appending (ok - 1, f', r')
  | state -> state
      
let exec2 = function (lenf, f, state, lenr, r) ->
  match exec (exec state) with
    | Done newf -> (lenf, newf, Idle, lenr, r)
    | newstate -> lenf, f, newstate, lenr, r
	
let check = function (lenf, f, state, lenr, r) as queue ->
  if lenr <= lenf then exec2 queue
  else let newstate = Reversing (0, f, [], r, []) in
    exec2 (lenf + lenr, f, newstate, 0, [])

let empty = function () -> (0, [], Idle, 0, [])

let is_empty = function
  | (0, _, _, _, _) -> true
  | _ -> false

let snoc = fun (lenf, f, state, lenr, r) x ->
  check (lenf, f, state, lenr + 1, x :: r)

let head = function
  | (lenf, [], state, lenr, r) -> raise Empty
  | (lenf, x :: f, state, lenr, r) -> x
      
let tail = function
  | (lenf, [], state, lenr, r) -> raise Empty
  | (lenf, x :: f, state, lenr, r) ->
      check (lenf - 1, f, invalidate state, lenr, r)
