open BairePervasives

type 'a seq =
  | Vide
  | Noeud of int * 'a * 'a seq * 'a seq

let jump = function
  | Noeud (_, _, _, Noeud (_, _, _, s)) -> s
  | _ -> failwith "erreur MyersStack.jump"

let zero = Vide

let is_empty = function
  | Vide -> true
  | _ -> false

let unit = function x -> Noeud (1, x, Vide, Vide)

let cons x = function
  | Noeud (i, _, _, Noeud (j, _, _, s)) as t when i = j ->
      Noeud (1 + i + j, x, t, s)
  | s -> Noeud (1, x, s, s)

let left_view = function
  | Noeud (_, x, s, _) -> (x, s)
  | _ -> raise Empty

let left_head = function
  | Noeud (_, x, _, _) -> x
  | _ -> raise Empty

let left_tail = function
  | Noeud (_, _, s, _) -> s
  | _ -> raise Empty

let rec right_head = function
  | Noeud (_, x, Vide, Vide) -> x
  | Noeud (_, _, s, Vide) -> right_head s
  | Noeud (_, _, _, (Noeud (_, _, _, _) as s)) -> right_head s
  | _ -> raise Empty

let rec right_tail_rec x = function
  | Vide -> Vide
  | Noeud (_, y, s, _) -> cons x (right_tail_rec y s)

let right_tail = function
  | Noeud (_, x, s, _) -> right_tail_rec x s
  | _ -> raise Empty

let rec size_rec total = function
  | Vide -> total
  | Noeud (j, _, _, s) -> size_rec (j + total) s

let size = function s -> size_rec 0 s

let rec reverse_onto rsultat = function 
  | Vide -> rsultat
  | Noeud (_, x, s, _) -> reverse_onto (cons x rsultat) s

let reverse = function s -> reverse_onto Vide s

let rec map f = function
  | Vide -> Vide
  | Noeud (j, x, s, _) when j = 1 -> 
      let t = map f s in Noeud (j, f x, t, t)
  | Noeud (j, x, s, _) ->
      let t = map f s in Noeud (j, f x, t, jump t)

let rec fold_right f e = function
  | Vide -> e
  | Noeud (_, x, s, _) -> f x (fold_right f e s)

let rec fold_left f e = function
  | Vide -> e
  | Noeud (_, x, s, _) -> fold_left f (f e x) s

let get = fun s n ->
  let rec get_rec i = function
    | Noeud (j, _, _, s) when i >= j -> get_rec (i - j) s 
    | Noeud (_, _, s, _) when i > 0 -> get_rec (i - 1) s
    | Noeud (_, x, _, _) when i = 0 -> x
    | _ -> raise (Out_of_bounds i)
  in
    get_rec n s

let set = fun s n y ->
  let rec set_rec k = function
    | Vide -> raise (Out_of_bounds k)
    | Noeud (j, _, s, t) when k = 0 -> Noeud (j, y, s, t)
    | Noeud (1, x, s, _) ->
	let t = set_rec (k - 1) s in Noeud (1, x, t, t)
    | Noeud (j, x, s, _) ->
	let t = set_rec (k - 1) s in Noeud (j, x, t, jump t)
  in
    set_rec n s

let drop = fun n s ->
  let rec drop_rec k = function
    | s when k <= 0 -> s
    | Noeud (j, _, s, _) when k < j -> drop_rec (k - 1) s
    | Noeud (j, _, _, s) -> drop_rec (k - j) s
    | _ -> raise (Out_of_bounds n)
  in
    drop_rec n s

let rec unzip = function
  | Vide -> (Vide, Vide)
  | Noeud (1, (x, y), s, _) -> let (xs, ys) = unzip s in 
      (Noeud (1, x, xs, xs), Noeud (1, y, ys, ys))
  | Noeud (j, (x, y), s, _) -> let (xs, ys) = unzip s in
      (Noeud (j, x, xs, jump xs), Noeud (j, y, ys, jump ys))


(* pour tests *)

let rec to_list = function
  | Vide -> []
  | s -> let (head, tail) = left_view s in
      head :: to_list tail

let rec from_list_rec rsultat = function
  | [] -> rsultat
  | tte :: queue -> from_list_rec (cons tte rsultat) queue

let from_list = function liste ->
  from_list_rec Vide (List.reverse liste)
