(*
 * Listes circulaires simplement chanes
 *
 *)

open BairePervasives

type 'a cell = { mutable content : 'a ; mutable next : 'a cell }
type 'a seq = 'a cell option ref

    
(** Fonctions utiles **)


let unit_cell = function x -> 
  let rec cell = { content = x ; next = cell } in cell

let first = function cell -> cell.next
let last = function cell -> cell

let rec copy_cells link stop = function
  | cell when (cell == stop) -> link
  | cell -> { content = cell.content ; next = copy_cells link stop cell.next}

let copy = function clist ->
  match !clist with
    | None -> ref None
    | Some old_last ->
	let last = unit_cell old_last.content in
	  last.next <- copy_cells 
	    (* link *) last 
	    (* stop *) old_last 
	    (* begin *) old_last.next ;
	  ref (Some last)


(** Signature monadique **)


let unit = function x -> ref (Some (unit_cell x))

let rec map_cells f link stop = function
  | cell when (cell == stop) -> link
  | cell -> 
      { content = f cell.content ; next = map_cells f link stop cell.next }

let map f = function clist ->
  match !clist with
    | None -> ref None
    | Some old_last ->
	let last = unit_cell (f old_last.content) in
	  last.next <- map_cells f 
	    (* link *) last 
	    (* stop *) old_last 
	    (* begin *) old_last.next ; 
	  ref (Some last)

let rec adjust_cells f stop = function
  | cell when cell == stop -> cell.content <- f cell.content
  | cell ->
      cell.content <- f cell.content ;
      adjust_cells f stop cell.next

let adjust f = function clist ->
  match !clist with 
    | None -> ()
    | Some last -> adjust_cells f last last.next

(* cette fois-ci il faut commencer la construction par le premier lment
 * d'o la rfrence sur le dernier lment qui sert de point d'entre
 * de la liste circulaire
 *)

let map_with_index f = function clist ->
  match !clist with
    | None -> ref None
    | Some old_last ->
	let 
	  first = unit_cell (f 0 old_last.next.content) and
	  last = ref old_last 
	in
	let rec mapi_cells k = function
	  | cell when (cell == old_last) -> 
	      last := { content = f k cell.content ; next = first } ;
	      !last
	  | cell -> 
	      { content = f k cell.content ; 
		next = mapi_cells (k + 1) cell.next }
	in
	  first.next <- mapi_cells 1 old_last.next.next ;
	  ref (Some !last)

let rec adjust_with_index_rec f k stop = function
  | cell when cell == stop -> cell.content <- f k cell.content
  | cell ->
      cell.content <- f k cell.content ;
      adjust_with_index_rec f (k + 1) stop cell.next

let adjust_with_index f = function clist ->
  match !clist with
    | None -> ()
    | Some last -> adjust_with_index_rec f 0 last last.next

let rec normalize_cells pointer stop = function 
  | cell when cell = stop -> ()
  | cell -> cell.content := pointer ;
      normalize_cells pointer stop cell.next

let rec normalize = function cclist ->
  match !cclist with
    | None -> failwith " "
    | Some cell -> normalize_cells !(cell.content) cell cell.next

(* toutes les listes sont modifies et gales aprs aplatissement *)

let rec join_cells stop left = function
  | cell when cell == stop -> 
      (match !(cell.content) with
	 | None -> raise (Unexpected_pattern "SimplyLCL.join_cells")
	 | Some last -> left
      )
  | cell ->
      match !(cell.content) with
	| None -> join_cells stop left cell.next
	| Some last ->
	    left.next <- last.next ;
	    join_cells stop last cell.next

let rec join_side = function cclist ->
  match !cclist with
    | None -> ()
    | Some cell ->
	match !(cell.content) with
	  | None -> join_side (ref (Some cell.next))
	  | Some last ->
	      let first = last.next in
		(join_cells cell last cell.next).next <- first

(* il faut recopier la totalit de chaque liste car les listes pointent vers
 * leur premier lment
 *)

let plus = fun a b ->
  match (!a, !b) with
    | (None, None) -> ref None
    | (Some _, None) -> copy a
    | (None, Some _ ) -> copy b
    | (Some last_a, Some last_b) -> 
	let last = unit_cell last_b.content in
	let rec 
	  copyPlus = function
	    | cell when (cell == last_a) -> 
		{ content = last_a.content ; 
		  next = copy_cells 
		(* link *) last
		(* stop *) last_b
		(* begin *) last_b.next }
	    | cell -> { content = cell.content ; next = copyPlus cell.next }
	in
	  last.next <- copyPlus last_a.next ;
	  ref (Some last)

(* les deux listes sont modifies et gales aprs la concatnation *)

let rec plus_side = fun a b ->
  match (!a, !b) with
    | (_, None) -> ()
    | (None, _) -> a := !b
    | (Some last_a, Some last_b) ->
	let first = last_a.next in
	  last_a.next <- last_b.next ;
	  last_b.next <- first ;
	  a := !b
	    
let zero = fun () -> ref None

let rec make_cells x last = function
  | 0 -> last
  | k -> { content = x ; next = make_cells x last (k - 1) }

let make = fun n x ->
  match n with
    | k when (k < 0) -> invalid_arg "make : argument ngatif"
    | 0 -> ref None
    | _ -> let last = unit_cell x in 
	last.next <- make_cells x last (n - 1) ; 
	ref (Some last)
	  
let rec get_cells k stop = function
  | cell when (cell == stop) -> raise (Out_of_bounds k)
  | cell when (k = 0) -> cell.content
  | cell -> get_cells (k - 1) stop cell.next

(* pour les listes  1 lment, [cell.next == cell.next.next] 
 * provoque un arrt immdiat de [get_cells] avec leve d'exception
 * on introduit donc un cas particulier pour n = 0 o n est l'indice
 * de l'lment demand (et non pas la taille de la liste) :
 * 
 * - si la liste contient 1 ou plus lments, l'lment 0 est correctement 
 * retourn (deuxime filtrage)
 * 
 * - si la liste contient 1 lment mais qu'un indice autre que 0 est
 * demande, get_cells lve une exception (troisime filtrage)
 * 
 * Remarquer que l'on a dcal le point d'entre de get_cells vers le second
 * lment et l'arrt vers le primier (l'autre solution aurait t de tester
 * dans [get_cells] non [cell == stop] mais [cell.next == stop])
 *)

let get = fun n clist ->
  match !clist with
    | None -> raise (Out_of_bounds n)
    | Some cell when (n = 0) -> cell.next.content
    | Some cell -> get_cells (n - 1)
	(* stop *) cell.next
	(* begin *) cell.next.next


(* On cre le dernier lment de la liste sans savoir si c'est lui que l'on
 * va finalement modifier, il faut donc l'carter ensuite si tel est le cas,
 * d'o le passage par une rfrence
 *
 * L'criture dans une liste d'un lment est gre sparment
 *)
	
let set = fun n x clist ->
  match !clist with
    | None -> raise (Out_of_bounds n)
    | Some cell when cell.next == cell ->
	(match n with
	   | 0 -> unit x
	   | _ -> raise (Out_of_bounds (n - 1))
	)
    | Some old_last ->
	let last = unit_cell old_last.content in
	let list_head = ref last in
	let rec set_cells x n link stop = function
	  | cell when cell == stop && n = 0 -> 
	      let new_last = unit_cell x in
		list_head := new_last ;
		new_last
	  | cell when n = 0 ->  
	      { content = x ; next = copy_cells link stop cell.next }
	  | cell when cell == stop -> raise (Out_of_bounds (n - 1))
	  | cell -> { content = cell.content ; 
		      next = set_cells x (n - 1) link stop cell.next }
	in
	  last.next <- set_cells x n
	    (* link *) last 
	    (* stop *) old_last 
	    (* begin *) old_last.next;
	  if !list_head <> last then !list_head.next <- last.next ;
	  ref (Some !list_head)

let rec set_side_rec stop x k = function
  | cell when cell == stop && k <> 0 -> raise (Out_of_bounds (k - 1))
  | cell when k = 0 -> cell.content <- x
  | cell -> set_side_rec stop x (k - 1) cell.next 

let set_side n x = function clist ->
  match !clist with
    | None -> raise (Out_of_bounds n)
    | Some last -> set_side_rec last x n last.next
	    
(* Constructeurs *)
	    
let cons x = function clist ->
  match !clist with
    | None -> unit x
    | Some old_last -> 
	let last = { content = old_last.content  ; next = unit_cell x } in
	  last.next.next <- copy_cells last old_last old_last.next ;
	  ref (Some last)
	    
let insert_left x = function clist ->
  match !clist with
    | None -> clist := Some (unit_cell  x)
    | Some cell -> cell.next <- { content = x ; next = cell.next }
	
let snoc = fun clist x ->
  match !clist with
    | None -> unit x
    | Some old_last ->
	let 
	  before_last = { content = old_last.content ; next = unit_cell x } 
	in
	  before_last.next.next <- 
	  copy_cells before_last old_last old_last.next ;
	  ref (Some before_last.next)

let insert_right = fun clist x ->
  match !clist with
    | None -> clist := Some (unit_cell x)
    | Some cell ->
	let temp = { content = x ; next = cell.next } in
	  cell.next <- temp ;
	  clist := Some temp

let from_list = function
  | [] -> ref None
  | head :: clist ->
      let first = unit_cell head in
      let last = ref first in
      let rec from_list_cells = function
	| [x] -> last := { content = x ; next = first } ; !last
	| head :: tail -> { content = head ; next = from_list_cells tail }
	| [] -> first
      in
	first.next <- from_list_cells clist ;
	ref (Some !last)

let init = fun n f ->
  if (n < 0) then invalid_arg "init : argument ngatif"
  else if (n = 0) then ref None
  else if (n = 1) then unit (f 0)
  else
    let last = unit_cell (f (n - 1)) in
    let rec initREC = function
      | k when (k = n - 2) -> { content = f (n - 2) ; next = last }
      | k -> { content = f k ; next = initREC (k + 1) }
    in
      last.next <- initREC 0 ;
      ref (Some last)

(* Destructeurs *)

let left_view = function clist ->
  match !clist with
    | None -> raise Empty
    | Some cell when (cell.next == cell) -> raise Empty
    | Some cell ->
	let last = unit_cell cell.content in
	  last.next <- copy_cells last cell cell.next.next ;
	  (cell.next.content, ref (Some last))

(* exercice pervers : coder directement la fonction join *)

let rec join = function cclist ->
  match !cclist with
    | None -> ref None
    | _ -> let (head, tail) = left_view cclist in
	plus head (join tail)

let left_head = function clist ->
  match !clist with
    | None -> raise Empty
    | Some cell -> cell.next.content

let left_tail = function clist ->
  match !clist with
    | None -> raise Empty
    | Some cell when (cell.next == cell) -> raise Empty
    | Some cell ->
	let last = unit_cell cell.content in
	  last.next <- copy_cells last cell cell.next.next ;
	  ref (Some last)

let delete_left = function clist ->
  match !clist with
    | None -> raise Empty
    | Some cell when (cell.next == cell) -> raise Empty
    | Some cell -> cell.next <- cell.next.next

let right_view = function clist ->
  match !clist with
    | None -> raise Empty
    | Some cell when (cell.next == cell) -> raise Empty
    | Some cell when (cell.next.next == cell) ->
	(unit cell.next.content, cell.content)
    | Some cell ->
	let 
	  last = ref cell and
	  first = unit_cell cell.next.content in
	let rec copyTail = function
	  | c when (c.next == cell) -> 
	      last := { content = c.content ; next = first } ;
	      !last
	  | c -> { content = c.content ; next = copyTail c.next }
	in
	  first.next <- copyTail cell.next.next ;
	  (ref (Some !last), cell.content)

let right_head = function clist ->
  match !clist with
    | None -> raise Empty
    | Some cell -> cell.content

let right_tail = function clist ->
  match !clist with
    | None -> raise Empty
    | Some cell when (cell.next == cell) -> raise Empty
    | Some cell when (cell.next.next == cell) -> unit cell.next.content
    | Some cell ->
	let 
	  last = ref cell and
	  first = unit_cell cell.next.content in
	let rec copyTail = function
	  | c when (c.next == cell) -> 
	      last := { content = c.content ; next = first } ;
	      !last
	  | c -> { content = c.content ; next = copyTail c.next }
	in
	  first.next <- copyTail cell.next.next ;
	  ref (Some !last)

let rec before_last last = function
  | cell when (cell.next == last) -> cell
  | cell -> before_last last cell.next

let delete_right = function clist ->
  match !clist with
    | None -> raise Empty
    | Some cell when (cell.next == cell) -> raise Empty
    | Some cell when (cell.next.next == cell) ->
	clist := Some (unit_cell cell.next.content)
    | Some cell ->
	let last = before_last cell cell.next in
	  last.next <- cell.next ;
	  clist := Some last

(* Observateurs *)

let is_empty = function clist ->
  match !clist with
    | None -> true
    | _ -> false

let rec size_cells k last = function
  | cell when cell == last -> k
  | cell -> size_cells (k + 1) last cell.next

let size = function clist ->
  match !clist with
    | None -> 0
    | Some cell -> size_cells 1 cell cell.next 

let rec to_list_cells stop = function
  | cell when (cell == stop) -> [cell.content]
  | cell -> cell.content :: to_list_cells stop cell.next
      
let to_list = function clist ->
  match !clist with
    | None -> []
    | Some cell -> to_list_cells cell cell.next

let reverse = function clist ->
  match !clist with
    | None -> ref None
    | Some old_last ->
	let last = unit_cell old_last.next.content in
	let rec reverse_rec next_rev = function
	  | cell when (cell == old_last) ->
	      { content = old_last.content ; next = next_rev }
	  | cell ->
	      reverse_rec { content = cell.content ; next = next_rev} cell.next
	in
	  last.next <- reverse_rec last old_last.next.next ;
	  ref (Some last)
	    
(* Map et folds *)

let rec iter_cells f stop = function
  | cell when cell == stop -> f cell.content
  | cell -> f cell.content ; iter_cells f stop cell.next

let iter f = function clist ->
  match !clist with
    | None -> ()
    | Some cell -> iter_cells f cell cell.next

let rec fold_right_cells f e stop = function
  | cell when cell == stop -> f cell.content e
  | cell -> f cell.content (fold_right_cells f e stop cell.next)

let fold_right f e = function clist ->
  match !clist with
    | None -> e
    | Some cell -> fold_right_cells f e 
	(* stop *) cell
	(* begin *) cell.next 

let rec fold_left_cells f e stop = function
  | cell when cell == stop -> f e cell.content
  | cell -> fold_left_cells f (f e cell.content) stop cell.next

let fold_left f e = function clist ->
  match !clist with
    | None -> e
    | Some cell -> fold_left_cells f e
	(* stop *) cell
	(* begin *) cell.next

let rec forall_cells f stop = function
  | cell when cell == stop -> f cell.content
  | cell when f cell.content -> forall_cells f stop cell.next
  | cell -> false
      
let for_all f = function clist ->
  match !clist with
    | None -> true
    | Some cell -> f cell.content && forall_cells f cell cell.next

let rec exists_cells f stop = function
  | cell when cell == stop -> f cell.content
  | cell when f cell.content -> true
  | cell -> exists_cells f stop cell.next

let exists f = function clist ->
  match !clist with
    | None -> false
    | Some cell -> f cell.content || exists_cells f cell cell.next

let rec find_cells f stop = function
  | cell when cell == stop -> 
      if f cell.content then cell.content else raise Not_found
  | cell when f cell.content -> cell.content
  | cell -> find_cells f stop cell.next

let find f = function clist ->
  match !clist with
    | None -> raise Not_found
    | Some last -> find_cells f last last.next

(*-- ne marche pas encore correctement --*)
(*-- is not working properly yet --*)

let rec filter f = function clist ->
  match !clist with
    | None -> clist
    | Some old_last when f old_last.content ->
	let last_true = ref old_last in
	let rec filter_cells f link stop = function
	  | cell when cell == stop -> link
	  | cell when f cell.content ->
	      last_true := cell ;
	      { content = cell.content ; 
		next = filter_cells f link stop cell.next }
	  | cell -> filter_cells f link stop cell.next 
	in
	let last = unit_cell old_last.content in
	  last.next <- filter_cells f 
	    (* link *) last 
	    (* stop *) old_last
	    (* begin *) old_last.next ;
	  !last_true.next <- last.next ;
	  ref (Some last)
    | Some cell -> filter f (ref (Some cell.next))	
	
