(* Diego Olivier Fernandez Pons
 * Universit Pierre et Marie Curie, Paris VI
 * diego-olivier.fernandez-pons@cicrp.jussieu.fr
 * 
 * Braun trees
 * Arbres de Braun
 * rboles de Braun
 *)

open BairePervasives

type 'a seq =
  | E
  | N of 'a seq * 'a * 'a seq

      
(** Utilitaires **)
(** Utils **)
(** tiles **)


let moduleName = "BraunTrees"

let invalid = fun function_name error_type -> 
  invalid_argument moduleName function_name error_type

let makeTree = fun l v r -> N (l, v, r)

let injection = function x -> makeTree E x E

let rec size_difference k = function
  | E -> 0
  | N _ when k = 0 -> 1
  | N (l, _, r) -> size_difference (k / 2) (if k mod 2 = 0 then l else r)
      
let rec size = function
  | E -> 0
  | N (l, _, r) -> let n = size r in 1 + 2 * n + size_difference n l

let rec cons x = function
  | E -> injection x
  | N (l, v, r) -> makeTree (cons v r) x l

let rec insert_raise = fun n x tree ->
  if n = 0 then injection x
  else match tree with
    | N (l, v, r) when n mod 2 = 0 -> makeTree l v (insert_raise x (n / 2) r)
    | N (l, v, r) -> makeTree (insert_raise x (n / 2) l) v r
    | _ -> invalid "insert_raise" UnexpectedPattern

let snoc = fun x tree -> insert_raise (size tree) x tree

let rec plus_rec n = fun a b ->
  if n = 0 then b
  else match (a, b) with
    | (s, E) | (E, s) -> s
    | (N (l, v, r), N (l', v', r')) when n mod 2 = 0 -> 
	makeTree (plus_rec n l l') v (plus_rec (n - 1) r (cons v' r'))
    | (N (l, v, r), N (l', v', r')) ->
    	 makeTree (plus_rec n l (cons v' r')) v (plus_rec n r l')

let plus = fun a b -> plus_rec (size a) a b

let rec combine = fun treeA treeB ->
  match (treeA, treeB) with
    | (E, _) -> E
    | (N (a, x, b), _) -> N (treeB, x, combine a b)

 let left_view = function
  | E -> raise Empty
  | N (a, x, b) -> (x, combine a b)

let left_head = function
  | E -> raise Empty
  | N (_, x, _) -> x

let left_tail = function
  | E -> raise Empty
  | N (a, x, b) -> combine a b

let rec effacer k = function 
  | _ when (k = 0) -> E
  | N (a, x, b) when k mod 2 = 1 -> N (effacer (k / 2) a, x, b)
  | N (a, x, b) -> N (a, x, effacer (k / 2 - 1) b)
  | E -> raise Empty
      
(*  let right_view = function
    | E -> raise Empty
    | N _ as x -> let m = size x - 1 in (effacer m x, lookup x m)

    let right_head = function
    | E -> raise Empty
    | N _ as x -> lookup x (size x - 1)
*)
let right_tail = function
  | E -> raise Empty
  | N _ as x -> effacer (size x - 1) x

let is_empty = function
  | E -> true
  | _ -> false

