open BairePervasives

type 'a set

val moduleName : string

(** Categorial operations **)
(** Oprations catgorielles **)
(** Operaciones categoriales **)

val injection : 'a -> 'a -> 'a set
val map : ('a * 'a -> 'b * 'b) -> 'a set -> 'b set
val plus : 'a set -> 'a set -> 'a set
val zero : unit -> 'a set

val from_list : ('a * 'a) list -> 'a set
val to_list : 'a set -> ('a * 'a) list

(** Binary operations **)
(** Oprations binaires **)
(** Operaciones binrias **)

val union        : 'a set -> 'a set -> 'a set
val intersection : 'a set -> 'a set -> 'a set
val difference   : 'a set -> 'a set -> 'a set
val symetric_difference : 'a set -> 'a set -> 'a set

val subset : 'a set -> 'a set -> bool
val equal  : 'a set -> 'a set -> bool

(** Acces **)
(** Accs **)
(** Acceso **)

val insert : 'a -> 'a -> 'a set -> 'a set
val insert_list : ('a * 'a) list -> 'a set -> 'a set
val insert_tree : 'a -> 'a tree -> 'a set -> 'a set

val insert_raise : 'a -> 'a -> 'a set -> 'a set (* [Found] *)

val delete : 'a -> 'a -> 'a set -> 'a set
val delete_list : ('a * 'a) list -> 'a set -> 'a set
val delete_tree : 'a -> 'a set -> 'a set

val delete_raise : 'a -> 'a -> 'a set -> 'a set (* [Not_found] *)
val delete_tree_raise : 'a -> 'a set -> 'a set  (* [Not_found] *)

val delete_min : 'a set -> 'a set
val delete_max : 'a set -> 'a set
val delete_min_tree : 'a set -> 'a set
val delete_max_tree : 'a set -> 'a set

val filter_lt : 'a -> 'a -> 'a set -> 'a set
val filter_gt : 'a -> 'a -> 'a set -> 'a set
val filter_le : 'a -> 'a -> 'a set -> 'a set
val filter_ge : 'a -> 'a -> 'a set -> 'a set

val range : 'a -> 'a -> 'a -> 'a -> 'a set -> 'a set

(** View **)
(** Vues **)
(** Vistas **)

val partition_inf_sup : 
  'a -> 'a -> 'a set -> 'a set * ('a * 'a) option * 'a set

val partition_inf_sup_tree : 
  'a -> 'a set -> 'a set * ('a * 'a tree) option * 'a set

(** Observers **)
(** Observateurs **)
(** Observadores **)

val is_empty : 'a set -> bool
val size : 'a set -> int

val member : 'a -> 'a -> 'a set -> bool
val look_for : ('a -> 'a -> bool) -> 'a set -> 'a (* [Not_found] *)

val choose      : 'a set -> 'a * 'a               (* [Empty] *)

val min_element : 'a set -> 'a * 'a               (* [Not_found] *)
val max_element : 'a set -> 'a * 'a               (* [Not_found] *)
val min_tree    : 'a set -> 'a * 'a tree          (* [Not_found] *)
val max_tree    : 'a set -> 'a * 'a tree          (* [Not_found] *)

val next     : 'a -> 'a -> 'a set -> 'a * 'a      (* [Not_found] *)
val previous : 'a -> 'a -> 'a set -> 'a * 'a      (* [Not_found] *)
val ceil     : 'a -> 'a -> 'a set -> 'a * 'a      (* [Not_found] *)
val floor    : 'a -> 'a -> 'a set -> 'a * 'a      (* [Not_found] *)

val next_tree     : 'a -> 'a set -> 'a * 'a tree  (* [Not_found] *)
val previous_tree : 'a -> 'a set -> 'a * 'a tree  (* [Not_found] *)
val ceil_tree     : 'a -> 'a set -> 'a * 'a tree  (* [Not_found] *)
val floor_tree    : 'a -> 'a set -> 'a * 'a tree  (* [Not_found] *)

val rank : 'a -> 'a -> 'a set -> int              (* [Not_found] *)
val rank_tree : 'a -> 'a set -> int

val exists : ('a -> 'a -> bool) -> 'a set -> bool
val for_all : ('a -> 'a -> bool) -> 'a set -> bool

(** Traversals **)
(** Parcours **)
(** Recorridos **)

val iter : ('a -> 'a -> 'b) -> 'a set -> unit

val fold : ('a -> 'a -> 'b -> 'b) -> 'a set -> 'b -> 'b
val fold_left  : ('b -> 'a -> 'a -> 'b) -> 'b -> 'a set -> 'b
val fold_right : ('a -> 'a -> 'b -> 'b) -> 'a set -> 'b -> 'b

(** Compatibility with the standard Caml library **)
(** Compatibilit avec la librairie standard **)
(** Compatibilidad con la librera estandar **)

val empty : 'a set
val mem : 'a -> 'a -> 'a set -> bool
val add : 'a -> 'a -> 'a set -> 'a set
val singleton : 'a -> 'a -> 'a set
val remove : 'a -> 'a -> 'a set -> 'a set
val inter : 'a set -> 'a set -> 'a set
val diff : 'a set -> 'a set -> 'a set
val cardinal : 'a set -> int
val elements : 'a set -> ('a * 'a) list
val min_elt : 'a set -> 'a * 'a (* [Not_found] *)
val max_elt : 'a set -> 'a * 'a (* [Not_found] *)
