open BairePervasives

val moduleName : string

type 'a zipper

(** Categorial operations **)
(** Oprations catgorielles **)
(** Operaciones categoriales **)

val injection : 'a -> 'a tree
val map : ('a -> 'b) -> 'a tree -> 'b tree
val join : 'a tree tree -> 'a tree
val plus : 'a tree -> 'a tree -> 'a tree
val zero : unit -> 'a tree
val from_list : 'a list -> 'a tree
val to_list : 'a tree -> 'a list

val simple_rebuild_avl : 'a tree -> 'a tree
val rebuild_avl : 'a tree -> 'a tree

(** Binary operations **)
(** Oprations binaires **)
(** Operaciones binrias **)

val union        : 'a tree -> 'a tree -> 'a tree
val intersection : 'a tree -> 'a tree -> 'a tree
val difference   : 'a tree -> 'a tree -> 'a tree
val symetric_difference : 'a tree -> 'a tree -> 'a tree

val subset  : 'a tree -> 'a tree -> bool
val equal   : 'a tree -> 'a tree -> bool
val compare : 'a tree -> 'a tree -> int

(** Acces **)
(** Accs **)
(** Acceso **)

val insert  : 'a -> 'a tree -> 'a tree
val insert' : 'a -> 'a tree -> 'a tree
val insert_list : 'a list -> 'a tree -> 'a tree

val insert_raise  : 'a -> 'a tree -> 'a tree (* [Found] *)
val insert_raise' : 'a -> 'a tree -> 'a tree (* [Found] *)

val delete : 'a -> 'a tree -> 'a tree
val delete_list : 'a list -> 'a tree -> 'a tree

val delete_min : 'a tree -> 'a tree
val delete_max : 'a tree -> 'a tree

val delete_raise : 'a -> 'a tree -> 'a tree (* [Not_found] *)

val filter : ('a -> bool) -> 'a tree -> 'a tree
val filter_lt : 'a -> 'a tree -> 'a tree
val filter_gt : 'a -> 'a tree -> 'a tree
val filter_le : 'a -> 'a tree -> 'a tree
val filter_ge : 'a -> 'a tree -> 'a tree

val range : 'a -> 'a -> 'a tree -> 'a tree

(** View **)
(** Vues **)
(** Vistas **)

val min_view : 'a tree -> 'a * 'a tree (* [Empty] *)
val max_view : 'a tree -> 'a tree * 'a (* [Empty] *)

val partition : ('a -> bool) -> 'a tree -> 'a tree * 'a tree
val partition_inf_sup : 'a -> 'a tree -> 'a tree * 'a option * 'a tree

(** Observers **)
(** Observateurs **)
(** Observadores **)

val is_empty : 'a tree -> bool
val size : 'a tree -> int

val member  : 'a -> 'a tree -> bool
val member' : 'a -> 'a tree -> bool
val look_for : ('a -> bool) -> 'a tree -> 'a (* [Not_found] *)

val choose      : 'a tree -> 'a (* [Empty] *)
val min_element : 'a tree -> 'a (* [Not_found] *)
val max_element : 'a tree -> 'a (* [Not_found] *)

val previous : 'a -> 'a tree -> 'a (* [Not_found] *)
val next     : 'a -> 'a tree -> 'a (* [Not_found] *)
val floor    : 'a -> 'a tree -> 'a (* [Not_found] *)
val ceil     : 'a -> 'a tree -> 'a (* [Not_found] *)

val rank : 'a -> 'a tree -> int (* [Not_found] *)
val get : int -> 'a tree -> 'a (* [Out_of_bounds] *)

val exists  : ('a -> bool) -> 'a tree -> bool
val for_all : ('a -> bool) -> 'a tree -> bool

(** Traversals **)
(** Parcours **)
(** Recorridos **)

val iter : ('a -> 'b) -> 'a tree -> unit

val fold : ('a -> 'b -> 'b) -> 'a tree -> 'b -> 'b
val fold_left  : ('b -> 'a -> 'b) -> 'b -> 'a tree -> 'b
val fold_right : ('a -> 'b -> 'b) -> 'a tree -> 'b -> 'b

(** Zipper operations **)
(** Operations sur les zippers **)
(** Operaciones sobre los zippers **)

val make_zipper : 'a tree -> 'a zipper

val move_left  : 'a zipper -> 'a zipper     (* [Bottom] *)
val move_right : 'a zipper -> 'a zipper     (* [Bottom] *)
val move_up    : 'a zipper -> 'a zipper     (* [Top] *)
val move_top   : 'a zipper -> 'a zipper

val move_to : 'a -> 'a zipper -> 'a zipper

val current_tree   : 'a zipper -> 'a tree
val current_value  : 'a zipper -> 'a option
val current_value' : 'a zipper -> 'a        (* [Empty] *)

val zipper_insert : 'a -> 'a zipper -> 'a zipper   (* [Found] *)

(* replacing a sub tree may violate binary search properties *)

val unsafe_replace_tree : 'a tree -> 'a zipper -> 'a zipper

(** Miscellaneous **)
(** Divers **)
(** Micelanios **)

val bind : 'a tree -> ('a -> 'b tree) -> 'b tree
val old_plus : 'a tree -> 'a tree -> 'a tree

(* to be used with care *)
(* oprations  utiliser avec prcaution *)
(* a usar con cautela *)

val unsafe_inc_map : ('a -> 'b) -> 'a tree -> 'b tree
val unsafe_dec_map : ('a -> 'b) -> 'a tree -> 'b tree
val unsafe_set : 'a -> int -> 'a tree -> 'a tree
val unsafe_insert_min : 'a -> 'a tree -> 'a tree
val unsafe_insert_max : 'a -> 'a tree -> 'a tree

(** Compatibility with the standard Caml library **)
(** Compatibilit avec la librairie standard **)
(** Compatibilidad con la librera estandar **)

val empty : 'a tree
val mem : 'a -> 'a tree -> bool
val add : 'a -> 'a tree -> 'a tree
val singleton : 'a -> 'a tree
val remove : 'a -> 'a tree -> 'a tree
val inter : 'a tree -> 'a tree -> 'a tree
val diff : 'a tree -> 'a tree -> 'a tree
val cardinal : 'a tree -> int
val elements : 'a tree -> 'a list
val min_elt : 'a tree -> 'a (* [Not_found] *)
val max_elt : 'a tree -> 'a (* [Not_found] *)

(** Object wrapper **)
(** Capsule objet **)
(** Componente objeto **)

class type ['a] o_set =
object ('b)
  
  method map : ('a -> 'a) -> 'b
  method plus : 'b -> 'b
  method to_list : 'a list
  method to_set : 'a tree

  method union : 'b -> 'b
  method intersection : 'b -> 'b
  method difference : 'b -> 'b
  method symetric_difference : 'b -> 'b

  method subset : 'b -> bool
  method equal : 'b -> bool
  method compare : 'b -> int

  method insert : 'a -> 'b
  method insert' : 'a -> 'b
  method insert_raise  : 'a -> 'b               (* [Found] *)
  method insert_raise' : 'a -> 'b               (* [Found] *)
  method insert_list : 'a list -> 'b

  method delete : 'a -> 'b
  method delete_raise : 'a -> 'b                (* [Not_Found] *)
  method delete_list : 'a list -> 'b
  method delete_max : 'b
  method delete_min : 'b

  method filter : ('a -> bool) -> 'b
  method filter_gt : 'a -> 'b
  method filter_lt : 'a -> 'b
  method filter_ge : 'a -> 'b
  method filter_le : 'a -> 'b

  method range : 'a -> 'a -> 'b

  method min_view : 'a * 'b                     (* [Empty] *)
  method max_view : 'b * 'a                     (* [Empty] *)
  method partition : ('a -> bool) -> 'b * 'b
  method partition_inf_sup : 'a -> 'b * 'a option * 'b

  method is_empty : bool
  method size : int

  method member : 'a -> bool
  method member' : 'a -> bool
  method look_for : ('a -> bool) -> 'a          (* [Not_found] *)

  method choose : 'a                            (* [Empty] *)
  method min_element : 'a                       (* [Not_found] *)
  method max_element : 'a                       (* [Not_found] *)

  method next : 'a -> 'a                        (* [Not_found] *)
  method previous : 'a -> 'a                    (* [Not_found] *)
  method floor : 'a -> 'a                       (* [Not_found] *)
  method ceil : 'a -> 'a                        (* [Not_found] *)

  method get : int -> 'a                        (* [Out_of_bounds] *)
  method rank : 'a -> int                       (* [Not_found] *)

  method for_all : ('a -> bool) -> bool
  method exists : ('a -> bool) -> bool

  method iter : ('a -> unit) -> unit
  method fold : ('a -> 'c -> 'c) -> 'c -> 'c
  method fold_left : ('c -> 'a -> 'c) -> 'c -> 'c
  method fold_right : ('a -> 'c -> 'c) -> 'c -> 'c

end

(** Constructors **)
(** Constructeurs **)
(** Constructores **)

val o_injection : 'a -> 'a o_set
val o_map : ('a -> 'b) -> 'a o_set -> 'b o_set
val o_plus : 'a o_set -> 'a o_set -> 'a o_set
val o_zero : unit -> 'a o_set
val o_from_list : 'a list -> 'a o_set
val o_from_set : 'a tree -> 'a o_set

val o_simple_rebuild_avl : 'a o_set -> 'a o_set
val o_rebuild_avl : 'a o_set -> 'a o_set

(** Binary operations **)
(** Oprations binaires **)
(** Operaciones binrias **)

val o_union        : 'a o_set -> 'a o_set -> 'a o_set
val o_intersection : 'a o_set -> 'a o_set -> 'a o_set
val o_difference   : 'a o_set -> 'a o_set -> 'a o_set
val o_symetric_difference : 'a o_set -> 'a o_set -> 'a o_set

val o_subset  : 'a o_set -> 'a o_set -> bool
val o_equal   : 'a o_set -> 'a o_set -> bool
val o_compare : 'a o_set -> 'a o_set -> int

module type Set = 
sig

  type t
  type elt

  val injection : elt -> t
  val map : (elt -> elt) -> t -> t
  val plus : t -> t -> t
  val zero : unit -> t
  val from_list : elt list -> t
  val to_list : t -> elt list
    
  val union : t -> t -> t
  val intersection : t -> t -> t
  val difference   : t -> t -> t
  val symetric_difference : t -> t -> t
  val subset  : t -> t -> bool
  val equal   : t -> t -> bool
  val compare : t -> t -> int

  val insert  : elt -> t -> t
  val insert' : elt -> t -> t
  val insert_raise  : elt -> t -> t (* [Found] *)
  val insert_raise' : elt -> t -> t (* [Found] *)
  val insert_list : elt list -> t -> t

  val delete_min : t -> t
  val delete_max : t -> t
  val delete : elt -> t -> t
  val delete_raise : elt -> t -> t (* [Not_found] *)
  val delete_list : elt list -> t -> t

  val filter : (elt -> bool) -> t -> t
  val filter_lt : elt -> t -> t
  val filter_gt : elt -> t -> t
  val filter_le : elt -> t -> t
  val filter_ge : elt -> t -> t

  val range : elt -> elt -> t -> t

  val min_view : t -> elt * t
  val max_view : t -> t * elt
  val partition : (elt -> bool) -> t -> t * t
  val partition_inf_sup : elt -> t -> t * elt option * t

  val is_empty : t -> bool
  val size : t -> int

  val member  : elt -> t -> bool
  val member' : elt -> t -> bool
  val look_for : (elt -> bool) -> t -> elt

  val choose : t -> elt (* [Empty] *)
  val min_element : t -> elt (* [Not_found] *)
  val max_element : t -> elt (* [Not_found] *)

  val next     : elt -> t -> elt (* [Not_found] *) 
  val previous : elt -> t -> elt (* [Not_found] *) 
  val ceil     : elt -> t -> elt (* [Not_found] *) 
  val floor    : elt -> t -> elt (* [Not_found] *) 

  val rank : elt -> t -> int  (* [Not_found] *) 
  val get  : int -> t -> elt  (* [Out_of_bounds] *)

  val for_all : (elt -> bool) -> t -> bool
  val exists  : (elt -> bool) -> t -> bool

  val iter : (elt -> 'a) -> t -> unit
  val fold_left  : ('a -> elt -> 'a) -> 'a -> t -> 'a
  val fold_right : (elt -> 'a -> 'a) -> t -> 'a -> 'a
  val fold : (elt -> 'a -> 'a) -> t -> 'a -> 'a

  val unsafe_inc_map : (elt -> elt) -> t -> t
  val unsafe_dec_map : (elt -> elt) -> t -> t
  val unsafe_set : elt -> int -> t -> t
  val unsafe_insert_min : elt -> t -> t
  val unsafe_insert_max : elt -> t -> t

  val old_plus : t -> t -> t

  val empty : t
  val mem : elt -> t -> bool
  val add : elt -> t -> t
  val singleton : elt -> t
  val remove : elt -> t -> t
  val inter : t -> t -> t
  val diff : t -> t -> t
  val cardinal : t -> int
  val elements : t -> elt list
  val min_elt : t -> elt (* [Not_found] *)
  val max_elt : t -> elt (* [Not_found] *)

  val stat : t -> int * int * int * int

  class type o_t =
  object ('a)

    method map : (elt -> elt) -> 'a
    method plus : 'a -> 'a
    method to_list : elt list
    method to_set : t
      
    method union        : 'a -> 'a
    method intersection : 'a -> 'a
    method difference   : 'a -> 'a
    method symetric_difference : 'a -> 'a
      
    method subset  : 'a -> bool
    method equal   : 'a -> bool
    method compare : 'a -> int
      
    method insert  : elt -> 'a
    method insert' : elt -> 'a
    method insert_raise  : elt -> 'a               (* [Found] *)
    method insert_raise' : elt -> 'a               (* [Found] *)
    method insert_list : elt list -> 'a
      
    method delete : elt -> 'a
    method delete_raise : elt -> 'a                (* [Not_Found] *)
    method delete_list : elt list -> 'a
    method delete_max : 'a
    method delete_min : 'a
      
    method filter : (elt -> bool) -> 'a
    method filter_gt : elt -> 'a
    method filter_lt : elt -> 'a
    method filter_ge : elt -> 'a
    method filter_le : elt -> 'a
      
    method range : elt -> elt -> 'a
      
    method min_view : elt * 'a                     (* [Empty] *)
    method max_view : 'a * elt                     (* [Empty] *)
    method partition : (elt -> bool) -> 'a * 'a
    method partition_inf_sup : elt -> 'a * elt option * 'a
      
    method is_empty : bool
    method size : int
      
    method member  : elt -> bool
    method member' : elt -> bool
    method look_for : (elt -> bool) -> elt         (* [Not_found] *)
      
    method choose : elt                            (* [Empty] *)
    method min_element : elt                       (* [Not_found] *)
    method max_element : elt                       (* [Not_found] *)
      
    method next     : elt -> elt                   (* [Not_found] *)
    method previous : elt -> elt                   (* [Not_found] *)
    method floor    : elt -> elt                   (* [Not_found] *)
    method ceil     : elt -> elt                   (* [Not_found] *)
      
    method get  : int -> elt                       (* [Out_of_bounds] *)
    method rank : elt -> int                       (* [Not_found] *)
      
    method for_all : (elt -> bool) -> bool
    method exists  : (elt -> bool) -> bool
      
    method iter : (elt -> unit) -> unit
    method fold : (elt -> 'b -> 'b) -> 'b -> 'b
    method fold_left  : ('b -> elt -> 'b) -> 'b -> 'b
    method fold_right : (elt -> 'b -> 'b) -> 'b -> 'b

  end

  val o_injection : elt -> o_t
  val o_map : (elt -> elt) -> o_t -> o_t
  val o_plus : o_t -> o_t -> o_t
  val o_zero : unit -> o_t
  val o_from_list : elt list -> o_t
  val o_from_set  : t -> o_t
    
  val o_union : o_t -> o_t -> o_t
  val o_intersection : o_t -> o_t -> o_t
  val o_difference   : o_t -> o_t -> o_t
  val o_symetric_difference : o_t -> o_t -> o_t

  val o_subset  : o_t -> o_t -> bool
  val o_equal   : o_t -> o_t -> bool
  val o_compare : o_t -> o_t -> int

end

(** Debug **)

val stat : 'a tree -> int * int * int * int
