type 'a stream =
  | Nil
  | Cons of 'a * 'a stream
  | Delayed of (unit -> 'a stream)

val lazy_ : 'a stream -> 'a stream
val force : 'a stream -> 'a stream

val unit : 'a -> 'a stream
val map : ('a -> 'b) -> 'a stream -> 'b stream
val plus : 'a stream -> 'a stream -> 'a stream
val join : 'a stream stream -> 'a stream
val bind : 'a stream -> ('a -> 'b stream) -> 'b stream
val make : int -> 'a -> 'a stream
val make_lazy : int -> 'a -> 'a stream
val get : 'a stream -> int -> 'a
val set : 'a stream -> int -> 'a -> 'a stream

val reverse_onto : 'a stream -> 'a stream -> 'a stream
val reverse : 'a stream -> 'a stream

val cons : 'a -> 'a stream -> 'a stream
val from_list : 'a list -> 'a stream
val from_list_lazy : 'a list -> 'a stream

val init : int -> (int -> 'a) -> 'a stream

val left_view : 'a stream -> 'a * 'a stream
val left_head : 'a stream -> 'a
val left_tail : 'a stream -> 'a stream

val is_empty : 'a stream -> bool
val to_list : 'a stream -> 'a list

val iter : ('a -> 'b) -> 'a stream -> unit
val fold_right : ('a -> 'b -> 'b) -> 'b -> 'a stream -> 'b
val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b stream -> 'a

val take : int -> 'a stream -> 'a stream
val drop : int -> 'a stream -> 'a stream

val exists : ('a -> bool) -> 'a stream -> bool
val for_all : ('a -> bool) -> 'a stream -> bool
val find : ('a -> bool) -> 'a stream -> 'a
val filter : ('a -> bool) -> 'a stream -> 'a stream
